/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.annotation.objects;

import javax.vecmath.Color3f;

import org.srs3d.viewer.bioatlas.objects.Feature;
import org.srs3d.viewer.j3d.Colorable;
import org.srs3d.viewer.objects.AbstractObject;

/**
 * Little wrapper around a feature. A feature can have more than one FeatureUnit. We
 * introduced this objects to be able to draw more than one instance of a Feature.
 *
 * @author Karsten Klein
 *
 * @created October 12, 2001
 */
public class FeatureUnit extends AbstractObject implements Referenceable,
    Colorable {
    private Feature feature = null;

    /**
     * <code>FeatureUnit</code> constructor.
     *
     * @param feature Description of parameter.
     */
    public FeatureUnit(Feature feature) {
        this.feature = feature;
    }

    /**
     * Gets the <code>reference</code> attribute of the <code>FeatureUnit</code> object.
     *
     * @return The <code>reference</code> value.
     */
    public AbstractObject getReference() {
        return feature;
    }

    /**
     * Gets the <code>feature</code> attribute of the <code>FeatureUnit</code> object.
     *
     * @return The <code>feature</code> value.
     */
    public Feature getFeature() {
        return feature;
    }

    /**
     * Method description.
     */
    public void cleanup() {
        feature = null;
    }

    /**
     * Method description.
     *
     * @param color Parameter description.
     */
    public void setColor(Color3f color) {
        feature.setColor(color);
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public Color3f getColor() {
        return feature.getColor();
    }
}
