/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.annotation.objects;

import org.srs3d.viewer.bioatlas.objects.Residue;
import org.srs3d.viewer.objects.AbstractObject;

/**
 * Gab objects; linked to segments
 *
 * @author Christian Zofka
 *
 * @created July 10, 2001
 */
public class Gap extends AbstractObject {
    private int length;
    private Residue previous = null;
    private Residue next = null;

    /**
     * <code>Gap</code> constructor.
     *
     * @param length Description of parameter.
     */
    public Gap(int length) {
        this.length = length;
    }

    /**
     * Sets the <code>previous</code> attribute of the <code>Gap</code> object.
     *
     * @param residue The new <code>previous</code> value.
     */
    public void setPrevious(Residue residue) {
        previous = residue;
    }

    /**
     * Sets the <code>next</code> attribute of the <code>Gap</code> object.
     *
     * @param residue The new <code>next</code> value.
     */
    public void setNext(Residue residue) {
        next = residue;
    }

    /**
     * Sets the <code>length</code> attribute of the <code>Gap</code> object.
     *
     * @param i The new <code>length</code> value.
     */
    public void setLength(int i) {
        length = i;
    }

    /**
     * Gets the <code>length</code> attribute of the <code>Gap</code> object.
     *
     * @return The <code>length</code> value.
     */
    public int getLength() {
        return length;
    }

    /**
     * Gets the <code>previous</code> attribute of the <code>Gap</code> object.
     *
     * @return The <code>previous</code> value.
     */
    public Residue getPrevious() {
        return previous;
    }

    /**
     * Gets the <code>next</code> attribute of the <code>Gap</code> object.
     *
     * @return The <code>next</code> value.
     */
    public Residue getNext() {
        return next;
    }

    /**
     * standard toString implementation
     *
     * @return string representation
     */
    public String toString() {
        return new String("[" + getPrevious() + "-gap(" + getLength() + ")-" +
            getNext() + "]");
    }

    /**
     * frees all resources
     */
    public void cleanup() {
        super.cleanup();
        setPrevious(null);
        setNext(null);
    }
}
