/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.annotation.visitors;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

import org.srs3d.viewer.annotation.objects.Alignment;
import org.srs3d.viewer.annotation.objects.ChainAnnotation;
import org.srs3d.viewer.annotation.objects.Ruler;
import org.srs3d.viewer.bioatlas.Parameter;

/**
 * Description of the class.
 *
 * @author Christian Zofka
 *
 * @created October 17, 2001
 */
public class AlignmentLayoutCreator extends AbstractAlignmentLayoutCreator {

    /**
     * <code>AlignmentLayoutCreator</code> constructor.
     *
     * @param alignment Description of parameter.
     * @param position Description of parameter.
     */
    public AlignmentLayoutCreator(Alignment alignment, float position) {
        super(alignment, position);
    }

    /**
     * totally old, don't use before updating (i.e. setting startChainPosition)
     */
    protected void createDefault() {
        int size = alignment.getSize() + 3;
        int i = 0;
        Iterator iterator;
        if (alignment.getAnnotationUnits().size() > 0) {

            // additional void-line
            size++;
        }
        rows = new Object[size];
        rowYPos = new float[size];
        ChainAnnotation chainAnnotation;
        boolean first = true;
        Map rulersMap = alignment.getRulersMap();
        Collection rulers;
        iterator = alignment.getChainAnnotations().iterator();
        while (iterator.hasNext()) {
            chainAnnotation = (ChainAnnotation) iterator.next();
            if (chainAnnotation != null && first) {

                //        rulers = ( Collection ) rulersMap.get( alignment.getFirst() );
                rulers = (Collection) rulersMap.get(chainAnnotation);

                //        insertRulers( rulers, Ruler.VERTICALS_DOWN );
                first = false;
            }

            // chainAnnotations
            insert(chainAnnotation, Parameter.chainAnnotationHeight);
        }

        // Ruler
        rulers = (Collection) rulersMap.get(alignment.getLast());
        if (alignment.getFirst().equals(alignment.getLast())) {

            //      insertRulers( rulers, Ruler.VERTICALS_BOTH );
        } else {

            //      insertRulers( rulers, Ruler.VERTICALS_UP );
        }
        if (i < size - 1) {
            insert(null, Parameter.voidHeight);
            iterator = alignment.getAnnotationUnits().iterator();
            while (iterator.hasNext()) {

                // annotationUnits
                insert(iterator.next(), Parameter.annotationUnitHeight);
            }
        }
    }

    /**
     * creates bar-scheme: 1) all annotations 2) all chains
     */
    protected void createChainsAtBottom() {

        // all objects (annotationUnits + chainAnnotations + 2 rulers)
        int size = alignment.getSize() + 2;
        int chainCount = alignment.getLayers().size();
        int i = 0;
        Iterator iterator;
        int unitCount = alignment.getAnnotationUnits().size();
        if (unitCount > 0) {

            // 2 additional void lines
            size += 2;
            rows = new Object[size];
            rowYPos = new float[size];

            // void
            insert(null, Parameter.voidHeight);
            iterator = alignment.getAnnotationUnits().iterator();
            while (iterator.hasNext()) {

                // annotationUnits
                insert(iterator.next(), Parameter.annotationUnitHeight);
            }

            // void
            insert(null, Parameter.voidHeight);
        } else {
            rows = new Object[size];
            rowYPos = new float[size];
        }
        Object[] chainAnnotations = alignment.getChainAnnotations().toArray();

        // 1 or more chainAnnotations
        // rulers
        insertRulers(alignment.getFirst(), Ruler.VERTICALS_DOWN);
        getAlignmentLayout().setStartChainPosition(getCurrentPosition());
        for (int j = 0; j < alignment.getLayers().size(); j++) {

            // chainAnnotations
            insert(getChainAnnotationAt(j), Parameter.chainAnnotationHeight);
        }
        getAlignmentLayout().setEndChainPosition(getCurrentPosition());

        // rulers
        insertRulers(alignment.getLast(), Ruler.VERTICALS_UP);
    }
}
