/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.bioatlas.colorschemes;

import java.util.Map;
import java.util.Vector;

import javax.media.j3d.Appearance;
import javax.vecmath.Color3f;

import org.srs3d.viewer.bioatlas.factories.AtomTemplateFactory;
import org.srs3d.viewer.bioatlas.objects.Atom;
import org.srs3d.viewer.bioatlas.objects.templates.AtomTemplate;
import org.srs3d.viewer.j3d.AbstractColorScheme;
import org.srs3d.viewer.j3d.AppearanceHelper;
import org.srs3d.viewer.j3d.ContextData;
import org.srs3d.viewer.objects.AbstractObject;

/**
 * The <code>CPKColorScheme</code> applies only for atom and bond representations. The
 * CPK color scheme is a standard method for coloring atoms by element.
 *
 * @author Karsten Klein
 *
 * @created March 20, 2001
 */
public class CPKColorScheme extends AbstractColorScheme {

    /**
     * Constructor description.
     *
     * @param contextData Parameter description.
     */
    public CPKColorScheme(ContextData contextData) {
        super(contextData);
    }

    /**
     * Description of the method
     *
     * @param object Description of parameter
     * @param appearance Description of parameter
     */
    public boolean modify(AbstractObject object, Appearance appearance) {
        Color3f color = null;
        if (object instanceof Atom) {
            color = new Color3f(((Atom) object).getTemplate().getColor());
            color.scale(getColorScale());
            AppearanceHelper.enableVertexColors(appearance, false);
            AppearanceHelper.modifyAppearance(appearance, color);
            return true;
        }

        // for atom geometry on higher hierarchy levels
        AppearanceHelper.enableVertexColors(appearance, true);
        return true;
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public boolean isVertexBased() {
        return true;
    }

    /**
     * Method description.
     *
     * @param map Parameter description.
     *
     * @return Return description.
     */
    public Map getInformation(Map map) {
        map = super.getInformation(map);
        Vector vector = new Vector();
        AtomTemplate template;
        Color3f color;
        template = AtomTemplateFactory.getTemplate(" C  ");
        color = template.getColor();
        color.scale(getLegendColorScale());
        map.put(template.getElement() + " - " + template.getElementName(), color);
        vector.add(template.getElement() + " - " + template.getElementName());
        template = AtomTemplateFactory.getTemplate(" H  ");
        color = template.getColor();
        color.scale(getLegendColorScale());
        map.put(template.getElement() + " - " + template.getElementName(), color);
        vector.add(template.getElement() + " - " + template.getElementName());
        template = AtomTemplateFactory.getTemplate(" O  ");
        color = template.getColor();
        color.scale(getLegendColorScale());
        map.put(template.getElement() + " - " + template.getElementName(), color);
        vector.add(template.getElement() + " - " + template.getElementName());
        template = AtomTemplateFactory.getTemplate(" N  ");
        color = template.getColor();
        color.scale(getLegendColorScale());
        map.put(template.getElement() + " - " + template.getElementName(), color);
        vector.add(template.getElement() + " - " + template.getElementName());
        template = AtomTemplateFactory.getTemplate(" P  ");
        color = template.getColor();
        color.scale(getLegendColorScale());
        map.put(template.getElement() + " - " + template.getElementName(), color);
        vector.add(template.getElement() + " - " + template.getElementName());
        template = AtomTemplateFactory.getTemplate(" S  ");
        color = template.getColor();
        color.scale(getLegendColorScale());
        map.put(template.getElement() + " - " + template.getElementName(), color);
        vector.add(template.getElement() + " - " + template.getElementName());
        map.put("ORDER", vector);
        map.put("NAME", "Element");
        return map;
    }
}
