/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.bioatlas.colorschemes;

import javax.media.j3d.Appearance;
import javax.vecmath.Color3f;

import org.srs3d.viewer.j3d.AppearanceHelper;
import org.srs3d.viewer.j3d.ContextData;
import org.srs3d.viewer.objects.AbstractObject;

/**
 * The <code>MonochromeColorScheme</code> applies only for atom and bond representations.
 * The CPK color scheme is a standard method for coloring atoms by element.
 *
 * @author Karsten Klein
 *
 * @created March 20, 2001
 */
public class FeatureColorScheme extends MonochromeColorScheme {
    public boolean isMixing = false;

    /**
     * Constructor description.
     *
     * @param contextData Parameter description.
     * @param color Parameter description.
     * @param isMixing Parameter description.
     */
    public FeatureColorScheme(ContextData contextData, Color3f color,
        boolean isMixing) {
        super(contextData, color);
        this.isMixing = isMixing;
    }

    /**
     * Description of the method
     *
     * @param object Description of parameter
     * @param appearance Description of parameter
     */
    public boolean modify(AbstractObject object, Appearance appearance) {
        if (isMixing) {
            Color3f color = new Color3f();
            if (appearance.getColoringAttributes() != null) {
                appearance.getColoringAttributes().getColor(color);
            }
            color.add(getColor());
            AppearanceHelper.enableVertexColors(appearance, false);
            AppearanceHelper.modifyAppearance(appearance, color);
            return true;
        } else {
            return super.modify(object, appearance);
        }
    }
}
