/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.bioatlas.colorschemes;

import javax.media.j3d.Appearance;

import org.srs3d.viewer.j3d.AbstractColorScheme;
import org.srs3d.viewer.j3d.AppearanceHelper;
import org.srs3d.viewer.j3d.ContextData;
import org.srs3d.viewer.objects.AbstractObject;

/**
 * The <code>MeshColorScheme</code> is used for testing the geometry creation algorithms.
 * It changes the drawing attributes to display the geometry with lines. Note that this
 * color scheme is really meant for testing and should not be shipped with as part of
 * the final product.
 *
 * @author Karsten Klein
 *
 * @created March 20, 2001
 */
public class MeshColorScheme extends AbstractColorScheme {

    /**
     * Constructor description.
     *
     * @param contextData Parameter description.
     */
    public MeshColorScheme(ContextData contextData) {
        super(contextData);
        setInformation(false);
    }

    /**
     * Gets the <code>Volatile</code> attribute of the <code>MeshColorScheme
     * </code>object
     *
     * @return The <code>Volatile</code> value
     */
    public boolean isVolatile() {
        return true;
    }

    /**
     * Processes a single object modifying the specified <code>Appearance</code> .
     *
     * @param object the object to apply the color scheme to.
     * @param appearance appearance to be modified
     */
    public boolean modify(AbstractObject object, Appearance appearance) {
        AppearanceHelper.enableMesh(appearance, true);
        return true;
    }
}
