/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.bioatlas.colorschemes;

import java.util.Map;

import javax.media.j3d.Appearance;
import javax.vecmath.Color3f;

import org.srs3d.viewer.j3d.AbstractColorScheme;
import org.srs3d.viewer.j3d.AppearanceHelper;
import org.srs3d.viewer.j3d.Colorable;
import org.srs3d.viewer.j3d.ContextData;
import org.srs3d.viewer.objects.AbstractObject;
import org.srs3d.viewer.util.ExceptionHandler;

/**
 * The <code>CPKColorScheme</code> applies only for atom and bond representations. The
 * CPK color scheme is a standard method for coloring atoms by element.
 *
 * @author Karsten Klein
 *
 * @created March 20, 2001
 */
public class SimpleColorScheme extends AbstractColorScheme {
    private Color3f defaultColor = new Color3f(1, 0, 1);

    /**
     * Constructor description.
     *
     * @param contextData Parameter description.
     */
    public SimpleColorScheme(ContextData contextData) {
        super(contextData);
        super.setComplete(true);
    }

    /**
     * Description of the method
     *
     * @param object Description of parameter
     * @param appearance Description of parameter
     */
    public boolean modify(AbstractObject object, Appearance appearance) {
        Color3f color = defaultColor;
        try {
            Colorable colorable = (Colorable) object;
            color = colorable.getColor();
        } catch (ClassCastException e) {

            // catch silently
            ExceptionHandler.handleException(e,
                ExceptionHandler.SILENT_IN_DEBUG, this);
        }
        AppearanceHelper.enableVertexColors(appearance, false);
        AppearanceHelper.modifyAppearance(appearance, color);
        return true;
    }

    /**
     * Method description.
     *
     * @param map Parameter description.
     *
     * @return Return description.
     */
    public Map getInformation(Map map) {
        map = super.getInformation(map);
        map.put("NAME", "Simple");
        return map;
    }
}
