/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.bioatlas.colorschemes;

import javax.media.j3d.Appearance;

import org.srs3d.viewer.j3d.AbstractColorScheme;
import org.srs3d.viewer.j3d.AppearanceHelper;
import org.srs3d.viewer.j3d.ContextData;
import org.srs3d.viewer.objects.AbstractObject;

/**
 * The <code>TransparentColorScheme</code> modifies the transparency attributes of the
 * objects appearances.
 *
 * @author Karsten Klein
 *
 * @created March 20, 2001
 */
public class TransparentColorScheme extends AbstractColorScheme {

    /** transparency value */
    public float transparency = 0.4f;

    /**
     * <code>TransparentColorScheme</code> constructor.
     *
     * @param contextData Description of parameter.
     */
    public TransparentColorScheme(ContextData contextData) {
        super(contextData);
        setInformation(false);
    }

    /**
     * Gets the <code>Volatile</code> attribute of the
     * <code>TransparentColorScheme</code> object
     *
     * @return The <code>Volatile</code> value
     */
    public boolean isVolatile() {
        return true;
    }

    /**
     * Method description.
     *
     * @param object Parameter description.
     * @param appearance Parameter description.
     *
     * @return Return description.
     */
    public boolean modify(AbstractObject object, Appearance appearance) {
        AppearanceHelper.enableTransparency(appearance, transparency);
        return true;
    }
}
