/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.bioatlas.creators;

import javax.media.j3d.BranchGroup;

import org.srs3d.viewer.bioatlas.objects.AbstractChain;
import org.srs3d.viewer.bioatlas.objects.NucleicChain;
import org.srs3d.viewer.j3d.ContextData;
import org.srs3d.viewer.objects.AbstractObject;

/**
 * Special geometry creator implementation concentrating on <code>Dna</code> objects.
 *
 * @author Karsten Klein
 *
 * @created March 20, 2001
 */
public class DnaGeometryCreator extends StrandGeometryCreator {

    /** Description of the field */
    public float arrowSizeFactor = 1.0f;

    /**
     * Description of the method.
     *
     * @param object Description of parameter.
     * @param branchGroup Description of parameter.
     */
    public void create(AbstractObject object, BranchGroup branchGroup) {
        createGeometry((NucleicChain) object, branchGroup);
    }

    /**
     * Description of the method.
     *
     * @param object Description of parameter.
     * @param contextData Description of parameter.
     */
    public void modifyAttributes(ContextData contextData, AbstractObject object) {
        super.modifyAttributes(contextData, object);
        useCircleContour =
            org.srs3d.viewer.bioatlas.Parameter.naUseCircleContour;
        framesPerResidue =
            org.srs3d.viewer.bioatlas.Parameter.naFramesPerResidue;
        divisionsPerFrame =
            org.srs3d.viewer.bioatlas.Parameter.naDivisionsPerFrame;
        setScale(0.5f, 1.5f, 1);
    }

    /**
     * Gets the <code>endSize</code> attribute of the <code>DnaGeometryCreator </code>
     * object.
     *
     * @param chain Description of parameter.
     *
     * @return The <code>endSize</code> value.
     */
    protected float getEndSize(AbstractChain chain) {
        float endSize = 0.5f;
        return endSize;
    }
}
