/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.bioatlas.modules;

import java.awt.Component;
import java.awt.event.ActionEvent;

import javax.swing.JCheckBoxMenuItem;

import org.srs3d.viewer.j3d.Context;
import org.srs3d.viewer.j3d.ContextData;

/**
 * @author Karsten Klein
 *
 * @created February 06, 2002
 */
public class AttenuationModule extends AbstractModule {
    private String name;
    private JCheckBoxMenuItem component = null;

    /**
     * <code>ResetModule</code> constructor.
     *
     * @param name Description of parameter.
     * @param context Description of parameter.
     */
    public AttenuationModule(String name, ContextData contextData) {
        super(name, contextData);
    }

    /**
     * Gets the <code>component</code> attribute of the <code>SolidRenderingModule</code>
     * object.
     *
     * @return The <code>component</code> value.
     */
    public Component getComponent() {
        if (component == null) {
            component = new JCheckBoxMenuItem(getName(), false);
            component.addActionListener(this);
        }
        return component;
    }

    /**
     * Description of the method.
     */
    public void updateIntern() {
        Context context = getContextData().getContext();
        ((JCheckBoxMenuItem) getComponent()).setState(context.isAttenuationActive());
        getComponent().setEnabled(true);
    }

    /**
     * Description of the method.
     *
     * @param e Description of parameter.
     */
    public void react(ActionEvent e) {
        Context context = getContextData().getContext();
        context.activateAttenuation(!context.isAttenuationActive());

        //    super.actionPerformed( e );
    }
}
