/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.bioatlas.modules;

import org.srs3d.viewer.j3d.ContextData;
import org.srs3d.viewer.objects.Operation;

/**
 * Menu checks the clipboard for string content and triggers the specified operations.
 *
 * @author Karsten Klein
 *
 * @created April 24, 2002
 */
public class PasteModule extends OperationModule {

    /**
     * <code>PasteModule</code> constructor.
     *
     * @param name Description of parameter.
     * @param contextData Description of parameter.
     * @param operation Description of parameter.
     */
    public PasteModule(String name, ContextData contextData, Operation operation) {
        super(name, contextData, operation);
        setOperationSerializeable(false);
    }

    /**
     * Description of the method.
     */
    public void updateIntern() {
        if (getComponent() != null) {
            getComponent().setEnabled(false);
            java.awt.datatransfer.Transferable transferable =
                java.awt.Toolkit.getDefaultToolkit().getSystemClipboard()
                                .getContents(null);
            if (transferable != null) {
                try {
                    String string =
                        (String) transferable.getTransferData(java.awt.datatransfer.DataFlavor.stringFlavor);
                    if (string.trim().length() > 0) {
                        getComponent().setEnabled(true);
                    }
                } catch (Exception e) {

                    // :SILENT EXCEPTION:
                }
            }
        }
    }
}
