/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.bioatlas.modules;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JFrame;

import org.srs3d.viewer.applet.AppletStub;
import org.srs3d.viewer.j3d.Context;
import org.srs3d.viewer.j3d.ContextData;
import org.srs3d.viewer.util.Chronometer;

/**
 * @author Karsten Klein
 *
 * @created March 05, 2002
 */
public class RunApplicationModule extends CheckModule {
    private org.srs3d.viewer.bioatlas.Application application = null;
    private JFrame jFrame = null;

    /**
     * <code>RunApplicationModule</code> constructor.
     *
     * @param name Description of parameter.
     * @param contextData Description of parameter.
     */
    public RunApplicationModule(String name, ContextData contextData) {
        super(name, contextData);
        setOperationSerializeable(false);
    }

    /**
     * Description of the method.
     */
    public void updateIntern() {
        Context context = getContextData().getContext();
        ((JCheckBoxMenuItem) getComponent()).setState(application != null);
        getComponent().setEnabled(true);
    }

    /**
     * Description of the method.
     *
     * @param e Description of parameter.
     */
    public void react(ActionEvent e) {
        if (application == null) {
            initializeApplication();
        } else {
            destroyApplication();
        }
    }

    /**
     * Description of the method.
     */
    private void destroyApplication() {
        Chronometer.displayAll();
        jFrame.remove(application.getComponent());
        jFrame.dispose();
        application.stop();
        application.destroy();
        Chronometer.displayAll();

        //    GCThread.runFinalization();
        //    GCThread.gc();
        //    org.srs3d.viewer.bioatlas.Capture.dumpMemoryAnalysis();
        application = null;
    }

    /**
     * Description of the method.
     */
    private void initializeApplication() {
        AppletStub appletStub =
            new AppletStub(getContextData().getAppletStub());
        appletStub.setParameter("applicationMode", "Frame");
        appletStub.setParameter("fullscreen", "on");
        application = new org.srs3d.viewer.bioatlas.Capture();

        //    application.setAppletStub( appletStub );
        application.getApplicationContext().set("AppletStub", appletStub);
        jFrame =
            new JFrame(org.srs3d.viewer.bioatlas.Parameter.applicationName);
        jFrame.setSize(800, 800);
        if (appletStub.getParameter("fullscreen") != null) {
            jFrame.setSize(Toolkit.getDefaultToolkit().getScreenSize());
        }

        // :FIXME: wrapped for the obfuscator, exception after file was not found
        try {
            URL iconUrl =
                appletStub.getClass().getResource("/org/srs3d/viewer/bioatlas/resource/icon.gif");
            jFrame.setIconImage(new ImageIcon(iconUrl).getImage());
        } catch (Exception e) {

            // no icon
        }

        //    application.setParent( jFrame );
        application.getApplicationContext().set("ParentContainer", jFrame);
        application.init();
        application.start();
        jFrame.setContentPane((JComponent) application.getComponent());
        jFrame.setVisible(true);
        application.update();

        // adding some listeners
        jFrame.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    destroyApplication();
                }
            });
        jFrame.addComponentListener(new ComponentAdapter() {
                public void componentResized(ComponentEvent e) {
                    int width = Math.max(500, jFrame.getWidth());
                    int height = Math.max(500, jFrame.getHeight());
                    jFrame.setSize(new Dimension(width, height));
                    if (application != null) {
                        application.update();
                    }
                }
            });
        application.startVisualization();
    }
}
