/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.bioatlas.modules;

import java.awt.event.ActionEvent;
import java.util.Collection;
import java.util.HashSet;

import org.srs3d.viewer.bioatlas.objects.Residue;
import org.srs3d.viewer.j3d.ContextData;
import org.srs3d.viewer.j3d.Selection;
import org.srs3d.viewer.objects.ObjectManager;
import org.srs3d.viewer.objects.Operation;

/**
 * Description of the class.
 *
 * @author Karsten Klein
 *
 * @created April 29, 2002
 */
public class SelectResiduesModule extends ProcessModule {
    private transient Collection residues = null;

    /**
     * <code>SelectResiduesModule</code> constructor.
     *
     * @param name Description of parameter.
     * @param contextData Description of parameter.
     */
    public SelectResiduesModule(String name, ContextData contextData) {
        super(name, contextData, true, true);
    }

    /**
     * Gets the <code>moduleIdPrefix</code> attribute of the
     * <code>SelectResiduesModule</code> object.
     *
     * @return The <code>moduleIdPrefix</code> value.
     */
    public String getModuleIdPrefix() {
        return "SELECT-";
    }

    /**
     * Description of the method.
     */
    public void computeResidues() {
        Selection selection =
            getContextData().getSelectionManager().getSelection();
        residues = new HashSet();
        if (!selection.isEmpty()) {
            getContextData().getObjectManager().getAssociations(selection,
                residues);
            ObjectManager.extract(residues, Residue.class);
            residues.removeAll(selection);
        }
    }

    /**
     * Description of the method.
     *
     * @param e Description of parameter.
     */
    public void process(ActionEvent e) {
        ContextData contextData = getContextData();
        Selection selection =
            getContextData().getSelectionManager().getSelection();
        if (!residues.isEmpty()) {
            ObjectManager.extract(selection, residues, Residue.class);
            Collection collapsed = new HashSet();
            contextData.getObjectManager().collapseUpExtended(residues,
                collapsed);
            Operation operation =
                new Operation(contextData.getContext(), "TRANSFER_SELECTION",
                    null);
            operation.setObjects(collapsed);
            operation.setSerializable(false);
            contextData.getDispatcher().runDispatch(operation);
        }
    }

    /**
     * Description of the method.
     */
    public void updateIntern() {
        computeResidues();
        if (getComponent() != null) {
            getComponent().setEnabled(!residues.isEmpty());
        }
    }
}
