/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.bioatlas.modules;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Vector;

import org.srs3d.viewer.bioatlas.attributes.AtomRepresentation;
import org.srs3d.viewer.bioatlas.attributes.Representation;
import org.srs3d.viewer.bioatlas.attributes.ResidueRepresentation;
import org.srs3d.viewer.bioatlas.attributes.SubchainRepresentation;
import org.srs3d.viewer.bioatlas.objects.Atom;
import org.srs3d.viewer.bioatlas.objects.Residue;
import org.srs3d.viewer.bioatlas.objects.Subchain;
import org.srs3d.viewer.j3d.ContextData;
import org.srs3d.viewer.j3d.commands.CommandCollection;
import org.srs3d.viewer.j3d.commands.ExecuteClassCommand;
import org.srs3d.viewer.j3d.commands.ParentCommand;
import org.srs3d.viewer.j3d.commands.PreSelectCommand;
import org.srs3d.viewer.j3d.commands.PropagateClassCommand;
import org.srs3d.viewer.j3d.commands.RepresentationCommand;
import org.srs3d.viewer.objects.Attribute;
import org.srs3d.viewer.objects.Command;
import org.srs3d.viewer.objects.State;

/**
 * @author Karsten Klein
 *
 * @created August 06, 2001
 */
public class WireframeCheck extends AtomCheck {
    public static final WireframeCheck sharedInstance = new WireframeCheck();

    /**
     * Description of the method.
     *
     * @param subchain Description of parameter.
     *
     * @return Description of the returned value.
     */
    public int check(ContextData contextData, Subchain subchain) {
        State.Immutable state =
            contextData.getStateManager().getImmutableState(subchain);
        SubchainRepresentation.Immutable representation =
            (SubchainRepresentation.Immutable) state.getAttribute(SubchainRepresentation.class);
        if (representation != null) {
            if ((representation.getMode() &
                  Representation.REPRESENTATION_WIREFRAME) != 0) {
                return CHECK_YES;
            } else {
                return CHECK_NO;
            }
        }
        return CHECK_YES;
    }

    /**
     * Description of the method.
     *
     * @param residue Description of parameter.
     *
     * @return Description of the returned value.
     */
    public int check(ContextData contextData, Residue residue) {
        State.Immutable state =
            contextData.getStateManager().getImmutableState(residue);
        ResidueRepresentation.Immutable representation =
            (ResidueRepresentation.Immutable) state.getAttribute(ResidueRepresentation.class);
        if (representation != null) {
            if ((representation.getMode() &
                  Representation.REPRESENTATION_WIREFRAME) != 0) {
                return CHECK_YES;
            } else {
                return CHECK_NO;
            }
        }
        return CHECK_YES;
    }

    /**
     * Description of the method.
     *
     * @param atom Description of parameter.
     *
     * @return Description of the returned value.
     */
    public int check(ContextData contextData, Atom atom) {
        State.Immutable state =
            contextData.getStateManager().getImmutableState(atom);
        AtomRepresentation.Immutable representation =
            (AtomRepresentation.Immutable) state.getAttribute(AtomRepresentation.class);
        if ((representation.getMode() &
              Representation.REPRESENTATION_WIREFRAME) != 0) {
            return CHECK_YES;
        }
        return CHECK_NO;
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public int getMode() {
        return Representation.REPRESENTATION_WIREFRAME;
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public int getUndoMode() {
        return Representation.REPRESENTATION_BALL_AND_STICK |
        Representation.REPRESENTATION_VAN_DER_WAALS;
    }

    /**
     * Gets the <code>allOnCommands</code> attribute of the <code>WireframeCheck</code>
     * class.
     *
     * @param contextData Description of parameter.
     * @param onCommands Description of parameter.
     */
    public void getAllOnCommands(ContextData contextData, Collection onCommands) {
        Command subchainCommand =
            new ExecuteClassCommand(contextData,
                new PreSelectCommand(contextData), Subchain.class);
        onCommands.add(new ExecuteClassCommand(contextData,
                new ParentCommand(contextData, subchainCommand), Residue.class));
        Collection commandCollection = new Vector();
        getAtomOnCommands(contextData, commandCollection);
        Command command =
            new PropagateClassCommand(contextData,
                new PropagateClassCommand(contextData,
                    new CommandCollection(contextData, commandCollection), null),
                Residue.class);
        onCommands.add(command);
        commandCollection = new Vector();
        Vector commands = new Vector();
        getResidueOnCommands(contextData, commands);
        command = new CommandCollection(contextData, commands);
        command = new ExecuteClassCommand(contextData, command, Residue.class);
        commandCollection.add(command);
        commands = new Vector();
        getSubchainOnCommands(contextData, commands);
        command = new CommandCollection(contextData, commands);
        command = new ExecuteClassCommand(contextData, command, Subchain.class);
        commandCollection.add(command);
        command = new CommandCollection(contextData, commandCollection);
        command =
            new PropagateClassCommand(contextData, command, Residue.class);
        onCommands.add(command);
    }

    /**
     * Gets the <code>allOffCommands</code> attribute of the <code>WireframeCheck</code>
     * class.
     *
     * @param contextData Description of parameter.
     * @param offCommands Description of parameter.
     */
    public void getAllOffCommands(ContextData contextData,
        Collection offCommands) {

        // atoms and bonds
        Collection commandCollection = new ArrayList();
        getAtomOffCommands(contextData, commandCollection);
        Command command =
            new PropagateClassCommand(contextData,
                new PropagateClassCommand(contextData,
                    new CommandCollection(contextData, commandCollection), null),
                Residue.class);
        offCommands.add(command);

        // residues and subchains
        commandCollection = new ArrayList();
        Collection commands = new ArrayList();
        getResidueOffCommands(contextData, commands);
        command = new CommandCollection(contextData, commands);
        command = new ExecuteClassCommand(contextData, command, Residue.class);
        commandCollection.add(command);
        commands = new ArrayList();
        getSubchainOffCommands(contextData, commands);
        command = new CommandCollection(contextData, commands);
        command = new ExecuteClassCommand(contextData, command, Subchain.class);
        commandCollection.add(command);
        command = new CommandCollection(contextData, commandCollection);
        command =
            new PropagateClassCommand(contextData, command, Residue.class);
        offCommands.add(command);
    }

    /**
     * Gets the <code>residueOnCommands</code> attribute of the
     * <code>WireframeCheck</code> class.
     *
     * @param contextData Description of parameter.
     * @param onCommands Description of parameter.
     */
    public void getResidueCommands(ContextData contextData, Collection commands) {
        RepresentationCommand representationCommand;
        ResidueRepresentation residueRepresentation;
        residueRepresentation =
            (ResidueRepresentation) Attribute.getInstance(ResidueRepresentation.class);
        residueRepresentation.setMode(getMode());
        representationCommand = new RepresentationCommand(contextData);
        representationCommand.setRepresentation(residueRepresentation);
        commands.add(representationCommand);
    }

    /**
     * Gets the <code>subchainOnCommands</code> attribute of the
     * <code>WireframeCheck</code> class.
     *
     * @param contextData Description of parameter.
     * @param onCommands Description of parameter.
     */
    public void getSubchainCommands(ContextData contextData, Collection commands) {
        RepresentationCommand representationCommand;
        SubchainRepresentation subchainRepresentation;
        subchainRepresentation =
            (SubchainRepresentation) Attribute.getInstance(SubchainRepresentation.class);
        subchainRepresentation.setMode(getMode());
        representationCommand = new RepresentationCommand(contextData);
        representationCommand.setRepresentation(subchainRepresentation);
        commands.add(representationCommand);
    }

    /**
     * Gets the <code>allOnCommands</code> attribute of the <code>WireframeCheck</code>
     * class.
     *
     * @param contextData Description of parameter.
     * @param onCommands Description of parameter.
     */
    public void getAllCommands(ContextData contextData, Collection commands) {
        getSubchainCommands(contextData, commands);
        getResidueCommands(contextData, commands);
        getAtomOnCommands(contextData, commands);
    }

    /**
     * Method description.
     *
     * @param contextData Parameter description.
     *
     * @return Return description.
     */
    public static RepresentationModule createRepresentationModule(
        ContextData contextData) {
        Vector onCommands = new Vector();
        Vector offCommands = new Vector();
        sharedInstance.getAllOnCommands(contextData, onCommands);
        sharedInstance.getAllOffCommands(contextData, offCommands);
        return new RepresentationModule("Wireframe", contextData, onCommands,
            offCommands, sharedInstance);
    }
}
