/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.bioatlas.objects.strategies;

import javax.media.j3d.Appearance;

import org.srs3d.viewer.bioatlas.colorschemes.SimpleColorScheme;
import org.srs3d.viewer.bioatlas.objects.Annotation;
import org.srs3d.viewer.j3d.ColorScheme;
import org.srs3d.viewer.j3d.Reaction;
import org.srs3d.viewer.j3d.commands.ColorCommand;
import org.srs3d.viewer.j3d.commands.IdentificationCommand;
import org.srs3d.viewer.j3d.objects.strategies.AbstractStrategy;
import org.srs3d.viewer.objects.Command;

/**
 * @author Karsten Klein
 *
 * @created April 03, 2001
 */
public class AnnotationStrategy extends AbstractStrategy {

    /**
     * Constructor description.
     */
    public AnnotationStrategy() {
        register(ColorCommand.class, new ColorReaction());
        register(IdentificationCommand.class, new IdentificationReaction());
    }

    /**
     * Class description.
     *
     * @author Karsten Klein, LION bioscience AG
     */
    public class ColorReaction implements Reaction {

        /**
         * Method description.
         *
         * @param command Parameter description.
         */
        public void execute(Command command) {
            ColorCommand colorCommand = (ColorCommand) command;
            colorCommand.execute();
            ColorScheme colorScheme =
                new SimpleColorScheme(colorCommand.getContextData());
            Appearance appearance =
                colorCommand.getContextData().getAppearanceManager()
                            .getAppearance(command.getObject());
            colorScheme.modify(colorCommand.getObject(), appearance);
            ColorCommand.apply(colorCommand.getContextData(),
                colorCommand.getObject(), appearance);
        }
    }

    /**
     * Reaction for retrieving the object identification.
     *
     * @author Karsten Klein
     *
     * @created March 26, 2002
     */
    public static class IdentificationReaction implements Reaction {

        /**
         * Execute implementation of the reaction interface.
         *
         * @param command Command to react to.
         */
        public void execute(Command command) {
            IdentificationCommand idCommand = (IdentificationCommand) command;
            Annotation annotation = (Annotation) command.getObject();
            idCommand.setObjectId("ANN:" + annotation.getName() +
                annotation.getDescriptionHash());
            idCommand.execute();
        }
    }
}
