/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.bioatlas.objects.strategies;

import java.util.ArrayList;

import org.srs3d.viewer.bioatlas.objects.Atom;
import org.srs3d.viewer.bioatlas.objects.Residue;
import org.srs3d.viewer.j3d.Reaction;
import org.srs3d.viewer.j3d.commands.ExpandCommand;
import org.srs3d.viewer.j3d.commands.IdentificationCommand;
import org.srs3d.viewer.j3d.commands.PreSelectCommand;
import org.srs3d.viewer.j3d.objects.strategies.AbstractStrategy;
import org.srs3d.viewer.j3d.objects.strategies.reactions.EmptyReaction;
import org.srs3d.viewer.objects.Command;
import org.srs3d.viewer.objects.ObjectManager;

/**
 * Encapsulates the behavior of <code>Atom</code> instances.
 *
 * @author Karsten Klein
 *
 * @created June 26, 2001
 */
public class AtomStrategy extends AbstractStrategy {

    /**
     * Constructor description.
     */
    public AtomStrategy() {
        register(ExpandCommand.class, EmptyReaction.getSharedInstance());
        register(PreSelectCommand.class, EmptyReaction.getSharedInstance());
        register(IdentificationCommand.class, new IdentificationReaction());
    }

    /**
     * Reaction for retrieving the object identification.
     *
     * @author Karsten Klein
     *
     * @created March 26, 2002
     */
    public static class IdentificationReaction implements Reaction {

        /**
         * Execute implementation of the reaction interface.
         *
         * @param command Command to react to.
         */
        public void execute(Command command) {
            IdentificationCommand idCommand = (IdentificationCommand) command;
            Atom atom = (Atom) command.getObject();

            // :TODO: need residue index for atom
            ArrayList residues = new ArrayList();
            ObjectManager.extract(idCommand.getObjects(), residues,
                Residue.class);
            if (!residues.isEmpty()) {
                Residue residue = (Residue) residues.get(0);
                idCommand.setObjectId("A:" + atom.getTemplate().getId() +
                    ((ArrayList) residue.getAtoms()).indexOf(atom));
                idCommand.execute();
            }
        }
    }
}
