/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.bioatlas.objects.strategies;

import java.util.ArrayList;

import javax.media.j3d.Appearance;

import org.srs3d.viewer.bioatlas.colorschemes.SimpleColorScheme;
import org.srs3d.viewer.bioatlas.objects.Annotation;
import org.srs3d.viewer.bioatlas.objects.Feature;
import org.srs3d.viewer.j3d.ColorScheme;
import org.srs3d.viewer.j3d.ContextData;
import org.srs3d.viewer.j3d.Reaction;
import org.srs3d.viewer.j3d.commands.ColorCommand;
import org.srs3d.viewer.j3d.commands.IdentificationCommand;
import org.srs3d.viewer.j3d.objects.strategies.AbstractStrategy;
import org.srs3d.viewer.objects.AbstractObject;
import org.srs3d.viewer.objects.Command;
import org.srs3d.viewer.objects.ObjectManager;

/**
 * @author Karsten Klein
 *
 * @created April 03, 2001
 */
public class FeatureStrategy extends AbstractStrategy {

    /**
     * Constructor description.
     */
    public FeatureStrategy() {
        register(ColorCommand.class, new ColorReaction());
        register(IdentificationCommand.class, new IdentificationReaction());
    }

    /**
     * Class description.
     *
     * @author Karsten Klein, LION bioscience AG
     */
    public class ColorReaction implements Reaction {

        /**
         * Method description.
         *
         * @param command Parameter description.
         */
        public void execute(Command command) {
            ColorCommand colorCommand = (ColorCommand) command;
            ContextData contextData = colorCommand.getContextData();
            ColorScheme colorScheme = new SimpleColorScheme(contextData);
            AbstractObject object = (AbstractObject) colorCommand.getObject();
            Appearance appearance =
                contextData.getAppearanceManager().getAppearance(object);
            colorScheme.modify(colorCommand.getObject(), appearance);
            ColorCommand.apply(contextData, colorCommand.getObject(), appearance);
        }
    }

    /**
     * Reaction for retrieving the object identification.
     *
     * @author Karsten Klein
     *
     * @created March 26, 2002
     */
    public static class IdentificationReaction implements Reaction {

        /**
         * Execute implementation of the reaction interface.
         *
         * @param command Command to react to.
         */
        public void execute(Command command) {
            IdentificationCommand idCommand = (IdentificationCommand) command;
            Feature feature = (Feature) command.getObject();
            ArrayList annotations = new ArrayList();
            ObjectManager.extract(idCommand.getObjects(), annotations,
                Annotation.class);
            if (!annotations.isEmpty()) {
                Annotation annotation = (Annotation) annotations.get(0);
                idCommand.setObjectId("FEATURE:" + feature.getName() +
                    
                // this doesn't work, because the ordering of the residues in the
                // description is not deterministic
                feature.getDescriptionHash() +
                    annotation.getFeatures().indexOf(feature));
            }
            idCommand.execute();
        }
    }
}
