/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.integration.component;

import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Implementation of the LookupStrategy for the ejb context.
 *
 * @author Karsten Klein
 *
 * @created March 27, 2003
 */
public class EjbLookupStrategy implements LookupStrategy {
    private static final Log log = LogFactory.getLog(EjbLookupStrategy.class);

    /**
     * Description of the method.
     *
     * @param serviceName Description of parameter.
     * @param serviceClass Description of parameter.
     *
     * @return Description of the returned value.
     */
    public Object lookup(String serviceName, Class serviceClass, Object context) {
        Object object = null;
        try {
            InitialContext namingContext = new InitialContext();
            object = namingContext.lookup(serviceName);
            if (serviceClass != null) {
                object = PortableRemoteObject.narrow(object, serviceClass);
            }
            if (object == null) {
                log.error("failed to lookup service " + serviceName);
            }
        } catch (Exception e) {
            log.error("failed to lookup service " + serviceName, e);
        }
        return object;
    }
}
