/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.integration.datamodel.contract;

import java.io.Serializable;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.srs3d.viewer.integration.interfaces.Contract;
import org.srs3d.viewer.integration.interfaces.ContractQualifier;
import org.srs3d.viewer.integration.interfaces.GenericDataProvider;

/**
 * Generic implementation of the Contract interface.
 *
 * @author Karsten Klein
 *
 * @created September 11, 2002
 */
public class GenericContract implements Contract, Serializable {
    private Map qualifierMap = new HashMap();

    /**
     * Store a qualifier for a certain key object;
     *
     * @param key The identifying key object.
     * @param qualifier The qualifier for the object associated with the key.
     */
    public void setContractQualifier(Object key, ContractQualifier qualifier) {
        qualifierMap.put(key, qualifier);
    }

    /**
     * Retrieve a ContractQualifier for a given key object.
     *
     * @param key Key object for qualifier identification.
     *
     * @return The qualifier for the object associated with the key.
     */
    public ContractQualifier getContractQualifier(Object key) {
        return (ContractQualifier) qualifierMap.get(key);
    }

    /**
     * Retrive all keys with specified in the contract.
     *
     * @return Iterator for accessing the keys.
     */
    public Collection getKeys() {
        return Collections.unmodifiableCollection(qualifierMap.keySet());
    }

    /**
     * Description of the method.
     *
     * @param provider Description of parameter.
     *
     * @return Description of the returned value.
     */
    public boolean check(GenericDataProvider provider) {
        Iterator keyIterator = qualifierMap.keySet().iterator();
        Iterator qualifierIterator = qualifierMap.values().iterator();
        ContractQualifier qualifier;
        Object key;
        Object object;
        boolean isSuccess = true;
        while (keyIterator.hasNext()) {
            key = keyIterator.next();
            qualifier = (ContractQualifier) qualifierIterator.next();
            object = provider.get(key);
            if (!qualifier.check(key, object)) {
                isSuccess = false;
            }
        }
        return isSuccess;
    }
}
