/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.j3d;

import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

import org.srs3d.viewer.j3d.objects.Overlay;
import org.srs3d.viewer.objects.AbstractManager;

/**
 * Description of the class.
 *
 * @author Karsten Klein
 * @author Christian Zofka
 *
 * @created August 1, 2001
 */
public class OverlayManager extends AbstractManager {
    private Map map = new Hashtable();

    /**
     * OverlayManager constructor.
     *
     * @param contextData Description of parameter.
     */
    public OverlayManager(ContextData contextData) {
        super(contextData);
    }

    /**
     * Retrieves the overlays attribute of the OverlayManager object.
     *
     * @return The overlays value.
     */
    public Collection getOverlays() {
        return map.values();
    }

    /**
     * Retrieves the overlay attribute of the OverlayManager object.
     *
     * @param object Description of parameter.
     *
     * @return The overlay value.
     */
    public Overlay getOverlay(Object object) {
        return (Overlay) map.get(object);
    }

    /**
     * Description of the method.
     *
     * @param object Description of parameter.
     */
    public void remove(Object object) {
        map.remove(object);
    }

    /**
     * Description of the method.
     */
    public void clear() {
        map.clear();
    }

    /**
     * Description of the method.
     *
     * @param overlay Description of parameter.
     */
    public void register(Overlay overlay) {
        map.put(overlay.getName(), overlay);
    }

    /**
     * Description of the method.
     *
     * @param contextData Description of parameter.
     */
    public static void spawnOverlays(ContextData contextData) {
        Iterator overlayIterator =
            contextData.getOverlayManager().getOverlays().iterator();
        org.srs3d.viewer.j3d.objects.Overlay overlay;
        while (overlayIterator.hasNext()) {
            overlay =
                (org.srs3d.viewer.j3d.objects.Overlay) overlayIterator.next();
            org.srs3d.viewer.j3d.commands.SpawnCommand spawnCommand =
                new org.srs3d.viewer.j3d.commands.SpawnCommand(contextData);
            spawnCommand.setBranchGroup(contextData.getContext()
                                                   .getAvatarJunction()
                                                   .getOnBranch());
            contextData.getStrategyManager().execute(overlay, spawnCommand);
        }
    }
}
