/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.j3d;

import java.util.Collection;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Map;

import org.srs3d.viewer.objects.ObjectManager;
import org.srs3d.viewer.objects.filters.ObjectClassFilter;
import org.srs3d.viewer.objects.visitors.ObjectCollector;

/**
 * The processed selection is used to optimize selction processing from various sites in
 * the project. It basically caches the operation that have been performed on the
 * current selection.
 *
 * @author Karsten Klein
 *
 * @created August 31, 2001
 */
public class ProcessedSelection {
    private Map extractMap = new Hashtable();
    private Map extractDeepMap = new Hashtable();
    private Map objectMap = new Hashtable();

    //  private Selection selection = null;
    private Collection collection = null;

    /**
     * Constructor description.
     *
     * @param collection Parameter description.
     */
    public ProcessedSelection(Collection collection) {
        this.collection = collection;
    }

    /**
     * Method description.
     *
     * @param objectClass Parameter description.
     *
     * @return Return description.
     */
    public Collection extract(Class objectClass) {
        Collection set = (Collection) extractMap.get(objectClass);
        if (set == null) {
            set = new HashSet();
            ObjectManager.extract(collection, set, objectClass);
        }
        return set;
    }

    /**
     * Method description.
     *
     * @param objectClass Parameter description.
     *
     * @return Return description.
     */
    public Collection extractDeep(Class objectClass) {
        Collection set = (Collection) extractDeepMap.get(objectClass);
        if (set == null) {
            ObjectCollector objectCollector =
                new ObjectCollector(new ObjectClassFilter(objectClass));
            objectCollector.visit(collection);
            set = objectCollector.getObjects();
            extractDeepMap.put(objectClass, set);
        }
        return set;
    }

    /**
     * Method description.
     *
     * @param object Parameter description.
     *
     * @return Return description.
     */
    public Collection getObjectCollection(Object object) {
        return (Collection) objectMap.get(object);
    }

    /**
     * Method description.
     *
     * @param object Parameter description.
     * @param collection Parameter description.
     */
    public void setObjectCollection(Object object, Collection collection) {
        objectMap.put(object, collection);
    }
}
