/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.j3d;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

import org.srs3d.viewer.objects.AbstractManager;

/**
 * Description of the class.
 *
 * @author Karsten Klein
 *
 * @created May 23, 2001
 */
public class SelectionManager extends AbstractManager {
    private Map selectionMap = new Hashtable();
    private Map processedSelectionMap = new Hashtable();
    private ArrayList applicationOrder = new ArrayList();

    /**
     * Constructor description.
     *
     * @param contextData Parameter description.
     */
    public SelectionManager(ContextData contextData) {
        super(contextData);
    }

    /**
     * Gets the <code>selection</code> attribute of the <code>SelectionManager </code>
     * class.
     *
     * @return The <code>selection</code> value.
     */
    public Selection getSelection() {
        return getSelection(this);
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public Collection getSelections() {
        return selectionMap.values();
    }

    /**
     * Method description.
     *
     * @param object Parameter description.
     *
     * @return Return description.
     */
    public Selection getSelection(Object object) {
        Selection selection = (Selection) selectionMap.get(object);
        if (selection == null) {
            selection = new Selection(getContextData());
            selectionMap.put(object, selection);
            applicationOrder.add(object);
        } else {
            applicationOrder.remove(object);
            applicationOrder.add(object);
        }
        return selection;
    }

    /**
     * Description of the method.
     *
     * @param object Description of parameter.
     */
    public void remove(Object object) {
        selectionMap.remove(object);
        applicationOrder.remove(object);
    }

    /**
     * Method description.
     */
    public void clear() {
        Iterator iterator = selectionMap.values().iterator();
        Selection selection;
        while (iterator.hasNext()) {
            selection = (Selection) iterator.next();
            selection.clear();
            selection.cleanup();
        }
        selectionMap.clear();
        selectionMap = null;
        processedSelectionMap.clear();
        applicationOrder.clear();
        applicationOrder = null;
    }

    /**
     * Method description.
     *
     * @param collection Parameter description.
     *
     * @return Return description.
     */
    public ProcessedSelection getProcessedSelection(Collection collection) {
        ProcessedSelection processedSelection =
            (ProcessedSelection) processedSelectionMap.get(collection);
        if (processedSelection == null) {

            // :FIXME: this won't work if we have more than one registered selection
            // for each context; wonder if we will have it once
            processedSelectionMap.clear();
            processedSelection = new ProcessedSelection(collection);
            processedSelectionMap.put(collection, processedSelection);
        }
        return processedSelection;
    }

    /**
     * Method description.
     *
     * @param collection Parameter description.
     */
    public void removeProcessedSelection(Collection collection) {
        processedSelectionMap.remove(collection);
    }

    /**
     * Method description.
     *
     * @param object Parameter description.
     *
     * @return Return description.
     */
    public boolean hasSelection(Object object) {
        return selectionMap.containsKey(object);
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public Collection getApplicationOrder() {
        ArrayList ordered = new ArrayList(applicationOrder.size());
        Iterator iterator = applicationOrder.iterator();
        while (iterator.hasNext()) {
            ordered.add(selectionMap.get(iterator.next()));
        }
        return ordered;
    }

    /**
     * Method description.
     *
     * @param collection Parameter description.
     */
    public void invalidateProcessedSelection(Collection collection) {
        processedSelectionMap.remove(collection);
    }
}
