/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.j3d;

import java.util.Collection;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Map;

import org.srs3d.viewer.j3d.objects.Sensor;
import org.srs3d.viewer.objects.AbstractManager;
import org.srs3d.viewer.objects.AbstractObject;

/**
 * Manages sensors. One object may have one sensor. Many objects can have one sensor.
 *
 * @author Karsten Klein
 *
 * @created November 05, 2001
 */
public class SensorManager extends AbstractManager {
    private Map objectMap = new Hashtable();

    /**
     * <code>SensorManager</code> constructor.
     *
     * @param contextData associated <code>ContextData</code> instance.
     */
    public SensorManager(ContextData contextData) {
        super(contextData);
    }

    //  public Sensor getSensor( AbstractObject object ) {
    //
    //    return (Sensor) objectMap.get( object );
    //
    //  }
    public void register(Sensor sensor, AbstractObject object) {
        Collection sensors = (Collection) objectMap.get(object);
        if (sensors == null) {
            sensors = new HashSet();
            objectMap.put(object, sensors);
        }
        sensors.add(sensor);
    }

    /**
     * Method description.
     *
     * @param object Parameter description.
     *
     * @return Return description.
     */
    public Collection getSensors(AbstractObject object) {
        return (Collection) objectMap.get(object);
    }

    /**
     * Clears all data of the manager.
     */
    public void clear() {
        objectMap.clear();
    }

    /**
     * Removes all object related shape references.
     *
     * @param object specifies the shapes to remove.
     */
    public void remove(AbstractObject object) {
        objectMap.remove(object);
    }
}
