/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.j3d;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

import org.srs3d.viewer.objects.AbstractManager;
import org.srs3d.viewer.objects.AbstractObject;

/**
 * Description of the class.
 *
 * @author Karsten Klein
 *
 * @created September 6, 2001
 */
public class TransformManager extends AbstractManager {
    private Map transformMap = new Hashtable();

    /**
     * <code>TransformManager</code> constructor.
     *
     * @param contextData Description of parameter.
     */
    public TransformManager(ContextData contextData) {
        super(contextData);
    }

    /**
     * Sets the <code>transformEnabled</code> attribute of the
     * <code>TransformManager</code> object.
     *
     * @param enabled The new <code>transformEnabled</code> value.
     */
    public void setTransformEnabled(boolean enabled) {
        Iterator iterator = transformMap.values().iterator();
        Transform transform;
        while (iterator.hasNext()) {
            transform = (Transform) iterator.next();
            transform.setEnabled(enabled);
        }
    }

    /**
     * Gets the <code>transform</code> attribute of the <code>TransformManager</code>
     * object.
     *
     * @param object Description of parameter.
     *
     * @return The <code>transform</code> value.
     */
    public Transform getTransform(AbstractObject object) {
        return (Transform) transformMap.get(object);
    }

    /**
     * Description of the method.
     *
     * @param object Description of parameter.
     * @param transform Description of parameter.
     */
    public void register(AbstractObject object, Transform transform) {
        transformMap.put(object, transform);
    }

    /**
     * Description of the method.
     */
    public void clear() {
        transformMap.clear();
    }

    /**
     * Description of the method.
     *
     * @param object Description of parameter.
     */
    public void remove(AbstractObject object) {
        transformMap.remove(object);
    }
}
