/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.j3d.behaviors;

import java.util.Enumeration;
import java.util.Vector;

import org.srs3d.viewer.j3d.ContextData;
import org.srs3d.viewer.j3d.ContextManager;
import org.srs3d.viewer.j3d.constraints.Constrainable;
import org.srs3d.viewer.j3d.constraints.Constraint;

/**
 * This class implements the <code>Constrainable</code> and <code>Behavior
 * </code>interface for all subsequent subclasses.
 *
 * @author Karsten Klein
 *
 * @created November 29, 2000
 */
public abstract class AbstractBehavior extends javax.media.j3d.Behavior
    implements Behavior, Constrainable {

    /** <code>Vector</code> instance accumulating the constraints */
    private Vector constraints = null;

    /**
     * Sets the <code>Context</code> attribute of the <code>AbstractBehavior
     * </code>object.
     *
     * @param context The new <code>Context</code> value.
     */
    public void setContextData(ContextData contextData) {
        ContextManager.register(this, contextData.getContext());
    }

    /**
     * Gets the <code>Context</code> attribute of the <code>AbstractBehavior
     * </code>object.
     *
     * @return The <code>Context</code> value.
     */
    public ContextData getContextData() {
        return ContextManager.getContext(this).getContextData();
    }

    /**
     * Implementation of the <code>Constrainable</code> interface method.
     *
     * @param constraint The feature to be added to the <code>Constraint</code>
     *        attribute.
     */
    public void addConstraint(Constraint constraint) {

        // create constraint vector if it doesn't exist yet
        if (constraints == null) {
            constraints = new Vector(1, 1);
        }

        // add constraint without checking for multiple ones
        constraints.addElement(constraint);
    }

    /**
     * Implementation of the <code>Constrainable</code> interface method.
     *
     * @param object Description of parameter.
     *
     * @return Description of the returned value.
     */
    public boolean processConstraints(Object object) {
        boolean isModified = false;
        if (constraints != null) {
            for (Enumeration enumer = constraints.elements();
            		enumer.hasMoreElements();) {
                isModified |= ((Constraint) enumer.nextElement()).process(object);
            }
        }
        return isModified;
    }

    /**
     * Description of the method.
     */
    public void cleanup() {
        if (constraints != null) {
            constraints.clear();
            constraints = null;
        }
    }
}
