/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.j3d.behaviors;

import javax.media.j3d.Alpha;

import org.srs3d.viewer.j3d.ContextData;
import org.srs3d.viewer.util.ExceptionHandler;

/**
 * @author Karsten Klein, 11/2000
 *
 * @since 1.0
 */
public abstract class AutonomicBehavior extends AbstractBehavior {
    private Alpha alpha = null;
    private long freezeTime = 0;

    /**
     * Method description.
     *
     * @param alpha Parameter description.
     */
    public void setAlpha(Alpha alpha) {
        this.alpha = alpha;
        alpha.setStartTime(System.currentTimeMillis());
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public Alpha getAlpha() {
        return alpha;
    }

    /**
     * Method description.
     *
     * @param freeze Parameter description.
     */
    public void freezeAlpha(boolean freeze) {
        if (alpha != null) {
            if (freeze) {
                freezeTime = System.currentTimeMillis() - alpha.getStartTime();
            } else {
                long time = System.currentTimeMillis();
                alpha.setStartTime(time - freezeTime);
            }
        }
    }

    /**
     * Method description.
     *
     * @param contextData Parameter description.
     */
    public void dumpImage(ContextData contextData) {
        try {
            String captureEnabled =
                (String) getContextData().getProperty("CAPTURE-ENABLED");
            if (captureEnabled != null) {
                try {
                    freezeAlpha(true);
                    org.srs3d.viewer.bioatlas.modules.CaptureModule.dumpImage(getContextData()
                                                                                  .getContext(),
                        false);
                    freezeAlpha(false);
                } catch (Exception e) {
                    ExceptionHandler.handleException(e);
                }
            }
        } catch (NoClassDefFoundError e) {

            // :SILENT-EXCEPTION:
            ExceptionHandler.handleException(e, ExceptionHandler.SILENT_IN_DEBUG);
        }
    }
}
