/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.j3d.behaviors;

import org.srs3d.viewer.j3d.ContextData;

/**
 * Description of the interface.
 *
 * @author Karsten Klein
 *
 * @created March 29, 2001
 */
public interface Behavior {

    /**
     * Sets the context of the Behavior. Note that, if you are storing the context
     * yourself in the behavior object, you require to clear the reference in the
     * cleanup method. You have of course to assure <code>cleanup()</code> is called.
     *
     * @param contextData The new <code>contextData</code> value.
     */
    public void setContextData(ContextData contextData);

    /**
     * Retrieve the context the behavior operators on.
     *
     * @return The context of the behavior.
     */
    public ContextData getContextData();

    /**
     * Behavior cleanup.
     */
    public void cleanup();
}
