/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.j3d.behaviors;

import java.awt.event.MouseEvent;

import javax.media.j3d.Transform3D;
import javax.media.j3d.WakeupOnAWTEvent;
import javax.media.j3d.WakeupOr;
import javax.vecmath.Matrix3f;
import javax.vecmath.Point3d;
import javax.vecmath.Vector3f;

import org.srs3d.viewer.j3d.Context;
import org.srs3d.viewer.j3d.ContextData;

/**
 * Description of the class
 *
 * @author Karsten Klein
 *
 * @created March 20, 2001
 */
public class PanXY2DBehavior extends MouseBehavior {

    /** Integers used for saving the mouse positions */
    private int lastX;

    /** Integers used for saving the mouse positions */
    private int x;

    /** Description of the field */
    private int lastY;

    /** Description of the field */
    private int y;

    /**
     * <code>PanXY2DBehavior</code> constructor.
     *
     * @param context Description of parameter.
     */
    public PanXY2DBehavior(ContextData contextData) {
        super(contextData);
    }

    /**
     * Description of the method
     */
    public void initialize() {
        WakeupOnAWTEvent[] conditions = new WakeupOnAWTEvent[2];

        // set wakeup condition (waiting for further dragging events)
        conditions[0] =
            new WakeupOnAWTEvent(java.awt.event.MouseEvent.MOUSE_DRAGGED);

        // set wakeup condition (waiting first click)
        conditions[1] =
            new WakeupOnAWTEvent(java.awt.event.MouseEvent.MOUSE_PRESSED);
        wakeupOn(new WakeupOr(conditions));
    }

    /**
     * When processStilmulus is called by the framework the behavior will adjust scaling
     * and translation values of the viewplatform tranformation
     *
     * @param mouseEvent Description of parameter
     */
    public void processStimulus(MouseEvent mouseEvent) {
        lastX = x;
        lastY = y;
        x = mouseEvent.getX();
        y = mouseEvent.getY();

        // prepare members
        // NOTE: for a MOUSE_PRESSED event this is the only thing to
        //  be done for initializing the subsequent dragging events
        if (checkCriteria(mouseEvent)) {
            Context context = getContextData().getContext();
            KeyHandler.setCursor(getContextData(), "PanXY", true);
            Transform3D transform = context.getViewingPlatformTransform();
            Matrix3f matrix = new Matrix3f();
            transform.get(matrix);
            Vector3f directionX = new Vector3f();
            Vector3f directionY = new Vector3f();
            matrix.getColumn(0, directionX);
            matrix.getColumn(1, directionY);

            // get current translation of the ViewingPlatform
            Vector3f translation = new Vector3f();
            transform.get(translation);
            float panSpeed = (2.0f + Math.abs(translation.z)) / 1000;
            int deltaX = x - lastX;
            int deltaY = y - lastY;
            deltaX = Math.min(deltaX, MAX_DELTA);
            deltaX = Math.max(deltaX, -MAX_DELTA);
            deltaY = Math.min(deltaY, MAX_DELTA);
            deltaY = Math.max(deltaY, -MAX_DELTA);
            directionX.scale(-panSpeed * deltaX);
            translation.add(directionX);
            directionY.scale(panSpeed * deltaY);
            translation.add(directionY);
            context.setViewingPlatformPosition(new Point3d(translation));
        }
    }

    /**
     * This method checks the mouseEvent (fail early, fail fast)
     *
     * @param mouseEvent Description of parameter
     *
     * @return Description of the returned value
     */
    protected boolean checkCriteria(MouseEvent mouseEvent) {

        // fail early, fail fast procedure for dragging
        if (mouseEvent.getID() != MouseEvent.MOUSE_DRAGGED) {
            return false;
        }
        if ((MouseEvent.BUTTON3_MASK & mouseEvent.getModifiers()) == 0) {
            return false;
        }
        if (mouseEvent.getClickCount() != 0) {
            return false;
        }
        if (mouseEvent.isAltDown()) {
            return false;
        }
        if (mouseEvent.isControlDown()) {
            return false;
        }
        if (mouseEvent.isShiftDown()) {
            return false;
        }
        return true;
    }
}
