/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.j3d.behaviors.dispatcher;

import java.util.Collection;

import org.srs3d.viewer.objects.Operation;

/**
 * @author Karsten Klein, 11/2000
 *
 * @created May 18, 2001
 * @since 1.0
 */
public interface Dispatcher {

    /**
     * Dispatches the operation on a single abstract object.
     *
     * @param operation <code>String</code> identifying a specific operation.
     */
    public void dispatch(Operation operation);

    /**
     * Directly dispatches the
     *
     * @param operation Description of parameter.
     */
    public void runDispatch(Operation operation);

    /**
     * Registes a collection of dispatches to the operationId. Dispatching an operation
     * with the specified id will execute the dispatches in order of appearance in the
     * collection.
     *
     * @param operationId Id of the operation to provide dispatches for.
     * @param runnables Collection of dispatches.
     */
    public void registerDispatches(Object object, Collection dispatches);

    /**
     * Returns the collection of dispatches registered to the operation id.
     *
     * @param operationId Id of the operation to provide dispatches for.
     *
     * @return Collection of dispatches.
     */
    public Collection getDispatches(Object object);

    /**
     * Method description.
     */
    public void cleanup();
}
