/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.j3d.behaviors.dispatcher;

import org.srs3d.viewer.j3d.ContextData;
import org.srs3d.viewer.objects.Operation;

/**
 * Description of the interface.
 *
 * @author Karsten Klein
 *
 * @created March 26, 2002
 */
public class OperationDispatch implements Dispatch {
    private String operationId = null;
    private boolean isOperationSerializeable = true;

    /**
     * Constructor description.
     *
     * @param operationId Parameter description.
     */
    public OperationDispatch(String operationId) {
        this.operationId = operationId;
        this.isOperationSerializeable = true;
    }

    /**
     * Constructor description.
     *
     * @param operationId Parameter description.
     * @param isOperationSerializeable Parameter description.
     */
    public OperationDispatch(String operationId,
        boolean isOperationSerializeable) {
        this.operationId = operationId;
        this.isOperationSerializeable = isOperationSerializeable;
    }

    /**
     * Description of the method.
     *
     * @param contextData Description of parameter.
     * @param operation Description of parameter.
     */
    public void dispatch(ContextData contextData, Operation operation) {
        Operation op =
            new Operation(contextData.getContext(), operationId, null);
        op.setSerializable(isOperationSerializeable);
        contextData.getDispatcher().runDispatch(op);
    }
}
