/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.j3d.commands;

import java.util.Collection;
import java.util.Iterator;

import javax.media.j3d.Shape3D;

import org.srs3d.viewer.j3d.ContextData;
import org.srs3d.viewer.j3d.ShapeManager;

/**
 * GeometryLock command.
 *
 * @author Karsten Klein
 *
 * @created April 03, 2001
 */
public class GeometryLockCommand extends ObjectCommand {
    private boolean isLocking = true;

    /**
     * Constructor description.
     *
     * @param contextData Parameter description.
     * @param isLocking Parameter description.
     */
    public GeometryLockCommand(ContextData contextData, boolean isLocking) {
        super(contextData);
        this.isLocking = isLocking;
    }

    /**
     * Description of the method.
     */
    public void execute() {
        ShapeManager shapeManager = getContextData().getShapeManager();
        Collection shapes = shapeManager.getShapes(getObject());
        if (shapes != null) {
            Iterator iterator = shapes.iterator();
            Shape3D shape;
            while (iterator.hasNext()) {
                shape = (Shape3D) iterator.next();
                shapeManager.setShapeLock(shape, isLocking());
            }
        }
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public final boolean isLocking() {
        return isLocking;
    }
}
