/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.j3d.commands;

import java.util.Collection;

import org.srs3d.viewer.j3d.ContextData;
import org.srs3d.viewer.objects.AbstractObject;

/**
 * Expand command.
 *
 * @author Karsten Klein
 *
 * @created April 03, 2001
 */
public abstract class ModificationCommand extends ObjectCommand {
    private boolean hasModified = false;
    private Collection modifiedObjects = null;

    /**
     * <code>ExpandCommand</code> constructor.
     *
     * @param contextData Description of parameter.
     */
    public ModificationCommand(ContextData contextData) {
        super(contextData);
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public final boolean hasModified() {
        return hasModified;
    }

    /**
     * Method description.
     *
     * @param modifiedObjects Parameter description.
     */
    public final void setModifiedObjects(Collection modifiedObjects) {
        this.modifiedObjects = modifiedObjects;
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public final Collection getModifiedObjects() {
        return modifiedObjects;
    }

    /**
     * Method description.
     *
     * @param object Parameter description.
     */
    public final void registerModifiedObject(AbstractObject object) {
        if (modifiedObjects != null) {
            modifiedObjects.add(object);
        }
        hasModified = true;
    }
}
