/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.j3d.commands;

import java.util.Collection;
import java.util.HashSet;

import org.srs3d.viewer.j3d.ContextData;
import org.srs3d.viewer.objects.Command;

/**
 * Description of the class.
 *
 * @author Karsten Klein
 *
 * @created January 21, 2002
 */
public class ParentCommand extends ObjectCommand {
    private Command command = null;

    /**
     * Constructor description.
     *
     * @param contextData Parameter description.
     * @param command Parameter description.
     */
    public ParentCommand(ContextData contextData, Command command) {
        super(contextData);
        this.command = command;
    }

    /**
     * Gets the <code>command</code> attribute of the <code>ParentCommand</code> object.
     *
     * @return The <code>command</code> value.
     */
    public Command getCommand() {
        return command;
    }

    /**
     * Description of the method.
     */
    public void execute() {
        Collection parents = new HashSet();
        if (getParent() != null) {
            parents.add(getParent());
        }
        getContextData().getObjectManager().getDirectUpAssociations(getObject(),
            parents);
        getContextData().getStrategyManager().execute(parents, getCommand());
    }
}
