/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.j3d.commands;

import org.srs3d.viewer.j3d.ContextData;
import org.srs3d.viewer.j3d.attributes.Visible;
import org.srs3d.viewer.objects.Attribute;
import org.srs3d.viewer.objects.State;
import org.srs3d.viewer.objects.StateManager;

/**
 * Visible command.
 *
 * @author Karsten Klein
 *
 * @created April 03, 2001
 */
public class VisibleCommand extends ModificationCommand {
    private boolean isShow = true;

    /**
     * Constructor description.
     *
     * @param contextData Parameter description.
     * @param isShow Parameter description.
     */
    public VisibleCommand(ContextData contextData, boolean isShow) {
        super(contextData);
        this.isShow = isShow;
    }

    /**
     * Description of the method.
     */
    public void execute() {
        StateManager stateManager = getContextData().getStateManager();
        State.Immutable immutableState =
            stateManager.getImmutableState(getObject());
        if (isShow) {
            if (!immutableState.hasAttribute(Visible.class)) {
                State state = immutableState.getCopy();
                state.setAttribute(Attribute.getInstance(Visible.class));
                stateManager.register(getObject(), state);
                registerModifiedObject(getObject());
            }
        } else {
            if (immutableState.hasAttribute(Visible.class)) {
                State state = immutableState.getCopy();
                state.removeAttribute(Visible.class);
                stateManager.register(getObject(), state);
                registerModifiedObject(getObject());
                getContextData().getShapeManager().remove(getObject());
            }
        }
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public boolean isShow() {
        return isShow;
    }
}
