/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.j3d.loaders.raster3d;

import java.util.StringTokenizer;
import java.util.Vector;

import javax.media.j3d.BranchGroup;
import javax.vecmath.Color4f;
import javax.vecmath.Point3f;

import org.srs3d.viewer.j3d.geometry.primitive.Sphere;

/**
 * Class description.
 *
 * @author Karsten Klein, LION bioscience AG
 */
public class Raster3DSphere extends Sphere implements AbstractRaster3DPrimitive {
    Color4f color = null;

    /**
     * Constructor description.
     */
    public Raster3DSphere() {
        super();
        color = new Color4f();
    }

    /**
     * Creates a sphere from a raster3D string. The string format  is "x y z R r g b".
     */
    public boolean create(String string) {
        StringTokenizer tokenizer = new StringTokenizer(string);
        float f;
        if (tokenizer.countTokens() == 7) {
            Point3f[] coordinates = getCoordinates().getBuffer();
            for (int index = 0; index < 7; index++) {
                f = Float.parseFloat(tokenizer.nextToken());
                switch (index) {

                    case 0:
                        coordinates[0].x = f;
                        break;

                    case 1:
                        coordinates[0].y = f;
                        break;

                    case 2:
                        coordinates[0].z = f;
                        break;

                    case 3:
                        setRadius(f);
                        break;

                    case 4:
                        color.x = f;
                        break;

                    case 5:
                        color.y = f;
                        break;

                    case 6:
                        color.z = f;
                        break;
                }
            }
            return true;
        } else {
            return false;
        }
    }

    /**
     * Method description.
     *
     * @param lines Parameter description.
     * @param triangles Parameter description.
     * @param branch Parameter description.
     */
    public void supply(Vector lines, Vector triangles, BranchGroup branch) {
        branch.addChild(getShape());
    }
}
