/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.j3d.objects;

import javax.media.j3d.ModelClip;
import javax.vecmath.Color3f;
import javax.vecmath.Vector3f;
import javax.vecmath.Vector4d;

import org.srs3d.viewer.j3d.Colorable;
import org.srs3d.viewer.objects.AbstractObject;
import org.srs3d.viewer.vecmath.PointArray3f;

/**
 * Instances of this class describe a box that can be displayed in 3d.
 *
 * @author Karsten Klein
 *
 * @created March 20, 2001
 */
public class Box extends AbstractObject implements Colorable {
    private PointArray3f coordinates = null;
    private Color3f color = new Color3f(0.3f, 0.3f, 0.3f);

    /**
     * <code>Box</code> constructor.
     */
    public Box() {
        coordinates = new PointArray3f(2);
        coordinates.setAt(0, -0.5f, -0.5f, -0.5f);
        coordinates.setAt(1, 0.5f, 0.5f, 0.5f);
    }

    /**
     * Gets the <code>Coordinates</code> attribute of the <code>Box</code> object.
     *
     * @return The <code>coordinates</code> value.
     */
    public PointArray3f getCoordinates() {
        return coordinates;
    }

    /**
     * Description of the method.
     */
    public void cleanup() {
        super.cleanup();
        color = null;
        coordinates = null;
    }

    /**
     * Description of the method.
     *
     * @param modelClip Description of parameter.
     */
    public void create(ModelClip modelClip) {
        Vector4d plane = new Vector4d();
        Vector3f vector = new Vector3f();
        coordinates.setAt(0, Float.MAX_VALUE, Float.MAX_VALUE, Float.MAX_VALUE);
        coordinates.setAt(1, Float.MIN_VALUE, Float.MIN_VALUE, Float.MIN_VALUE);
        for (int i = 0; i < 6; i++) {
            modelClip.getPlane(i, plane);
            vector.x = (float) plane.x;
            vector.y = (float) plane.y;
            vector.z = (float) plane.z;
            vector.scale((float) plane.w);
            coordinates.min(0, vector);
            coordinates.max(1, vector);
        }
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public Color3f getColor() {
        return color;
    }

    /**
     * Method description.
     *
     * @param color Parameter description.
     */
    public void setColor(Color3f color) {
        this.color = color;
    }
}
