/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.j3d.objects.strategies;

import java.util.Collection;
import java.util.Iterator;

import javax.media.j3d.Raster;
import javax.media.j3d.Shape3D;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

import org.srs3d.viewer.j3d.Reaction;
import org.srs3d.viewer.j3d.commands.ColorCommand;
import org.srs3d.viewer.j3d.commands.TranslateCommand;
import org.srs3d.viewer.j3d.objects.Label;
import org.srs3d.viewer.j3d.objects.strategies.reactions.EmptyReaction;
import org.srs3d.viewer.objects.Command;

/**
 * @author Karsten Klein
 *
 * @created April 03, 2001
 */
public class LabelStrategy extends AbstractStrategy {

    /**
     * Constructor description.
     */
    public LabelStrategy() {
        register(TranslateCommand.class, new LabelTransformReaction());
        register(ColorCommand.class, EmptyReaction.getSharedInstance());
    }

    /**
     * Class description.
     *
     * @author Karsten Klein, LION bioscience AG
     */
    public class LabelTransformReaction implements Reaction {

        /**
         * Method description.
         *
         * @param command Parameter description.
         */
        public void execute(Command command) {
            TranslateCommand translate = (TranslateCommand) command;

            // the translation acts on the label position directly
            Label label = (Label) translate.getObject();
            Collection shapes =
                translate.getContextData().getShapeManager().getShapes(label);
            Vector3f vector = new Vector3f(translate.getTranslation());
            if (shapes != null) {
                Iterator iterator = shapes.iterator();
                Raster raster;
                while (iterator.hasNext()) {
                    raster = (Raster) ((Shape3D) iterator.next()).getGeometry();
                    raster.setPosition(new Point3f(vector));
                }
            }
            label.getCoordinate().set(vector);
        }
    }
}
