/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.j3d.operations;

import org.srs3d.viewer.j3d.Context;
import org.srs3d.viewer.j3d.behaviors.UpdateBehavior.Callback;
import org.srs3d.viewer.objects.Operation;

/**
 * Operation carrying a callback for execution from the dispatcher.
 *
 * @author Karsten Klein
 *
 * @created March 19, 2002
 */
public class CallbackOperation extends Operation {
    private Callback callback = null;

    /**
     * <code>CallbackOperation</code> constructor.
     *
     * @param context Description of parameter.
     * @param id Description of parameter.
     * @param callback Description of parameter.
     */
    public CallbackOperation(Context context, String id, Callback callback) {
        super(context, id, null);
        setCallback(callback);
        setSerializable(false);
    }

    /**
     * Sets the <code>callback</code> attribute of the <code>CallbackOperation</code>
     * object.
     *
     * @param callback The new <code>callback</code> value.
     */
    public void setCallback(Callback callback) {
        this.callback = callback;
    }

    /**
     * Gets the <code>callback</code> attribute of the <code>CallbackOperation</code>
     * object.
     *
     * @return The <code>callback</code> value.
     */
    public Callback getCallback() {
        return callback;
    }
}
