/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.j3d.operations;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.srs3d.viewer.j3d.Context;
import org.srs3d.viewer.j3d.ContextData;
import org.srs3d.viewer.objects.Operation;
import org.srs3d.viewer.util.XmlTag;

/**
 * @author Karsten Klein
 *
 * @created June 28, 2001
 */
public class TestOperation extends Operation {
    private Map reportMap = null;

    /**
     * Constructor description.
     *
     * @param context Parameter description.
     * @param id Parameter description.
     */
    public TestOperation(Context context, String id) {
        super(context, id, null);
    }

    /**
     * Method description.
     *
     * @param reportMap Parameter description.
     */
    public void setReport(Map reportMap) {
        this.reportMap = reportMap;
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public Map getReport() {
        return reportMap;
    }

    /**
     * Method description.
     *
     * @param contextData Parameter description.
     * @param xmlTag Parameter description.
     *
     * @return Return description.
     */
    public boolean deserializeAttributes(ContextData contextData, XmlTag xmlTag) {
        if (super.deserializeAttributes(contextData, xmlTag)) {
            XmlTag reportTag = xmlTag.getTag("report");
            Map localReportMap = new HashMap();
            Iterator iterator = reportTag.getAttributes().iterator();
            String attribute;
            String value;
            while (iterator.hasNext()) {
                attribute = (String) iterator.next();
                value = reportTag.getAttribute(attribute);
                if (value != null) {
                    localReportMap.put(attribute.toUpperCase(), value);
                }
            }
            setReport(localReportMap);
            return true;
        }
        return false;
    }

    /**
     * Method description.
     *
     * @param contextData Parameter description.
     * @param xmlTag Parameter description.
     *
     * @return Return description.
     */
    public boolean serializeAttributes(ContextData contextData, XmlTag xmlTag) {
        if (super.serializeAttributes(contextData, xmlTag)) {
            xmlTag.setAttribute("subId", "Test");
            Map reportMap = getReport();
            XmlTag reportTag = new XmlTag("report");
            if (reportMap != null) {
                Iterator iterator = reportMap.keySet().iterator();
                String attribute;
                while (iterator.hasNext()) {
                    attribute = (String) iterator.next();
                    reportTag.setAttribute(attribute.toLowerCase(),
                        (String) reportMap.get(attribute));
                }
            }
            xmlTag.getSubtags().add(reportTag);
            return true;
        }
        return false;
    }
}
