/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.j3d.operations;

import javax.vecmath.Tuple3f;
import javax.vecmath.Vector3f;

import org.srs3d.viewer.j3d.Context;
import org.srs3d.viewer.objects.AbstractObject;
import org.srs3d.viewer.objects.Operation;

/**
 * Description of the class.
 *
 * @author Christian Zofka
 *
 * @created September 5, 2001
 */
public class ZoomBoxOperation extends Operation {
    private Vector3f deltaPosition = new Vector3f();
    private Vector3f deltaSize = new Vector3f();

    /**
     * <code>ZoomBoxOperation</code> constructor.
     *
     * @param context Description of parameter.
     * @param id Description of parameter.
     * @param object Description of parameter.
     */
    public ZoomBoxOperation(Context context, String id, AbstractObject object) {
        super(context, id, object);
        setSerializable(false);
    }

    /**
     * Sets the <code>deltaPosition</code> attribute of the <code>ZoomBoxOperation</code>
     * object.
     *
     * @param deltaPosition The new <code>deltaPosition</code> value.
     */
    public void setDeltaPosition(Tuple3f deltaPosition) {
        this.deltaPosition.set(deltaPosition);
    }

    /**
     * Sets the <code>deltaSize</code> attribute of the <code>ZoomBoxOperation</code>
     * object.
     *
     * @param deltaSize The new <code>deltaSize</code> value.
     */
    public void setDeltaSize(Tuple3f deltaSize) {
        this.deltaSize.set(deltaSize);
    }

    /**
     * Gets the <code>deltaPosition</code> attribute of the <code>ZoomBoxOperation</code>
     * object.
     *
     * @return The <code>deltaPosition</code> value.
     */
    public Vector3f getDeltaPosition() {
        return deltaPosition;
    }

    /**
     * Gets the <code>deltaSize</code> attribute of the <code>ZoomBoxOperation</code>
     * object.
     *
     * @return The <code>deltaSize</code> value.
     */
    public Vector3f getDeltaSize() {
        return deltaSize;
    }
}
