/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.j3d.visitors;

import javax.media.j3d.ImageComponent2D;
import javax.media.j3d.Texture;

/**
 * Class description.
 *
 * @author Karsten Klein, LION bioscience AG
 */
public class TextureAlphaModifier {
    public float transparencyFactor = 1.0f;

    /**
     * Method description.
     *
     * @param texture Parameter description.
     */
    public void visit(Texture texture) {
        ImageComponent2D image = (ImageComponent2D) texture.getImage(0);
        visit(image);
        texture.setImage(0, image);
    }

    /**
     * Method description.
     *
     * @param image Parameter description.
     */
    public void visit(ImageComponent2D image) {
        java.awt.image.BufferedImage bufferedImage = image.getImage();
        image.set(bufferedImage);
    }

    /**
     * Method description.
     *
     * @param bufferedImage Parameter description.
     */
    public void visit(java.awt.image.BufferedImage bufferedImage) {
        int x;
        int y;
        int argb = 0;
        int r;
        int g;
        int b;
        int a;
        for (y = 0; y < bufferedImage.getHeight(); y++) {
            for (x = 0; x < bufferedImage.getWidth(); x++) {
                argb = bufferedImage.getRGB(x, y);
                r = argb & 0x00FF0000;
                r = r >> 16;
                g = argb & 0x0000FF00;
                g = g >> 8;
                b = argb & 0x000000FF;

                // make every shade of gray transparent
                if (r - 100 < g && r + 100 > g && g - 100 < b && g + 100 > b) {
                    a = 255 - (r + g + b) / 3;

                    // this is done to get at least some gray
                    a = (int) (transparencyFactor * a);
                    if (a > 255) {
                        a = 255;
                    }

                    // shade transparent
                    argb &= 0x00FFFFFF;
                    argb |= a << 24;
                    bufferedImage.setRGB(x, y, argb);
                } else {

                    // fully opaque
                    argb &= 0x00FFFFFF;
                    argb |= 0xFF000000;
                    bufferedImage.setRGB(x, y, argb);
                }
            }
        }
    }

    /**
     * Method description.
     *
     * @param bufferedImage Parameter description.
     * @param alpha Parameter description.
     */
    public void visit(java.awt.image.BufferedImage bufferedImage, char alpha) {
        int x;
        int y;
        int argb = 0;
        int r;
        int g;
        int b;
        int a;
        for (y = 0; y < bufferedImage.getHeight(); y++) {
            for (x = 0; x < bufferedImage.getWidth(); x++) {
                argb = bufferedImage.getRGB(x, y);
                argb &= 0x00FFFFFF;
                argb |= alpha << 24;
                bufferedImage.setRGB(x, y, argb);
            }
        }
    }
}
