/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.objects;

import org.srs3d.viewer.j3d.ContextData;
import org.srs3d.viewer.j3d.Contextual;

/**
 * The AbstractManager implements methods common to all context deoendent managers.
 *
 * @author Karsten Klein
 * @version April 03, 2001
 */
public abstract class AbstractManager implements Contextual {
    private final ContextData contextData;
    private boolean isVerbose = false;

    /**
     * <code>AbstractManager</code> constructor.
     *
     * @param contextData Description of parameter.
     */
    public AbstractManager(ContextData contextData) {
        this.contextData = contextData;
    }

    /**
     * Sets the <code>verbose</code> attribute of the <code>AbstractManager</code>
     * object.
     *
     * @param isVerbose The new <code>verbose</code> value.
     */
    public final void setVerbose(boolean isVerbose) {
        this.isVerbose = isVerbose;
    }

    /**
     * Gets the <code>verbose</code> attribute of the <code>AbstractManager</code>
     * object.
     *
     * @return The <code>verbose</code> value.
     */
    public final boolean isVerbose() {
        return isVerbose;
    }

    /**
     * Gets the <code>contextData</code> attribute of the <code>AbstractManager</code>
     * object.
     *
     * @return The <code>contextData</code> value.
     */
    public final ContextData getContextData() {
        return contextData;
    }
}
