/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.objects;


/**
 * The command interface defines the top-level interface of comamnds. Commands represent
 * are to be understood as low granularity operation (operations are made up from
 * commands). Commands are processed by <code>Strategy</code> instances of the
 * respective objects.
 *
 * @author Karsten Klein
 * @version April 03, 2001
 */
public interface Command {

    /**
     * Description of the method.
     */
    public void execute();

    /**
     * Gets the object the command has to be executed on.
     *
     * @return Command target object.
     */
    public AbstractObject getObject();

    /**
     * Sets the commands target object.
     *
     * @param object Command target object.
     */
    public void setObject(AbstractObject object);

    /**
     * Often commands refer to its target objects parent.
     *
     * @return Parent of the target object.
     */
    public AbstractObject getParent();

    /**
     * Sets the parent object of the target object.
     *
     * @param parent Parent of the target object.
     */
    public void setParent(AbstractObject parent);

    /**
     * Method description.
     *
     * @return Return description.
     */
    public String getDescription();
}
