/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.objects;


/**
 * The Computation interface is intended to enable a monitoring of a computation.
 *
 * @author Karsten Klein
 * @version August 22, 2001
 */
public interface Computation {

    /**
     * Retrieves the progress of the current computation in percent. Note that the
     * progress is meant as local progress (for this specific description) or global. It
     * should never bew a mixture of both.
     *
     * @return Progress in percent.
     */
    public int getProgress();

    /**
     * Checks if the computation is finished.
     *
     * @return <code>true</code> if computation has terminated
     */
    public boolean hasFinished();

    /**
     * Method that cancels the computation. Each computation should try to cancel as soon
     * as possible and without artefacts (in terms of memory ore visible changes).
     */
    public void cancel();

    /**
     * Determines whether the computation can be canceled.
     *
     * @return <code>true</code> if computation can cancel.
     */
    public boolean canCancel();

    /**
     * Retrieves the description of a computation.
     *
     * @return Description of the computation.
     */
    public String getDescription();
}
