/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.objects;


/**
 * The <code>ObjectFilter</code> provides a common interface for filtering objects.
 *
 * @author Karsten Klein
 * @version April 03, 2001
 */
public interface ObjectFilter {

    /**
     * Apply filter to the specified object.
     *
     * @param object Object to process.
     *
     * @return <code>true</code> if the object resisted the filter.
     */
    public boolean filter(AbstractObject object);

    /**
     * Decide whether there is a object of interest rooted by the specified object.
     *
     * @param object Object to process.
     *
     * @return <code>true</code> if the object resisted the filter.
     */
    public boolean preFilter(AbstractObject object);
}
