/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.objects;

import java.util.Hashtable;
import java.util.Map;

import org.srs3d.viewer.j3d.ContextData;

/**
 * This <code>StatePrototypeManager</code> class is an associative registry of
 * <code>AbstractObject</code> classes to default <code>State</code> objects.
 *
 * @author Karsten Klein
 * @version April 05, 2001
 */
public class StatePrototypeManager extends AbstractManager {

    /** Association store for the state prototypes. */
    private Map prototypeMap = new Hashtable();
    private transient State cachedPrototype = null;
    private transient Class cachedObjectClass = null;
    private int hit = 0;
    private int miss = 0;

    /**
     * <code>StatePrototypeManager</code> constructor.
     *
     * @param contextData Description of parameter.
     */
    public StatePrototypeManager(ContextData contextData) {
        super(contextData);
    }

    /**
     * Returns a <code>State</code> prototype if available (register first!!).
     *
     * @param object Object to retrieve the state for.
     *
     * @return The new <code>State</code> instance, or <code>null</code> .
     */
    public State getState(AbstractObject object) {
        State state;
        if (object.getClass() == cachedObjectClass) {
            state = cachedPrototype;
            hit++;
        } else {
            cachedObjectClass = object.getClass();
            state = (State) prototypeMap.get(cachedObjectClass);
            cachedPrototype = state;
            miss++;
        }
        if (state != null) {

            //      StateManager.countCopy( object );
            // use the copy() method of the state object to clone the instance
            return state.copy();
        }
        return null;
    }

    /**
     * Method description.
     *
     * @param object Parameter description.
     *
     * @return Return description.
     */
    public State.Immutable getImmutableState(AbstractObject object) {
        State state;
        if (object.getClass() == cachedObjectClass) {
            state = cachedPrototype;
            hit++;
        } else {
            cachedObjectClass = object.getClass();
            state = (State) prototypeMap.get(cachedObjectClass);
            cachedPrototype = state;
            miss++;
        }
        if (state != null) {

            // use the copy() method of the state object to clone the instance
            return state.getImmutable();
        }
        return null;
    }

    /**
     * Registers the specified class with a prototype <code>State</code> .
     *
     * @param objectClass Object class.
     * @param state Prototype <code>State</code> to associate with class.
     */
    public void register(Class objectClass, State state) {
        if (state != null) {
            prototypeMap.put(objectClass, state.copy());
        }
    }

    /**
     * Description of the method.
     */
    public void clearRegistry() {
        prototypeMap.clear();
    }
}
