/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.objects.filters;

import org.srs3d.viewer.objects.AbstractObject;
import org.srs3d.viewer.objects.ObjectFilter;

/**
 * The <code>ObjectFilter</code> provides a common interface for filtering objects.
 *
 * @author Karsten Klein
 * @version April 03, 2001
 */
public class ObjectSubclassFilter implements ObjectFilter {
    private Class objectClass = null;

    /**
     * <code>ObjectClassFilter</code> constructor.
     *
     * @param objectClass Class of the object that are to be filtered.
     */
    public ObjectSubclassFilter(Class objectClass) {
        setObjectClass(objectClass);
    }

    /**
     * Explicitly sets the class of the objects that are to be filtered.
     *
     * @param objectClass The new <code>objectClass</code> to be filtered.
     */
    public void setObjectClass(Class objectClass) {
        this.objectClass = objectClass;
    }

    /**
     * Implementation of the appropriate <code>ObjectFilter</code> method. Note that this
     * method uses the isAssignableFrom() method to determine whether the object is of a
     * certain class.
     *
     * @param object Object to apply filter on.
     *
     * @return <code>true</code> if object was filtered; <code>false</code> otherwise.
     */
    public boolean filter(AbstractObject object) {
        return (objectClass.isAssignableFrom(object.getClass()));
    }

    /**
     * Implementation of the appropriate <code>ObjectFilter</code> method. Note that this
     * method uses the isAssignableFrom() method to determine whether the object is of a
     * certain class.
     *
     * @param object Object to be prefiltered.
     *
     * @return <code>true</code> if the object is in the upper hierarchy of the objects
     *         to be filtered; <code>false</code> otherwise.
     */
    public boolean preFilter(AbstractObject object) {
        return (!objectClass.isAssignableFrom(object.getClass()));
    }
}
