/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.objects.tests;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.srs3d.viewer.j3d.ContextData;
import org.srs3d.viewer.j3d.attributes.Visible;
import org.srs3d.viewer.objects.AbstractObject;
import org.srs3d.viewer.objects.Attribute;
import org.srs3d.viewer.objects.ObjectContainer;
import org.srs3d.viewer.objects.State;
import org.srs3d.viewer.objects.StateManager;
import org.srs3d.viewer.objects.StatePrototypeManager;

/**
 * Test for the State class.
 *
 * @author Karsten Klein
 * @version January 7, 2002
 */
public class StateManagerTest extends TestCase {
    private transient ContextData contextData = null;
    private transient AbstractObject objA = null;
    private transient AbstractObject objB = null;
    private transient State stateA = null;

    /**
     * Constructor description.
     *
     * @param name Parameter description.
     */
    public StateManagerTest(String name) {
        super(name);
    }

    /**
     * A unit test for JUnit.
     */
    public void test() {
        StatePrototypeManager prototypeManager =
            contextData.getStatePrototypeManager();
        StateManager stateManager = contextData.getStateManager();
        State state = stateManager.getState(objA);
        State.Immutable immutableState = stateManager.getImmutableState(objA);
        Assert.assertTrue("State incorrect", state.isEqual(stateA));
        Assert.assertTrue("Immutable state incorrect",
            immutableState.getCopy().isEqual(stateA));
        state = stateManager.getState(objB);
        Assert.assertTrue("State incorrect", state.hasAttribute(Visible.class));
        state.removeAttribute(Visible.class);
        stateManager.register(objB, state);
        state = stateManager.getState(objB);
        Assert.assertTrue("State incorrect", !state.hasAttributes());
    }

    /**
     * Method description.
     *
     * @throws Exception Exception description.
     */
    public void setUp() throws Exception {
        super.setUp();
        contextData = new ContextData(null);
        StatePrototypeManager prototypeManager =
            contextData.getStatePrototypeManager();
        StateManager stateManager = contextData.getStateManager();
        State statePrototype = new State();
        statePrototype.setAttribute(Attribute.getInstance(Visible.class));
        prototypeManager.register(ObjectContainer.class, statePrototype);
        stateA = new State();
        stateA.setAttribute(Attribute.getInstance(Visible.class));
        objA = new ObjectContainer();
        stateManager.register(objA, stateA);
        objB = new ObjectContainer();
    }

    /**
     * Method description.
     *
     * @throws Exception Exception description.
     */
    public void tearDown() throws Exception {
        super.tearDown();
        contextData.cleanup();
    }
}
