/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.objects.tests;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.srs3d.viewer.j3d.attributes.Visible;
import org.srs3d.viewer.objects.Attribute;
import org.srs3d.viewer.objects.State;

/**
 * Test for the State class.
 *
 * @author Karsten Klein
 * @version January 7, 2002
 */
public class StateTest extends TestCase {

    /**
     * Constructor description.
     *
     * @param name Parameter description.
     */
    public StateTest(String name) {
        super(name);
    }

    /**
     * A unit test for JUnit.
     */
    public void testAttributes() {
        State state = new State();
        Assert.assertTrue(!state.hasAttributes());
        Assert.assertTrue(!state.hasAttribute(Visible.class));
        Visible visible = (Visible) Attribute.getInstance(Visible.class);
        state.setAttribute(visible);
        Assert.assertTrue(state.hasAttributes());
        Assert.assertTrue(state.hasAttribute(Visible.class));
    }

    /**
     * A unit test for JUnit.
     */
    public void testPerformance() {
        State state = new State();
        Visible visible = (Visible) Attribute.getInstance(Visible.class);
        state.setAttribute(visible);
        long timer = System.currentTimeMillis();
        for (int i = 0; i < 1000; i++) {
            if (state.hasAttribute(Visible.class)) {
                state.getAttribute(Visible.class);
            }
        }
        timer = System.currentTimeMillis() - timer;
        Assert.assertTrue(timer < 1000);
    }
}
