/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.objects.visitors;

import java.util.Collection;
import java.util.Iterator;

import org.srs3d.viewer.objects.AbstractObject;
import org.srs3d.viewer.objects.ObjectContainer;

/**
 * This class implements the <code>vistor</code> interface for
 * <code>AbstractObjects</code> . It's specialized on processing objects that are
 * collections of other objects.
 *
 * @author Karsten Klein
 * @version July 11, 2001
 */
public abstract class AbstractVisitor implements Visitor {

    /**
     * Method description.
     *
     * @param object Parameter description.
     */
    public void visit(AbstractObject object) {
        if (object instanceof ObjectContainer) {
            visit((ObjectContainer) object);
        } else {
        }
    }

    /**
     * Iterates through the specified collection and visits each encountered object. The
     * object provided in the collection have to be instances of
     * <code>AbstractObject</code>. Missmatches are not checked and will cause the
     * default <code>ClassCastException</code>.
     *
     * @param collection Description of parameter.
     */
    public void visit(Collection collection) {
        Iterator iterator = collection.iterator();
        while (iterator.hasNext()) {
            visit(((AbstractObject) iterator.next()));
        }
    }

    /**
     * Visits an object container. All the elemnts in the container are visited as a
     * result.
     *
     * @param ObjectContainer The <code>ObjectContainer</code> to visit.
     */
    public void visit(ObjectContainer objectContainer) {
        visit(objectContainer.getObjects());
    }
}
