/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.objects.visitors;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;

import org.srs3d.viewer.j3d.ContextData;
import org.srs3d.viewer.j3d.commands.IdentificationCommand;
import org.srs3d.viewer.objects.AbstractObject;
import org.srs3d.viewer.objects.ObjectManager;
import org.srs3d.viewer.objects.StrategyManager;
import org.srs3d.viewer.util.Log;

/**
 * @author Karsten Klein
 * @version March 22, 2001
 */
public final class ObjectIdentifier extends AbstractVisitor {
    public static final Log log = new Log(ObjectIdentifier.class);
    public static final int ASSOCIATION_STRATEGY = 0;
    public static final int LOCALIZATION_STRATEGY = 1;
    public static final int OBJECT_TO_ID = 1;
    public static final int ID_TO_OBJECT = 2;
    private HashMap objectToIdCache = new HashMap();
    private AbstractObject object = null;
    private String objectId = null;
    private ContextData contextData;
    private Collection filterClasses = null;
    private int mode = OBJECT_TO_ID;
    private int strategy = ASSOCIATION_STRATEGY;

    /**
     * Constructor description.
     *
     * @param contextData Parameter description.
     */
    public ObjectIdentifier(ContextData contextData) {
        this.contextData = contextData;
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public final ContextData getContextData() {
        return contextData;
    }

    /**
     * Sets the <code>searchObject</code> attribute of the <code>ObjectLocalizer </code>
     * object.
     *
     * @param searchObject The new <code>searchObject</code> value.
     */
    public void setObject(AbstractObject object) {
        this.object = object;
    }

    /**
     * Method description.
     *
     * @param objectId Parameter description.
     */
    public void setObjectId(String objectId) {
        this.objectId = objectId;
    }

    /**
     * Method description.
     *
     * @param object Parameter description.
     */
    public void visit(AbstractObject object) {
        if (mode == OBJECT_TO_ID) {
            identifyObjectToId(object);
        } else {
            identifyIdToObject(object);
        }
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public String getObjectId() {
        return objectId;
    }

    /**
     * Method description.
     *
     * @param mode Parameter description.
     */
    public void setMode(int mode) {
        this.mode = mode;
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public AbstractObject getObject() {
        return object;
    }

    /**
     * Method description.
     *
     * @param object Parameter description.
     */
    public void identifyObjectToId(AbstractObject object) {
        if (strategy == ASSOCIATION_STRATEGY) {
            identifyObjectToId_A(object);
        } else {
            identifyObjectToId_B(object);
        }
    }

    /**
     * Method description.
     *
     * @param object Parameter description.
     */
    public void identifyObjectToId_A(AbstractObject object) {

        // first strategy; hopefully much faster than the second one
        // we use the associations to build the id
        ObjectManager objectManager = getContextData().getObjectManager();
        StrategyManager strategyManager = getContextData().getStrategyManager();
        ArrayList parents = new ArrayList();
        objectManager.getUpAssociations(getObject(), parents);
        if (parents.contains(object)) {

            // there must be a way from object to getObject()
            IdentificationCommand idCommand =
                new IdentificationCommand(getContextData(), parents);
            AbstractObject currentObject = getObject();
            ArrayList directUp = new ArrayList();
            HashSet localParents = new HashSet();
            Iterator iterator;
            boolean isValidWay;
            boolean found = false;
            objectId = "";
            while (!found) {

                //        if ( objectToIdCache.containsKey( currentObject ) ) {
                //
                //          if ( objectId.length() > 0 ) {
                //
                //            objectId += "|";
                //
                //          }
                //
                //          objectId += objectToIdCache.get( currentObject );
                //
                //          found = true;
                //
                //        }
                //        else {
                strategyManager.execute(currentObject, idCommand);
                if (idCommand.getObjectId() == null) {
                    objectId = null;
                    return;
                }
                if (objectId.length() > 0) {
                    objectId += "|";
                }
                objectId += idCommand.getObjectId();
                if (currentObject != object) {
                    directUp.clear();
                    objectManager.getDirectUpAssociations(currentObject,
                        directUp);
                    if (!directUp.isEmpty()) {
                        iterator = directUp.iterator();
                        isValidWay = false;
                        while (!isValidWay && iterator.hasNext()) {
                            currentObject = (AbstractObject) iterator.next();
                            objectManager.getUpAssociations(currentObject,
                                localParents);
                            if (localParents.contains(object)) {
                                isValidWay = true;
                            } else {
                                iterator.remove();
                            }
                        }
                    }
                } else {
                    found = true;
                }

                //        }
            }

            // fill cache with this identification
            //      String localId = objectId;
            //      String localObjectId = "";
            //      int index = localId.length();
            //
            //      iterator = parents.iterator();
            //
            //      while ( iterator.hasNext() ) {
            //
            //        object = (AbstractObject) iterator.next();
            //
            //        index = localId.lastIndexOf( "|" );
            //
            //        if ( index != -1 ) {
            //
            //          localObjectId = localId.substring( index + 1 ) + localObjectId;
            //          localId = localId.substring( 0, index );
            //
            //        }
            //
            //        objectToIdCache.put( object, localObjectId );
            //        localObjectId = "|" + localObjectId;
            //
            //      }
        }
    }

    /**
     * Method description.
     *
     * @param object Parameter description.
     */
    public void identifyObjectToId_B(AbstractObject object) {
        boolean isError = false;
        ObjectLocalizer objectLocalizer = new ObjectLocalizer();
        objectLocalizer.setSearchMode(ObjectLocalizer.SEARCH_FIRST);
        objectLocalizer.setSearchObject(this.object);
        objectLocalizer.setFilterClasses(filterClasses);
        objectLocalizer.visit(object);
        if (objectLocalizer.getObjects().size() > 0) {

            // create identifier from the returned sequence of objects
            ArrayList list = new ArrayList(objectLocalizer.getObjects());
            Iterator iterator = list.iterator();
            IdentificationCommand idCommand =
                new IdentificationCommand(getContextData(), list);
            StrategyManager strategyManager =
                getContextData().getStrategyManager();
            objectId = "";
            while (!isError && iterator.hasNext()) {
                object = (AbstractObject) iterator.next();
                strategyManager.execute(object, idCommand);
                if (idCommand.getObjectId() != null &&
                      idCommand.getObjectId().length() > 0) {
                    objectId += idCommand.getObjectId();
                    idCommand.setObjectId(null);
                    if (iterator.hasNext()) {
                        objectId += "|";
                    }
                } else {
                    isError = true;
                    objectId = null;
                    log.error("" + object + " not identifyable.");
                }
            }
        } else {

            // no object found in this search branch
            // there might be an chance to find it somewhere else
        }
    }

    /**
     * Method description.
     *
     * @param object Parameter description.
     */
    public void identifyIdToObject(AbstractObject object) {

        // use the object id to identify the appropriate object
        String subId;
        int index;
        boolean found = true;
        ArrayList objects = new ArrayList();
        ArrayList searchObjects = new ArrayList();
        searchObjects.add(object);
        StrategyManager strategyManager = getContextData().getStrategyManager();
        IdentificationCommand idCommand =
            new IdentificationCommand(getContextData(), objects);
        String localId = objectId;
        AbstractObject currentObject;
        boolean searchLevelSuccess;
        while (found && localId != null) {
            index = localId.lastIndexOf("|");
            if (index != -1) {
                subId = localId.substring(index + 1);
                localId = localId.substring(0, index);
            } else {
                subId = localId;
                localId = null;
            }
            Iterator iterator = searchObjects.iterator();
            searchLevelSuccess = false;

            // make a guess
            found = false;
            while (!searchLevelSuccess && iterator.hasNext()) {
                currentObject = (AbstractObject) iterator.next();
                strategyManager.execute(currentObject, idCommand);
                if (idCommand.getObjectId() != null) {
                    if (subId.compareToIgnoreCase(idCommand.getObjectId()) == 0) {
                        searchObjects.clear();
                        currentObject.getAllObjects(searchObjects);
                        objects.add(currentObject);
                        searchLevelSuccess = true;

                        // correct guess
                        found = true;
                        if (localId == null) {
                            this.object = currentObject;
                            objectId = "";
                        }
                    }
                }
            }
        }
    }

    /**
     * Method description.
     *
     * @param strategy Parameter description.
     */
    public void setStrategy(int strategy) {
        this.strategy = strategy;
    }
}
