/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.sequence;

import org.srs3d.viewer.annotation.objects.Alignment;
import org.srs3d.viewer.annotation.objects.AnnotationContainer;
import org.srs3d.viewer.annotation.objects.ChainAnnotation;
import org.srs3d.viewer.annotation.objects.Ruler;
import org.srs3d.viewer.annotation.objects.Segment;
import org.srs3d.viewer.j3d.ContextData;
import org.srs3d.viewer.j3d.attributes.Expanded;
import org.srs3d.viewer.j3d.attributes.Visible;
import org.srs3d.viewer.j3d.factories.GeometryCreatorFactory;
import org.srs3d.viewer.j3d.factories.SpawnerFactory;
import org.srs3d.viewer.objects.Attribute;
import org.srs3d.viewer.objects.ObjectContainer;
import org.srs3d.viewer.objects.State;
import org.srs3d.viewer.objects.StatePrototypeManager;
import org.srs3d.viewer.objects.StrategyManager;
import org.srs3d.viewer.sequence.creators.AlignmentGeometryCreator;
import org.srs3d.viewer.sequence.creators.ChainAnnotationGeometryCreator;
import org.srs3d.viewer.sequence.creators.SegmentGeometryCreator;
import org.srs3d.viewer.sequence.creators.SequenceGeometryCreator;
import org.srs3d.viewer.sequence.objects.strategies.ResidueStrategy;
import org.srs3d.viewer.sequence.objects.strategies.SegmentStrategy;

/**
 * Description of the class.
 *
 * @author Karsten Klein
 *
 * @created March 22, 2001
 */
public class Configuration {

    /**
     * Description of the method.
     *
     * @param contextData Description of parameter.
     */
    public static void registerGeometryCreators(ContextData contextData) {
        GeometryCreatorFactory factory =
            contextData.getGeometryCreatorFactory();
        factory.register(AnnotationContainer.class,
            SequenceGeometryCreator.class);
        factory.register(Alignment.class, AlignmentGeometryCreator.class);
        factory.register(ChainAnnotation.class,
            ChainAnnotationGeometryCreator.class);
        factory.register(Segment.class, SegmentGeometryCreator.class);
        factory.register(Ruler.class,
            org.srs3d.viewer.annotation.creators.RulerGeometryCreator.class);
    }

    /**
     * Description of the method.
     *
     * @param contextData Description of parameter.
     */
    public static void registerSpawners(ContextData contextData) {
        SpawnerFactory factory = contextData.getSpawnerFactory();
        factory.register(AnnotationContainer.class);
        factory.register(Alignment.class);
    }

    /**
     * Description of the method.
     *
     * @param contextData Description of parameter.
     */
    public static void registerStrategies(ContextData contextData) {
        StrategyManager manager = contextData.getStrategyManager();
        manager.register(AnnotationContainer.class,
            new org.srs3d.viewer.annotation.objects.strategies.AnnotationContainerStrategy());
        manager.register(Alignment.class, new SegmentStrategy());
        manager.register(Segment.class, new SegmentStrategy());
        manager.register(org.srs3d.viewer.bioatlas.objects.Residue.class,
            new ResidueStrategy());
    }

    /**
     * Description of the method.
     *
     * @param contextData Description of parameter.
     */
    public static void registerPrototypes(ContextData contextData) {
        StatePrototypeManager manager = contextData.getStatePrototypeManager();
        State visibleState = new State();
        visibleState.setAttribute(Attribute.getInstance(Visible.class));
        State expandedState = new State();
        expandedState.setAttribute(Attribute.getInstance(Visible.class));
        expandedState.setAttribute(Attribute.getInstance(Expanded.class));

        // expanded only objects
        manager.register(ObjectContainer.class, expandedState);
        manager.register(AnnotationContainer.class, expandedState);
        manager.register(Alignment.class, expandedState);
        manager.register(ChainAnnotation.class, visibleState);
        manager.register(Ruler.class, visibleState);
        manager.register(ChainAnnotation.class, expandedState);
        manager.register(Segment.class, visibleState);

        // visible and selectable objects
        manager.register(org.srs3d.viewer.bioatlas.objects.Residue.class,
            visibleState);
    }

    /**
     * Description of the method.
     *
     * @param contextData Description of parameter.
     */
    public static void configure(ContextData contextData) {
        org.srs3d.viewer.bioatlas.Configuration.registerPrototypes(contextData);
        registerGeometryCreators(contextData);
        registerSpawners(contextData);
        registerPrototypes(contextData);
        registerStrategies(contextData);
    }
}
