/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.sequence.objects.strategies;

import javax.vecmath.Color3f;

import org.srs3d.viewer.annotation.contexts.AnnotationContextData;
import org.srs3d.viewer.j3d.AppearanceManager;
import org.srs3d.viewer.j3d.AppearanceManager.TextureColors;
import org.srs3d.viewer.j3d.ContextData;
import org.srs3d.viewer.j3d.Reaction;
import org.srs3d.viewer.j3d.commands.ColorCommand;
import org.srs3d.viewer.j3d.objects.strategies.AbstractStrategy;
import org.srs3d.viewer.j3d.objects.strategies.reactions.EmptyReaction;
import org.srs3d.viewer.objects.AbstractObject;
import org.srs3d.viewer.objects.Command;

/**
 * @author Karsten Klein
 *
 * @created April 03, 2001
 */
public class ResidueStrategy extends AbstractStrategy {

    /**
     * Constructor description.
     */
    public ResidueStrategy() {

        // :TEST: texture coloring
        register(ColorCommand.class, EmptyReaction.getSharedInstance());
    }

    /**
     * Class description.
     *
     * @author Karsten Klein, LION bioscience AG
     */
    public class ColorReaction implements Reaction {

        /**
         * Method description.
         *
         * @param command Parameter description.
         */
        public void execute(Command command) {
            ColorCommand colorCommand = (ColorCommand) command;
            AbstractObject object = (AbstractObject) colorCommand.getObject();
            colorCommand.execute();
            ContextData contextData = colorCommand.getContextData();
            ContextData referenceContextData =
                ((AnnotationContextData) contextData).getReferenceContextData();
            AppearanceManager appearanceManager =
                referenceContextData.getAppearanceManager();
            TextureColors textureColors =
                appearanceManager.getTextureColors(object);
            if (textureColors != null) {

                //        Appearance appearance = appearanceManager.getAppearance( object );
                //        Color3f color = AppearanceHelper.getColor( appearance );
                textureColors.setColor(0, new Color3f(1, 1, 1));

                //        updateTexture( colorCommand, textureColors );
            }
        }
    }
}
