/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.sequence.objects.strategies;

import javax.vecmath.Point3d;

import org.srs3d.viewer.annotation.attributes.LayoutPosition;
import org.srs3d.viewer.annotation.objects.Segment;
import org.srs3d.viewer.bioatlas.objects.Residue;
import org.srs3d.viewer.j3d.ContextData;
import org.srs3d.viewer.j3d.Reaction;
import org.srs3d.viewer.j3d.commands.IntersectCommand;
import org.srs3d.viewer.objects.AbstractObject;
import org.srs3d.viewer.objects.Command;
import org.srs3d.viewer.objects.State;

/**
 * @author Karsten Klein
 *
 * @created April 03, 2001
 */
public class SegmentStrategy
    extends org.srs3d.viewer.annotation.objects.strategies.SegmentStrategy {

    /**
     * Constructor description.
     */
    public SegmentStrategy() {

        // note that there are registrations done in the superclass constructor
        // here we simply add specific registrations
        register(IntersectCommand.class, new IntersectReaction());
    }

    /**
     * Class description.
     *
     * @author Karsten Klein, LION bioscience AG
     */
    public class IntersectReaction implements Reaction {

        /**
         * Method description.
         *
         * @param command Parameter description.
         */
        public void execute(Command command) {
            IntersectCommand intersectCommand = (IntersectCommand) command;
            ContextData contextData = intersectCommand.getContextData();
            AbstractObject object = intersectCommand.getObject();

            // we have to find the residue that was picked
            State.Immutable state =
                contextData.getStateManager().getImmutableState(object);
            LayoutPosition.Immutable position =
                (LayoutPosition.Immutable) state.getAttribute(LayoutPosition.class);
            float x = 0;
            if (position != null) {
                x = position.getPosition().x;
            }
            if (intersectCommand.getPickResult().numIntersections() > 0) {
                Point3d point =
                    intersectCommand.getPickResult().getIntersection(0)
                                    .getPointCoordinates();
                x = (float) point.x - x;
                int i = (int) (x + 0.5f);
                i -= ((Segment) object).getStartIndex();
                Residue residue = ((Segment) object).getInitialResidue();
                while (i > 0 && residue != null) {
                    i--;
                    residue = residue.getProceeding();
                }
                intersectCommand.setObject(residue);
            } else {
                intersectCommand.setObject(((Segment) object).getInitialResidue());
            }
        }
    }
}
