/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.sequence.visitors;

import java.util.Iterator;

import org.srs3d.viewer.annotation.objects.Alignment;
import org.srs3d.viewer.annotation.objects.ChainAnnotation;
import org.srs3d.viewer.annotation.objects.Ruler;
import org.srs3d.viewer.annotation.visitors.AbstractAlignmentLayoutCreator;
import org.srs3d.viewer.bioatlas.Parameter;

/**
 * Description of the class.
 *
 * @author Christian Zofka
 *
 * @created December 20, 2001
 */
public class AlignmentLayoutCreator extends AbstractAlignmentLayoutCreator {

    /**
     * <code>AlignmentLayoutCreator</code> constructor.
     *
     * @param alignment Description of parameter.
     * @param position Description of parameter.
     */
    public AlignmentLayoutCreator(Alignment alignment, float position) {
        super(alignment, position);
    }

    /**
     * Description of the method.
     */
    protected void createDefault() {
        int size = alignment.getChainAnnotationCount();
        int i = 0;
        Iterator iterator;
        rows = new Object[size];
        rowYPos = new float[size];
        ChainAnnotation chainAnnotation;
        boolean first = true;
        iterator = alignment.getChainAnnotations().iterator();
        while (iterator.hasNext()) {
            chainAnnotation = (ChainAnnotation) iterator.next();
            if (chainAnnotation != null && first) {
                first = false;
            }

            // chainAnnotations
            insert(chainAnnotation, Parameter.chainAnnotationHeight);
        }
    }

    /**
     * creates bar-scheme: 1) all annotations 2) all chains
     */
    protected void createChainsAtBottom() {

        // all objects (annotationUnits + chainAnnotations + 2 rulers)
        int size = 0;
        size = alignment.getLayers().size();
        int i = 0;
        Iterator iterator;
        rows = new Object[size + 2];
        rowYPos = new float[size + 2];
        Object[] chainAnnotations = alignment.getChainAnnotations().toArray();
        insertRulers(alignment.getFirst(), Ruler.VERTICALS_DOWN);
        for (int j = 0; j < size; j++) {

            // chainAnnotations
            insert(getChainAnnotationAt(j), Parameter.chainAnnotationHeight);
        }

        // rulers
        insertRulers(getChainAnnotationAt(size - 1), Ruler.VERTICALS_UP);
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public float getRulerHeight() {
        return Parameter.sequenceRulerHeight;
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public boolean isRulerLabeled() {
        return true;
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public boolean isDetailRuler() {
        return true;
    }
}
