/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.swing;

import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import org.srs3d.viewer.j3d.ContextData;
import org.srs3d.viewer.j3d.Contextual;
import org.srs3d.viewer.objects.AbstractObject;

/**
 * Description of the class.
 *
 * @author Karsten Klein
 *
 * @created May 18, 2001
 */
public class AbstractObjectTreeModel implements TreeModel, Contextual {
    private Map objectMap = new Hashtable();
    private AbstractObject object = null;
    private ContextData contextData;
    private Vector listeners = new Vector();

    /**
     * <code>AbstractObjectTreeModel</code> constructor.
     *
     * @param object Description of parameter.
     */
    public AbstractObjectTreeModel(ContextData contextData,
        AbstractObject object) {
        this.contextData = contextData;
        this.object = object;
        updateData(object);
    }

    /**
     * Gets the <code>child</code> attribute of the <code>AbstractObjectTreeModel </code>
     * object.
     *
     * @param parent Description of parameter.
     * @param index Description of parameter.
     *
     * @return The <code>child</code> value.
     */
    public Object getChild(Object parent, int index) {
        Vector vector = getVector((AbstractObject) parent);
        return vector.elementAt(index);
    }

    /**
     * Gets the <code>vector</code> attribute of the <code>AbstractObjectTreeModel</code>
     * object.
     *
     * @param object Description of parameter.
     *
     * @return The <code>vector</code> value.
     */
    public Vector getVector(Object object) {
        return (Vector) objectMap.get(object);
    }

    /**
     * Gets the <code>childCount</code> attribute of the
     * <code>AbstractObjectTreeModel</code> object.
     *
     * @param parent Description of parameter.
     *
     * @return The <code>childCount</code> value.
     */
    public int getChildCount(Object parent) {
        Vector vector = getVector((AbstractObject) parent);
        return vector.size();
    }

    /**
     * Gets the <code>indexOfChild</code> attribute of the
     * <code>AbstractObjectTreeModel</code> object.
     *
     * @param parent Description of parameter.
     * @param child Description of parameter.
     *
     * @return The <code>indexOfChild</code> value.
     */
    public int getIndexOfChild(Object parent, Object child) {
        Vector vector = getVector((AbstractObject) parent);
        return vector.indexOf(child);
    }

    /**
     * Gets the <code>root</code> attribute of the <code>AbstractObjectTreeModel </code>
     * object.
     *
     * @return The <code>root</code> value.
     */
    public Object getRoot() {
        return object;
    }

    /**
     * Gets the <code>leaf</code> attribute of the <code>AbstractObjectTreeModel </code>
     * object.
     *
     * @param node Description of parameter.
     *
     * @return The <code>leaf</code> value.
     */
    public boolean isLeaf(Object node) {

        //    State state = getContextData().getStateManager().getState( ( AbstractObject ) node );
        //    return !state.hasAttribute( Expanded.class ) || ( getChildCount( node ) == 0 );
        return getChildCount(node) == 0;
    }

    /**
     * Adds a <code>TreeModelListener</code> object to the
     * <code>AbstractObjectTreeModel</code> object.
     *
     * @param listener The <code>TreeModelListener</code> object to be added.
     */
    public void addTreeModelListener(TreeModelListener listener) {
        listeners.add(listener);
    }

    /**
     * Description of the method.
     *
     * @param listener Description of parameter.
     */
    public void removeTreeModelListener(TreeModelListener listener) {
        listeners.remove(listener);
    }

    /**
     * Description of the method.
     *
     * @param path Description of parameter.
     * @param newValue Description of parameter.
     */
    public void valueForPathChanged(TreePath path, Object newValue) {
    }

    /**
     * Description of the method.
     *
     * @param object Description of parameter.
     */
    public void updateData(AbstractObject object) {
        objectMap.clear();
        add(object);
    }

    /**
     * Description of the method.
     *
     * @param object Description of parameter.
     */
    public void add(AbstractObject object) {
        Vector vector = new Vector();
        objectMap.put(object, vector);
        Collection set = new Vector();
        object.getAllObjects(set);
        vector.addAll(set);
        Iterator iterator = set.iterator();
        while (iterator.hasNext()) {
            add((AbstractObject) iterator.next());
        }
    }

    /**
     * Description of the method.
     *
     * @param object Description of parameter.
     */
    public void update(AbstractObject object) {
        add(object);
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public final ContextData getContextData() {
        return contextData;
    }
}
