/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.swing;

import java.awt.Component;

import javax.swing.Icon;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

import org.srs3d.viewer.objects.Computation;
import org.srs3d.viewer.util.ExceptionHandler;

/**
 * @author Karsten Klein
 *
 * @created August 22, 2001
 */
public class ComponentFactory {
    public static final int TIME_TO_OPEN_PROGRESS_DISPLAY = 2000;

    /**
     * Description of the method.
     *
     * @param message Description of parameter.
     */
    public static void modalMessageBox(String message) {
        modalMessageBox(message, null);
    }

    /**
     * Description of the method.
     *
     * @param message Description of parameter.
     * @param component Description of parameter.
     */
    public static void modalMessageBox(final String message,
        final Component component) {
        try {

            // we need this to ensure the dialog is closed properly
            SwingUtilities.invokeAndWait(new Runnable() {
                    public void run() {
                        SwingSettings.update();
                        JOptionPane optionPane = new JOptionPane(message);
                        String[] options = { "OK" };
                        optionPane.setOptions(options);
                        JDialog dialog =
                            optionPane.createDialog(component,
                                org.srs3d.viewer.bioatlas.Parameter.applicationName +
                                " message");
                        dialog.setVisible(true);
                        dialog.setResizable(false);
                    }
                });
        } catch (Exception e) {
            ExceptionHandler.handleException(e, ExceptionHandler.SILENT_IN_DEBUG);
        }
    }

    /**
     * Method description.
     *
     * @param component Parameter description.
     * @param message Parameter description.
     * @param title Parameter description.
     * @param messageType Parameter description.
     * @param icon Parameter description.
     */
    public static void messageDialog(Component component, String message,
        String title, int messageType, Icon icon) {
        SwingSettings.update();
        title =
            org.srs3d.viewer.bioatlas.Parameter.applicationName + " - " +
            title;
        if (icon != null) {
            JOptionPane.showMessageDialog(component, message, title,
                messageType, icon);
        } else {
            JOptionPane.showMessageDialog(component, message, title, messageType);
        }
        component.requestFocus();
    }

    /**
     * Method description.
     *
     * @param component Parameter description.
     * @param message Parameter description.
     * @param title Parameter description.
     * @param messageType Parameter description.
     * @param icon Parameter description.
     *
     * @return Return description.
     */
    public static int confirmDialog(Component component, String message,
        String title, int messageType, Icon icon) {
        SwingSettings.update();
        title =
            org.srs3d.viewer.bioatlas.Parameter.applicationName + " - " +
            title;
        int result = JOptionPane.CANCEL_OPTION;
        if (icon != null) {
            result =
                JOptionPane.showConfirmDialog(component, message, title,
                    JOptionPane.OK_CANCEL_OPTION, messageType, icon);
        } else {
            result =
                JOptionPane.showConfirmDialog(component, message, title,
                    JOptionPane.OK_CANCEL_OPTION, messageType);
        }
        component.requestFocus();
        return result;
    }

    /**
     * Description of the method.
     *
     * @param message Description of parameter.
     * @param component Description of parameter.
     * @param computation Description of parameter.
     */
    public static void progressDialog(String message,
        final Component component, final Computation computation,
        final org.srs3d.viewer.j3d.Context context) {
        SwingSettings.update();
        final ProgressPane progressPane = new ProgressPane();
        JOptionPane optionPane = new JOptionPane();
        final JDialog dialog =
            optionPane.createDialog(component,
                org.srs3d.viewer.bioatlas.Parameter.applicationName +
                " - Progress");
        dialog.getContentPane().removeAll();
        dialog.getContentPane().add(progressPane);
        final ProgressDisplay progressDisplay =
            new ProgressDisplay() {
                public void setProgress(final int progress) {
                    javax.swing.SwingUtilities.invokeLater(new Runnable() {
                            public void run() {
                                progressPane.setProgress(progress);
                            }
                        });
                }

                /**
                 * Method description.
                 *
                 * @param description Parameter description.
                 */
                public void setDescription(final String description) {
                    javax.swing.SwingUtilities.invokeLater(new Runnable() {
                            public void run() {
                                progressPane.setDescription(description);
                            }
                        });
                }

                /**
                 * Method description.
                 *
                 * @param isCancelEnabled Parameter description.
                 */
                public void setCancelEnabled(final boolean isCancelEnabled) {
                    javax.swing.SwingUtilities.invokeLater(new Runnable() {
                            public void run() {
                                progressPane.setCancelEnabled(isCancelEnabled);
                            }
                        });
                }

                /**
                 * Method description.
                 */
                public void start() {
                }

                /**
                 * Method description.
                 *
                 * @return Return description.
                 */
                public boolean hasCanceled() {
                    boolean hasCanceled =
                        progressPane.isCanceled() || computation.hasFinished();
                    if (hasCanceled) {

                        // close if not already closed
                        close();
                    }
                    return hasCanceled;
                }

                /**
                 * Method description.
                 */
                public void close() {
                    try {

                        // changed for build3 (we use invokeAndWait to ensure the dialog is closed)
                        javax.swing.SwingUtilities.invokeAndWait(new Runnable() {
                                public void run() {
                                    dialog.dispose();
                                }
                            });
                    } catch (Exception e) {
                        ExceptionHandler.handleException(e,
                            ExceptionHandler.SILENT_IN_DEBUG);
                    }
                    context.requestFocus();
                }
            };
        final Runnable displayProgress =
            new Runnable() {
                public void run() {
                    SwingSettings.update();

                    // if the computation has not completed yet
                    if (!computation.hasFinished()) {

                        // start a progress update thread
                        ProgressThread progressThread =
                            new ProgressThread(progressDisplay, computation);
                        progressThread.start();

                        // :FIXME: doing it this way looses the icon; removing the resize
                        // display the dialog
                        dialog.setResizable(false);
                        dialog.show();
                        context.requestFocus();
                    }
                }
            };
        try {
            new Thread(new Runnable() {
                    public void run() {

                        // wait a moment
                        try {
                            Thread.sleep(TIME_TO_OPEN_PROGRESS_DISPLAY);
                        } catch (InterruptedException e) {

                            // :SILENT EXCEPTION:
                        }
                        SwingUtilities.invokeLater(displayProgress);
                    }
                }).start();
        } catch (Exception e) {
            context.requestFocus();
            ExceptionHandler.handleException(e, ExceptionHandler.SILENT_IN_DEBUG);
        }
    }
}
