/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.swing;

import java.awt.BorderLayout;
import java.awt.Graphics;
import java.awt.LayoutManager;
import java.awt.image.BufferedImage;

import javax.media.j3d.Canvas3D;
import javax.media.j3d.ImageComponent2D;
import javax.media.j3d.Screen3D;
import javax.media.j3d.VirtualUniverse;
import javax.swing.JPanel;

import org.srs3d.viewer.j3d.Context;

/**
 * @author Karsten Klein
 *
 * @created May 14, 2001
 */
public class ContextPanel extends JPanel {
    private Canvas3D context = null;
    private transient ImageComponent2D image = null;

    /**
     * <code>ContextPanel</code> constructor.
     *
     * @param context Description of parameter.
     */
    public ContextPanel(Canvas3D context) {
        super();
        setup(context);
    }

    /**
     * <code>ContextPanel</code> constructor.
     *
     * @param context Description of parameter.
     * @param layoutManager Description of parameter.
     */
    public ContextPanel(Context context, LayoutManager layoutManager) {
        super(layoutManager);
        setup(context);
    }

    /**
     * <code>ContextPanel</code> constructor.
     *
     * @param context Description of parameter.
     * @param isDoubleBuffered Description of parameter.
     */
    public ContextPanel(Context context, boolean isDoubleBuffered) {
        super(isDoubleBuffered);
        setup(context);
    }

    /**
     * <code>ContextPanel</code> constructor.
     *
     * @param context Description of parameter.
     * @param layoutManager Description of parameter.
     * @param isDoubleBuffered Description of parameter.
     */
    public ContextPanel(Context context, LayoutManager layoutManager,
        boolean isDoubleBuffered) {
        super(layoutManager, isDoubleBuffered);
        setup(context);
    }

    private final boolean setup(Canvas3D context) {
        this.context = context;
        int width = 200;
        int height = 200;
        if (context.isOffScreen()) {
            if (getWidth() > 0 && getHeight() > 0) {
                if (image == null || image.getImage().getWidth() != width ||
                      image.getImage().getHeight() != height) {
                    context.setSize(width, height);
                    Screen3D screen = context.getScreen3D();
                    screen.setSize(width, height);
                    screen.setPhysicalScreenWidth(0.33866666);
                    screen.setPhysicalScreenHeight(0.45155555);
                    BufferedImage buffer =
                        new BufferedImage(getWidth(), getHeight(),
                            BufferedImage.TYPE_INT_ARGB);
                    image =
                        new ImageComponent2D(ImageComponent2D.FORMAT_RGBA,
                            buffer);
                    image.setCapability(ImageComponent2D.ALLOW_IMAGE_READ);
                    context.setOffScreenBuffer(image);
                    VirtualUniverse.setJ3DThreadPriority(Thread.currentThread()
                                                               .getPriority());
                    Thread.currentThread().setPriority(Thread.currentThread()
                                                             .getPriority() -
                        1);
                    int priority = Thread.currentThread().getPriority();
                    Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
                    context.renderOffScreenBuffer();
                    context.waitForOffScreenRendering();
                    Thread.currentThread().setPriority(priority);
                    return true;
                }
            }
        } else {
            setLayout(new BorderLayout());
            add("Center", context);
        }
        return false;
    }

    /**
     * Method description.
     *
     * @param graphics Parameter description.
     */
    public void paint(Graphics graphics) {
        super.paint(graphics);
        if (context.isOffScreen()) {
            if (setup(context)) {
                graphics.drawImage(context.getOffScreenBuffer().getImage(), 0,
                    0, this);
            }
        }
    }
}
