/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.swing;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.net.URL;
import java.util.Hashtable;
import java.util.Map;

import javax.swing.ImageIcon;

/**
 * @author Karsten Klein
 *
 * @created September 14, 2001
 */
public class CursorManager {
    private static Map cursorMap = new Hashtable();
    private static Map componentCursorMap = new Hashtable();

    /**
     * Gets the <code>cursor</code> attribute of the <code>CursorManager</code> class.
     *
     * @param object Description of parameter.
     *
     * @return The <code>cursor</code> value.
     */
    public static Cursor getCursor(Object object) {
        return (Cursor) cursorMap.get(object);
    }

    /**
     * Description of the method.
     *
     * @param cursorName Description of parameter.
     * @param fileName Description of parameter.
     * @param hotSpot Description of parameter.
     */
    public static void createCursor(String cursorName, String fileName,
        Point hotSpot) {
        Image image;
        image = new ImageIcon(fileName).getImage();
        Cursor cursor =
            Toolkit.getDefaultToolkit().createCustomCursor(image, hotSpot,
                cursorName);
        cursorMap.put(cursorName, cursor);
    }

    /**
     * Description of the method.
     *
     * @param cursorName Description of parameter.
     * @param url Description of parameter.
     * @param hotSpot Description of parameter.
     */
    public static void createCursor(String cursorName, URL url, Point hotSpot) {
        Image image;
        image = new ImageIcon(url).getImage();
        Cursor cursor =
            Toolkit.getDefaultToolkit().createCustomCursor(image, hotSpot,
                cursorName);
        cursorMap.put(cursorName, cursor);
    }

    /**
     * Method description.
     *
     * @param cursorName Parameter description.
     * @param cursor Parameter description.
     */
    public static void createCursor(String cursorName, Cursor cursor) {
        cursorMap.put(cursorName, cursor);
    }

    /**
     * Method description.
     *
     * @param component Parameter description.
     */
    public static void setWaitCursor(Component component) {
        if (component != null) {
            componentCursorMap.put(component, component.getCursor());
            component.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        }
    }

    /**
     * Method description.
     *
     * @param component Parameter description.
     */
    public static void restoreCursor(Component component) {
        if (component != null) {
            Cursor cursor = (Cursor) componentCursorMap.remove(component);
            if (cursor != null) {
                component.setCursor(cursor);
            }

            //      else {
            //
            //        component.setCursor(Cursor.getDefaultCursor());
            //
            //      }
        }
    }

    /**
     * Method description.
     *
     * @param component Parameter description.
     * @param cursor Parameter description.
     */
    public static void setCursor(Component component, Cursor cursor) {
        Cursor storedCursor = (Cursor) componentCursorMap.get(component);
        if (storedCursor == null) {

            // directly display the cursor
            component.setCursor(cursor);
        } else {

            // currently in wait state, just update the map
            componentCursorMap.put(component, cursor);
        }
    }
}
