/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.swing;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.StringReader;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.StyledDocument;

import org.srs3d.viewer.util.Log;

/**
 * Class description.
 *
 * @author Karsten Klein, LION bioscience AG
 */
public class JConsole extends JTextPane {
    private static final Log log = new Log(JConsole.class);
    private JScrollPane scrollPane = null;
    private StyledDocument document = null;
    private AttributeSet attributeSet = null;
    private Object interpreter = null;

    /**
     * Constructor description.
     */
    public JConsole() {
        super();
        scrollPane = new JScrollPane(this);
        scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        scrollPane.setPreferredSize(new Dimension(300, 200));
        document = getStyledDocument();
        attributeSet = getCharacterAttributes();
        setAutoscrolls(true);
        scrollPane.setAutoscrolls(true);
    }

    /**
     * Method description.
     *
     * @param string Parameter description.
     */
    public void echo(String string) {
        try {
            int endOffset = document.getEndPosition().getOffset() - 1;
            string = new String(string + "\r\n");
            document.insertString(endOffset, string, attributeSet);
            endOffset = document.getEndPosition().getOffset() - 1;
            setCaretPosition(endOffset);
            int pos = document.getLength();
            setSelectionStart(pos);
            setSelectionEnd(pos);
        } catch (BadLocationException e) {
            log.debug(e, e);
        }
    }

    /**
     * Method description.
     *
     * @param string Parameter description.
     */
    public void execute(String string) {
        try {
            int endOffset = document.getEndPosition().getOffset() - 1;
            string = new String("EXEC> " + string + "\r\n");
            document.insertString(endOffset, string, attributeSet);
            endOffset = document.getEndPosition().getOffset() - 1;
            setCaretPosition(endOffset);
            int pos = document.getLength();
            setSelectionStart(pos);
            setSelectionEnd(pos);
        } catch (BadLocationException e) {
            log.debug(e, e);
        }
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public JScrollPane getScrollPane() {
        return scrollPane;
    }

    /**
     * Method description.
     */
    public void showInFrame() {
        JFrame frame =
            new JFrame(org.srs3d.viewer.bioatlas.Parameter.applicationName +
                " - Console");
        frame.setSize(300, 100);
        frame.getContentPane().add(this, BorderLayout.CENTER);
        frame.setVisible(true);
        final JConsole console = this;

        // creates the button to modify
        JButton button = new JButton("Execute");
        button.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    interpret(console.getText());
                }
            });
        frame.getContentPane().add(button, BorderLayout.SOUTH);
        interpret("import javax.media.j3d.*;");
        interpret("import javax.vecmath.*;");
        interpret("import org.srs3d.viewer.j3d.*;");
        interpret("import org.srs3d.viewer.objects.*;");
        interpret("import org.srs3d.viewer.bioatlas.*;");
        interpret("import org.srs3d.viewer.bioatlas.objects.*;");
        interpret("import org.srs3d.viewer.bioatlas.visitors.*;");
        interpret("import org.srs3d.viewer.bioatlas.colorschemes.*;");
        interpret("import java.lang.*;");
        interpret("import java.util.*;");
        interpret(
            "SecondaryStructureColorScheme scs = new SecondaryStructureColorScheme();");
        interpret(
            "FirstImpressionColorScheme fics = new FirstImpressionColorScheme();");
        interpret("Color3f color = null;");
        interpret("Appearance appearance = null;");
        interpret("Appearance appearance = null;");
        interpret("Iterator iterator = null;");
        interpret("ObjectFinder objf = null;");
    }

    /**
     * Evaluates the given string. The errors are printed on the standard error stream.
     *
     * @param s the string to evaluate
     */
    public void interpret(String s) {
        try {
            if (interpreter == null) {
                interpreter =
                    new koala.dynamicjava.interpreter.TreeInterpreter(new koala.dynamicjava.parser.wrapper.JavaCCParserFactory());
                interpret("import java.awt.*;");
            }
            ((koala.dynamicjava.interpreter.Interpreter) interpreter).interpret(new StringReader(
                    s), "topLevel");
        } catch (Exception e) {
            log.debug(e, e);
        }
    }
}
