/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.swing;

import java.awt.Component;
import java.awt.event.ActionListener;
import java.util.Map;

/**
 * <code>Module</code> interface. Modules are used for menus and popup items. These
 * classes implement their appearance as Component in the menu and define the action to
 * be performed on activation.
 *
 * @author Karsten Klein
 *
 * @created May 14, 2001
 */
public interface Module extends ActionListener {

    /**
     * Gets the <code>Component</code> that represents the module in a menu.
     *
     * @return The <code>Component</code> .
     */
    public Component getComponent();

    /**
     * Adds a <code>ActionListener</code> to the <code>Module</code> object.
     *
     * @param actionListener The <code>ActionListener</code> object to be added.
     */
    public void addActionListener(ActionListener actionListener);

    /**
     * Removes a <code>ActionListener</code> from the <code>Module</code> objects
     * listener list.
     *
     * @param actionListener Description of parameter.
     */
    public void removeActionListener(ActionListener actionListener);

    /**
     * Cleanup method for the module. The module will never be used again and therefore
     * should assure all its data can be carbage collected.
     */
    public void cleanup();

    /**
     * The module should do an update if it's a context sensitive and or a dynamic
     * component that can remove/gray itself.
     */
    public void update();

    /**
     * Method description.
     *
     * @return Return description.
     */
    public String getName();

    //  public String createReport();
    public void createReport(Map map);
}
