/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.swing;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import javax.swing.JMenu;
import javax.swing.JPopupMenu;

import org.srs3d.viewer.bioatlas.PopupContext;

/**
 * Customized popup menu. This implementations gathers the modules that finally make up
 * the popup menu. Note that this is basically the datamodel for a JPopupMenu.
 *
 * @author Karsten Klein
 *
 * @created May 14, 2001
 */
public class PopupMenu extends Vector implements Module {
    private static boolean isFirstPopup = true;
    private JPopupMenu popupMenu = null;
    private Component parentComponent = null;
    private PopupMouseAdapter mouseListener = null;

    /**
     * Constructor description.
     *
     * @param parentComponent Parameter description.
     * @param popupContext Parameter description.
     */
    public PopupMenu(Component parentComponent, PopupContext popupContext) {
        this.parentComponent = parentComponent;
        mouseListener = getMouseListener(parentComponent, popupContext);
    }

    /**
     * Gets the <code>component</code> attribute of the <code>PopupMenu</code> object.
     *
     * @return The <code>component</code> value.
     */
    public Component getComponent() {
        create();
        return popupMenu;
    }

    /**
     * Method description.
     *
     * @param e Parameter description.
     */
    public void actionPerformed(ActionEvent e) {
    }

    /**
     * Method description.
     *
     * @param actionListener Parameter description.
     */
    public void addActionListener(ActionListener actionListener) {
    }

    /**
     * Method description.
     *
     * @param actionListener Parameter description.
     */
    public void removeActionListener(ActionListener actionListener) {
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public PopupMouseAdapter getMouseListener() {
        return mouseListener;
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public Module getLastModule() {
        return (Module) lastElement();
    }

    /**
     * Method description.
     */
    public void clear() {
        cleanupItems();
        super.clear();
        popupMenu = null;
        isFirstPopup = true;
    }

    /**
     * Method description.
     */
    public void cleanup() {
        clear();
    }

    /**
     * Method description.
     */
    public void cleanupItems() {
        Iterator iterator = iterator();
        while (iterator.hasNext()) {
            ((Module) iterator.next()).cleanup();
        }
    }

    /**
     * Method description.
     */
    public void create() {

        // create menu if not existing
        if (popupMenu == null) {
            popupMenu = new JPopupMenu();
            popupMenu.setLightWeightPopupEnabled(false);
            JPopupMenu.setDefaultLightWeightPopupEnabled(false);
            isFirstPopup = true;
        }
    }

    /**
     * Method description.
     */
    public void update() {
        if (popupMenu != null) {
            Iterator iterator;
            Module module;

            // remove all entries
            popupMenu.removeAll();
            SwingSettings.update();

            // enter all entries (the order can have changed)
            iterator = iterator();
            while (iterator.hasNext()) {
                module = (Module) iterator.next();
                popupMenu.add(module.getComponent());
            }

            // update all entries
            iterator = iterator();
            while (iterator.hasNext()) {
                module = (Module) iterator.next();
                module.update();
            }
        }
    }

    /**
     * Method description.
     *
     * @param component Parameter description.
     * @param popupContext Parameter description.
     *
     * @return Return description.
     */
    public PopupMouseAdapter getMouseListener(Component component,
        PopupContext popupContext) {
        return new PopupMouseAdapter(component, popupContext);
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public String getName() {
        return "Popupmenu";
    }

    private Point computeLocation(int x, int y, Component component,
        JPopupMenu popupMenu) {
        Dimension validOrigin = Toolkit.getDefaultToolkit().getScreenSize();
        validOrigin.width -= component.getLocationOnScreen().x;
        validOrigin.height -= component.getLocationOnScreen().y;
        if (popupMenu.getSize().width > 0) {
            validOrigin.width -= popupMenu.getSize().width;
            validOrigin.height -= popupMenu.getSize().height;
        } else {
            return null;
        }
        if (x > validOrigin.getWidth()) {
            x = (int) validOrigin.getWidth();
        }
        if (y > validOrigin.getHeight()) {
            y = (int) validOrigin.getHeight();
        }
        return new Point(x, y);
    }

    /**
     * Method description.
     *
     * @param map Parameter description.
     */
    public void createReport(Map map) {
        update();
        Iterator iterator = iterator();
        while (iterator.hasNext()) {
            ((Module) iterator.next()).createReport(map);
        }
    }

    /**
     * Method description.
     *
     * @param name Parameter description.
     *
     * @return Return description.
     */
    public JMenu updateMenu(String name) {
        Iterator iterator;
        Module module;
        JMenu menu = new JMenu(name);

        // enter all entries (the order may have changed)
        iterator = iterator();
        while (iterator.hasNext()) {
            module = (Module) iterator.next();
            if (module.getName().equals(name)) {
                menu.add(module.getComponent());
                module.update();
                if (module instanceof Menu) {
                    ((Menu) module).updateEntries();
                }
                return (JMenu) module.getComponent();
            }
        }
        return null;
    }

    /**
     * Mouse adapter implemetation for poping up the menu.
     *
     * @author Karsten Klein
     *
     * @created March 20, 2001
     */
    public class PopupMouseAdapter extends MouseAdapter
        implements MouseMotionListener {
        private Component component;
        private PopupContext popupContext;
        private boolean isPopupTrigger = false;

        /**
         * Constructor description.
         *
         * @param component Parameter description.
         * @param popupContext Parameter description.
         */
        public PopupMouseAdapter(Component component, PopupContext popupContext) {
            this.component = component;
            this.popupContext = popupContext;
        }

        /**
         * Method description.
         *
         * @param e Parameter description.
         */
        public void mousePressed(MouseEvent e) {

            // potential popup trigger
            isPopupTrigger = true;
        }

        /**
         * Method description.
         *
         * @param e Parameter description.
         */
        public void mouseDragged(MouseEvent e) {
            isPopupTrigger = false;
        }

        /**
         * Method description.
         *
         * @param e Parameter description.
         */
        public void mouseEntered(MouseEvent e) {
            isPopupTrigger = false;
        }

        /**
         * Method description.
         *
         * @param e Parameter description.
         */
        public void mouseExited(MouseEvent e) {
            isPopupTrigger = false;
        }

        /**
         * Method description.
         *
         * @param e Parameter description.
         */
        public void mouseMoved(MouseEvent e) {

            // do not reset popuptrigger here
        }

        /**
         * Right mouse click triggers the popup;
         *
         * @param e Description of parameter
         */
        public void mouseReleased(MouseEvent e) {
            if (isPopupTrigger) {
                if (e.getClickCount() == 1) {
                    if (!e.isShiftDown()) {
                        if (!e.isControlDown()) {
                            if (!e.isAltDown()) {
                                if (!e.isAltGraphDown()) {
                                    if ((e.getModifiers() &
                                          MouseEvent.BUTTON3_MASK) != 0) {
                                        CursorManager.setWaitCursor((Component) e.getSource());
                                        create();
                                        update();
                                        Point location =
                                            computeLocation(e.getX(), e.getY(),
                                                component, popupMenu);
                                        Point mouseLocation = e.getPoint();
                                        if (location == null) {
                                            popupMenu.show(component,
                                                mouseLocation.x, mouseLocation.y);
                                            popupMenu.setVisible(true);

                                            // open first time (computes the correct dimension of the popup)
                                            location =
                                                computeLocation(e.getX(),
                                                    e.getY(), component,
                                                    popupMenu);
                                            if (location != null) {
                                                if (!location.equals(
                                                          mouseLocation)) {

                                                    // readjust
                                                    popupMenu.show(component,
                                                        location.x, location.y);
                                                }
                                            }
                                        } else {
                                            popupMenu.show(component,
                                                location.x, location.y);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}
