/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.swing;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JTextArea;

/**
 * Description of the class.
 *
 * @author Karsten Klein
 *
 * @created August 30, 2001
 */
public class ProgressPane extends JPanel {
    private static final String NO_PROGRESS = "<no progress information>";
    private JTextArea textArea;
    private JLabel textLabel;
    private JButton cancelButton;
    private JProgressBar progressBar;
    private boolean isCanceled = false;

    /**
     * <code>ProgressPane</code> constructor.
     */
    public ProgressPane() {
        textArea = new JTextArea();
        textArea.setMargin(new Insets(5, 5, 5, 5));
        textArea.setEditable(false);
        textLabel = new JLabel("", JLabel.CENTER);
        textLabel.setPreferredSize(new Dimension(250, 50));
        cancelButton = new JButton("Cancel");
        cancelButton.setActionCommand("cancel");
        cancelButton.addActionListener(new ButtonListener());
        progressBar = new JProgressBar(0, 100);
        progressBar.setStringPainted(true);
        progressBar.setString(NO_PROGRESS);
        setLayout(new BorderLayout());

        // this way also centers the string in the progress bar vertically
        JPanel progressPanel = new JPanel(new BorderLayout());
        progressPanel.add(textLabel, BorderLayout.CENTER);
        progressPanel.add(progressBar, BorderLayout.SOUTH);
        add(progressPanel, BorderLayout.CENTER);
        add(cancelButton, BorderLayout.SOUTH);
    }

    /**
     * Sets the <code>description</code> attribute of the <code>ProgressPane</code>
     * object.
     *
     * @param description The new <code>description</code> value.
     */
    public void setDescription(String description) {
        textLabel.setText(description);
    }

    /**
     * Method description.
     *
     * @param progress Parameter description.
     */
    public void setProgress(int progress) {
        if (progress == 0) {
            progressBar.setString(NO_PROGRESS);
            progressBar.setStringPainted(true);
        } else {
            progressBar.setString(null);
            progressBar.setValue(progress);
        }
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public boolean isCanceled() {
        return isCanceled;
    }

    /**
     * Method description.
     *
     * @param isCancelEnabled Parameter description.
     */
    public void setCancelEnabled(boolean isCancelEnabled) {
        cancelButton.setEnabled(isCancelEnabled);
    }

    /**
     * Description of the class.
     *
     * @author Karsten Klein
     *
     * @created August 30, 2001
     */
    class ButtonListener implements ActionListener {

        /**
         * Description of the method.
         *
         * @param evt Description of parameter.
         */
        public void actionPerformed(ActionEvent evt) {
            isCanceled = true;
            cancelButton.setEnabled(false);
        }
    }
}
