/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.swing;

import org.srs3d.viewer.objects.Computation;
import org.srs3d.viewer.util.ExceptionHandler;

/**
 * This thread serves the progress dialog. It requests the status from the running
 * computation and forwards the information to the progress display.
 *
 * @author Karsten Klein
 *
 * @created August 22, 2001
 */
public class ProgressThread extends Thread {
    private Computation computation = null;
    private ProgressDisplay progressDisplay = null;
    private int refreshDelay = 100;

    /**
     * <code>ProgressThread</code> constructor.
     *
     * @param progressDisplay Description of parameter.
     * @param computation Description of parameter.
     */
    public ProgressThread(ProgressDisplay progressDisplay,
        Computation computation) {
        this.computation = computation;
        this.progressDisplay = progressDisplay;
    }

    /**
     * Main processing method for the <code>ProgressThread</code> object.
     */
    public void run() {
        SwingSettings.update();
        try {
            int loop = 0;
            boolean isCanceled = false;
            while (!computation.hasFinished()) {
                if (progressDisplay.hasCanceled()) {
                    if (!isCanceled) {
                        computation.cancel();
                        isCanceled = true;
                    }
                } else {
                    progressDisplay.setDescription(computation.getDescription());
                    progressDisplay.setProgress(computation.getProgress());
                    progressDisplay.setCancelEnabled(computation.canCancel());
                }
                try {
                    sleep(refreshDelay);
                } catch (InterruptedException e) {

                    // :SILENT EXCEPTION:
                }
            }

            // close the progress window, when terminating (could be without canceling)
            progressDisplay.close();
        } catch (Throwable t) {
            ExceptionHandler.handleException(t, this);

            // close the progress window, when terminating (could be without canceling)
            progressDisplay.close();
            try {
                throw (t);
            } catch (Throwable e) {

                // silent; should not happen; nothing to do more
                ExceptionHandler.handleException(e,
                    ExceptionHandler.SILENT_IN_DEBUG, this);
            }
        }
    }
}
