/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.swing;

import java.util.Map;

import org.srs3d.viewer.util.Replacer;

/**
 * Description of the class.
 *
 * @author Karsten Klein
 *
 * @created May 15, 2002
 */
public abstract class StrategyModule implements Module {
    private Check visibleCheck = null;

    /**
     * Sets the <code>visibleCheck</code> attribute of the <code>StrategyModule</code>
     * object.
     *
     * @param visibleCheck The new <code>visibleCheck</code> value.
     */
    public void setVisibleCheck(Check visibleCheck) {
        this.visibleCheck = visibleCheck;
    }

    /**
     * Description of the method.
     *
     * @return Description of the returned value.
     */
    public boolean checkVisible() {
        if (visibleCheck != null) {
            return visibleCheck.getResult();
        }
        return true;
    }

    /**
     * Method description.
     *
     * @param string Parameter description.
     *
     * @return Return description.
     */
    public static String convert(String string) {
        string = string.toUpperCase();
        string = Replacer.replace(string, "", "A");
        string = Replacer.replace(string, " ", "");
        string = Replacer.replace(string, "&", "AND");
        string = Replacer.replace(string, "(", "");
        string = Replacer.replace(string, ")", "");
        string = Replacer.replace(string, ".", "");
        return string;
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public String createReport() {
        return (checkVisible() ? "V" : "I");
    }

    /**
     * Method description.
     *
     * @param map Parameter description.
     */
    public void createReport(Map map) {
        update();
        map.put(convert(getName()), createReport());
    }
}
