/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.urn;

import java.applet.AppletStub;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Properties;

import org.srs3d.viewer.bioatlas.Application;
import org.srs3d.viewer.util.Replacer;

/**
 * Description of the class.
 *
 * @author Karsten Klein
 *
 * @created October 8, 2002
 */
public class LocalURNResolver implements URNResolver {
    private static final LocalURNResolver instance = new LocalURNResolver();
    private Properties inputMap = new Properties();
    private ArrayList applications = new ArrayList();

    /**
     * Constructor description.
     */
    private LocalURNResolver() {
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public static LocalURNResolver getInstance() {
        return instance;
    }

    /**
     * Method description.
     *
     * @param urn Parameter description.
     * @param dataString Parameter description.
     */
    public static void register(String urn, String dataString) {
        getInstance().inputMap.setProperty(urn, dataString);
    }

    /**
     * Method description.
     *
     * @param urn Parameter description.
     */
    public static void remove(String urn) {
        getInstance().inputMap.remove(urn);
    }

    /**
     * Method description.
     *
     * @param application Parameter description.
     */
    public static void register(Application application) {
        getInstance().applications.add(application);
    }

    /**
     * Method description.
     *
     * @param application Parameter description.
     */
    public static void remove(Application application) {
        getInstance().applications.remove(application);
    }

    /**
     * Method description.
     *
     * @param urn Parameter description.
     */
    public void connect(String urn) {
    }

    /**
     * Method description.
     *
     * @param urn Parameter description.
     *
     * @return Return description.
     */
    public InputStream openStream(String urn) {
        String string = inputMap.getProperty(urn);
        if (string == null) {
            Iterator iterator = applications.iterator();
            Application application;
            AppletStub appletStub;
            urn = urn.substring(urn.lastIndexOf(":") + 1);
            while (string == null && iterator.hasNext()) {
                application = (Application) iterator.next();
                appletStub =
                    (AppletStub) application.getApplicationContext().get("AppletStub");
                string = appletStub.getParameter(urn);
            }
        }
        if (string == null) {
            return null;
        }
        string = Replacer.replace(string, "|", "\n\r");
        return new ByteArrayInputStream(string.getBytes());
    }
}
