/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.util;


/**
 * PluggableLogger implementation.
 *
 * @author Karsten Klein
 */
public class DefaultLogger implements PluggableLogger {
    private Object logObject = null;
    private boolean isDebug = false;

    /**
     * Write the message to the appropriate log level.
     *
     * @param message Message to write.
     */
    public void trace(Object object) {
        if (isDebug()) {
            System.out.println("(TRACE) " + object);
        }
    }

    /**
     * Write the message to the appropriate log level.
     *
     * @param message Message to write.
     */
    public void debug(Object object) {
        if (isDebug()) {
            System.out.println("(DEBUG) " + object);
        }
    }

    /**
     * Write the message to the appropriate log level.
     *
     * @param message Message to write.
     */
    public void info(Object object) {
        System.out.println("(INFO) " + object);
    }

    /**
     * Write the message to the appropriate log level.
     *
     * @param message Message to write.
     */
    public void warn(Object object) {
        System.out.println("(WARN) " + object);
    }

    /**
     * Write the message to the appropriate log level.
     *
     * @param message Message to write.
     */
    public void error(Object object) {
        System.err.println("(ERROR) " + logObject + " " + object);
    }

    /**
     * Write the message to the appropriate log level.
     *
     * @param message Message to write.
     */
    public void fatal(Object object) {
        System.err.println("(FATAL) " + object);
    }

    /**
     * Write the message to the appropriate log level.
     *
     * @param message Message to write.
     * @param throwable Parameter description.
     */
    public void trace(Object object, Throwable throwable) {
        if (isDebug()) {
            System.err.println("(TRACE) " + object);
        }
    }

    /**
     * Write the message to the appropriate log level.
     *
     * @param message Message to write.
     * @param throwable Parameter description.
     */
    public void debug(Object object, Throwable throwable) {
        if (isDebug()) {
            System.err.println("(DEBUG) " + object);
        }
    }

    /**
     * Write the message to the appropriate log level.
     *
     * @param message Message to write.
     * @param throwable Parameter description.
     */
    public void info(Object object, Throwable throwable) {
        System.err.println("(DEBUG) " + object);
    }

    /**
     * Write the message to the appropriate log level.
     *
     * @param message Message to write.
     * @param throwable Parameter description.
     */
    public void warn(Object object, Throwable throwable) {
        System.err.println("(WARN) " + object);
    }

    /**
     * Write the message to the appropriate log level.
     *
     * @param message Message to write.
     * @param throwable Parameter description.
     */
    public void error(Object object, Throwable throwable) {
        System.err.println("(ERROR) " + object);
    }

    /**
     * Write the message to the appropriate log level.
     *
     * @param message Message to write.
     * @param Throwable associated with the message.
     */
    public void fatal(Object object, Throwable throwable) {
        System.err.println("(FATAL) " + object);
    }

    /**
     * Set the appropriate log object.
     *
     * @param logObject Object that produces the log information.
     */
    public void setLogObject(Class logObject) {
        this.logObject = logObject;
    }

    private final boolean isDebug() {
        return isDebug;
    }
}
