/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.util;


/**
 * Has methods for handling exceptions with different importance types.
 *
 * @author Karsten Klein
 *
 * @created September 04, 2002
 */
public class ExceptionHandler {
    private static final Log log = new Log(ExceptionHandler.class);
    public static final int SILENT_IN_DEBUG = 1;
    public static final int SILENT_IN_RELEASE = 2;
    public static final int TRACE_IN_RELEASE = 3;
    public static final int VISIBLE_IN_RELEASE = 4;
    public static final int CRITICAL = 5;
    public static final int PRINT_ALL_EXCEPTIONS = 0;
    public static final int PRINT_TYPE_SENSITIVE = 1;
    private static final int DEFAULT_TYPE = VISIBLE_IN_RELEASE;
    private static boolean isRelease = false;
    private static int MODE = PRINT_TYPE_SENSITIVE;

    //  private static int mode = PRINT_ALL_EXCEPTIONS;

    /**
     * Sets the <code>mode</code> attribute of the <code>ExceptionHandler</code> object.
     *
     * @param mode The new <code>mode</code> value.
     */
    public void setMode(int mode) {
        ExceptionHandler.MODE = mode;
    }

    /**
     * Handle throwable with the default importance.
     *
     * @param t Throwable to handle.
     */
    public static void handleException(Throwable t) {
        handleException(t, DEFAULT_TYPE);
    }

    /**
     * Description of the method.
     *
     * @param t Description of parameter.
     * @param object Description of parameter.
     */
    public static void handleException(Throwable t, Object object) {
        handleException(t, DEFAULT_TYPE, object);
    }

    /**
     * Description of the method.
     *
     * @param t Description of parameter.
     * @param exceptionType Description of parameter.
     */
    public static void handleException(Throwable t, int exceptionType) {
        handleException(t, exceptionType, null);
    }

    /**
     * Description of the method.
     *
     * @param t Description of parameter.
     * @param exceptionType Description of parameter.
     * @param source Description of parameter.
     */
    public static void handleException(Throwable t, int exceptionType,
        Object source) {
        boolean isPrintStackTrace = false;
        boolean isPrintException = MODE == VISIBLE_IN_RELEASE;
        if (MODE == PRINT_ALL_EXCEPTIONS) {
            isPrintStackTrace = true;
        } else {
            if (exceptionType != SILENT_IN_DEBUG) {
                if (exceptionType != SILENT_IN_RELEASE) {
                    if (exceptionType == CRITICAL) {
                        isPrintStackTrace = true;
                    } else {
                        if (isRelease && exceptionType == TRACE_IN_RELEASE) {
                            isPrintStackTrace = true;
                        }
                    }
                }
            }
        }
        if (isPrintException || isPrintStackTrace) {
            log.error(t.getMessage(), t);
            t.printStackTrace();
        }
    }
}
