/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;

/**
 * Class for replacing strings with strings
 *
 * @author Karsten Klein
 *
 * @created September 4, 2002
 */
public class Replacer {
    private static final Log log = new Log(Replacer.class);

    /**
     * The main program for the <code>Replacer</code> class.
     *
     * @param args The command line arguments: filename, search string, replace string.
     */
    public static void main(String[] args) {
        try {

            // arg 0: filename
            // arg 1: search string
            // arg 2: replacement string
            File inFile = new File(args[0]);
            File outFile = new File(args[0] + ".replaced");
            inFile.renameTo(outFile);
            FileReader reader = new FileReader(outFile);
            FileWriter writer = new FileWriter(inFile);
            BufferedReader br = new BufferedReader(reader);
            System.out.println("Replacing '" + args[1] + "' with '" + args[2] +
                "' in " + args[0]);
            String line;
            do {
                line = br.readLine();
                if (line != null) {
                    line = replace(line, args[1], args[2]);
                    writer.write(line + "\r\n");
                }
            } while (line != null);
            br.close();
            reader.close();
            writer.flush();
            writer.close();
            outFile.delete();
        } catch (Exception e) {
            log.error(e, e);
        }
    }

    /**
     * Method for replacing a string in a string with another string.
     *
     * @param line String to perform the replacement in.
     * @param search String to be replaced.
     * @param replace String for the replacement.
     *
     * @return The processed string.
     */
    public static String replace(String line, String search, String replace) {
        int currentIndex = 0;
        int index = line.indexOf(search);
        String replaced = line;
        while (index != -1) {
            replaced = line.substring(0, index);
            replaced += replace;
            replaced += line.substring(index + search.length());
            currentIndex = index + replace.length();
            line = replaced;
            index = line.indexOf(search, currentIndex);
        }
        return replaced;
    }
}
