/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.util;

import java.util.Collection;
import java.util.Iterator;

/**
 * Collection of methods for operating on sets.
 *
 * @author Karsten Klein
 *
 * @created September 11, 2002
 */
public final class SetUtility {

    /**
     * Checks the passed collections for an intersection.
     *
     * @param setA First collection.
     * @param setB Second collection.
     *
     * @return <code>true</code> if at least one object is contained in both sets.
     */
    public static boolean hasIntersection(Collection setA, Collection setB) {
        Chronometer c = Chronometer.getChronometer("SetUtility");
        c.start();
        Iterator iterator = setA.iterator();
        Object object;
        while (iterator.hasNext()) {
            object = iterator.next();
            if (setB.contains(object)) {
                c.stop("hasIntersection( Collection, Collection )");
                return true;
            }
        }
        c.stop("hasIntersection( Collection, Collection )");
        return false;
    }

    /**
     * Removes all objects from the intersection collection that are not included in the
     * objects collection
     *
     * @param intersection Intersection collection.
     * @param objects Objects collection.
     */
    public static void intersect(Collection intersection, Collection objects) {
        Chronometer c = Chronometer.getChronometer("SetUtility");
        c.start();
        try {

            // uses the collections implementation
            intersection.retainAll(objects);
            c.stop("collection.retainAll( Collection )");
        } catch (UnsupportedOperationException e) {
            ExceptionHandler.handleException(e, ExceptionHandler.SILENT_IN_DEBUG);

            // use SetUtility implementation
            SetUtility.retainAll(intersection, objects);
            c.stop("SetUtility.retainAll( Collection, Collection )");
        }
    }

    /**
     * Description of the method.
     *
     * @param intersection Description of parameter.
     * @param objects Description of parameter.
     */
    private static void retainAll(Collection intersection, Collection objects) {
        if (SetUtility.hasIntersection(objects, intersection)) {
            Iterator iterator = intersection.iterator();
            while (iterator.hasNext()) {
                if (!objects.contains(iterator.next())) {
                    iterator.remove();
                }
            }
        } else {
            intersection.clear();
        }
    }
}
