/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.vecmath;

import javax.vecmath.Color4f;

/**
 * Description of the class.
 *
 * @author Karsten Klein
 * @version Dezember 11, 2000
 */
public final class ColorArray4f extends TupleArray4f {

    /**
     * <code>ColorArray4f</code> contructor.
     *
     * @param size Description of parameter.
     */
    public ColorArray4f(int size) {
        create(size);
    }

    /**
     * Retrieves the color with the specified index.
     *
     * @param index Description of parameter.
     *
     * @return The <code>At</code> value.
     */
    public final Color4f getAt(int index) {
        return (Color4f) buffer[index];
    }

    /**
     * Gets the <code>Buffer</code> attribute of the <code>ColorArray4f</code> object.
     *
     * @return The <code>Buffer</code> value.
     */
    public final Color4f[] getBuffer() {
        return (Color4f[]) buffer;
    }

    /**
     * Creates an array with specified size.
     *
     * @param size Description of parameter.
     */
    private final void create(int size) {
        buffer = new Color4f[size];
        for (int i = 0; i < size; i++) {
            this.buffer[i] = new Color4f();
        }
    }
}
