/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.vecmath;

import javax.vecmath.TexCoord2f;
import javax.vecmath.Vector3f;

/**
 * Description of the class.
 *
 * @author Karsten Klein
 *
 * @created March 20, 2001
 */
public final class Frame {

    /** Description of the field. */
    private Vector3f normal = new Vector3f(1, 0, 0);
    private Vector3f invertedNormal = new Vector3f(1, 0, 0);
    private TexCoord2f textureCoordinate = null;

    /**
     * Description of the field.
     */

    //  private boolean clockwise = true;

    /**
     * Sets the <code>Normal</code> attribute of the <code>Frame</code> object.
     *
     * @param normal The new <code>Normal</code> value.
     */
    public final void setNormal(Vector3f normal) {
        this.normal = normal;
        invertedNormal.set(normal);
        invertedNormal.negate();
    }

    /**
     * Sets the <code>Clockwise</code> attribute of the <code>Frame</code> object.
     *
     * @param clockwise The new <code>Clockwise</code> value.
     */

    //  public final void setClockwise( boolean clockwise ) {
    //
    //    this.clockwise = clockwise;
    //
    //  }

    /**
     * Gets the <code>Normal</code> attribute of the <code>Frame</code> object.
     *
     * @return The <code>Normal</code> value.
     */
    public final Vector3f getNormal(boolean isClockwise) {
        return isClockwise ? normal : invertedNormal;
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public final TexCoord2f getTextureCoordinate() {
        return textureCoordinate;
    }

    /**
     * Gets the <code>Clockwise</code> attribute of the <code>Frame</code> object.
     */

    //  public final boolean isClockwise() {
    //
    //    return clockwise;
    //  }
}
