/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.vecmath;

import java.util.ArrayList;

import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

/**
 * Description of the class.
 *
 * @author Karsten Klein
 *
 * @created March 20, 2001
 */
public final class LinearCurve3f extends AbstractCurve3f {

    /**
     * Method description.
     *
     * @param parameter Parameter description.
     *
     * @return Return description.
     */
    public final Vector3f computeTangent(float parameter) {
        ArrayList coordinates = getCoordinates();
        int size = coordinates.size();
        Vector3f tangent = new Vector3f();
        if (size > 1) {
            int index = mapCurveParameterToSegmentIndex(parameter);
            float t = mapCurveParameterToSegmentParamter(index, parameter);
            tangent.set((Point3f) coordinates.get(index + 1));
            tangent.sub((Point3f) coordinates.get(index));
        }
        return tangent;
    }

    /**
     * Method description.
     *
     * @param parameter Parameter description.
     *
     * @return Return description.
     */
    public final Point3f computePoint(float parameter) {
        ArrayList coordinates = getCoordinates();
        int size = coordinates.size();
        Point3f point = null;
        if (size > 1) {
            int index = mapCurveParameterToSegmentIndex(parameter);
            float t = mapCurveParameterToSegmentParamter(index, parameter);
            point = new Point3f((Point3f) coordinates.get(index));
            point.scale(t);
            t = 1 - t;
            index++;
            if (t != 0 && index < size) {
                Point3f interpolate = (Point3f) coordinates.get(index);
                point.x += interpolate.x * t;
                point.y += interpolate.y * t;
                point.z += interpolate.z * t;
            }
        } else {
            if (size == 1) {
                point = new Point3f((Point3f) coordinates.get(0));
            } else {
                point = new Point3f(0, 0, 0);
            }
        }
        return point;
    }
}
