/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.vecmath;

import javax.vecmath.Matrix3f;
import javax.vecmath.Point3f;
import javax.vecmath.Tuple3f;

/**
 * Description of the class.
 *
 * @author Karsten Klein
 *
 * @created March 20, 2001
 */
public final class PointArray3f extends TupleArray3f {

    /**
     * <code>PointArray3f</code> contructor.
     *
     * @param size Description of parameter.
     */
    public PointArray3f(int size) {
        create(size);
    }

    /**
     * Retrieves the point with the specified index.
     *
     * @param index Description of parameter.
     *
     * @return The <code>At</code> value.
     */
    public final Point3f getAt(int index) {
        return (Point3f) buffer[index];
    }

    /**
     * Gets the <code>Buffer</code> attribute of the <code>PointArray3f</code> object.
     *
     * @return The <code>Buffer</code> value.
     */
    public final Point3f[] getBuffer() {
        return (Point3f[]) buffer;
    }

    /**
     * Description of the method.
     *
     * @param size Description of parameter.
     */
    private final void create(int size) {
        buffer = new Point3f[size];
        for (int i = 0; i < size; i++) {
            this.buffer[i] = new Point3f();
        }
    }

    /**
     * Fill the specified array with transformed data starting at the given index
     *
     * @param rotation Description of parameter.
     * @param translation Description of parameter.
     * @param scale Description of parameter.
     * @param pointArray Description of parameter.
     * @param index Description of parameter.
     */
    public void transform(Matrix3f rotation, Tuple3f translation,
        Tuple3f scale, PointArray3f pointArray, int index) {
        Tuple3f point = null;
        for (int i = 0; i < buffer.length; i++) {
            point = pointArray.getAt(i + index);
            point.set(buffer[i]);
            point.x *= scale.x;
            point.y *= scale.y;
            point.z *= scale.z;
            rotation.transform(point);
            point.add(translation);
        }
    }
}
