/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.vecmath;

import javax.vecmath.Matrix3f;
import javax.vecmath.Tuple3f;
import javax.vecmath.Vector3f;

/**
 * Description of the class
 *
 * @author Karsten Klein
 *
 * @created March 20, 2001
 */
public final class VectorArray3f extends TupleArray3f {

    /**
     * <code>VectorArray3f</code> contructor.
     *
     * @param size Description of parameter
     */
    public VectorArray3f(int size) {
        create(size);
    }

    /**
     * Retrieves the vector with the specified index.
     *
     * @param index Description of parameter
     *
     * @return The <code>At</code> value
     */
    public final Vector3f getAt(int index) {
        return (Vector3f) buffer[index];
    }

    /**
     * Retrieves the vector buffer.
     *
     * @return The <code>Buffer</code> value
     */
    public final Vector3f[] getBuffer() {
        return (Vector3f[]) buffer;
    }

    /**
     * Description of the method
     *
     * @param size Description of parameter
     */
    public final void create(int size) {
        buffer = new Vector3f[size];
        for (int i = 0; i < size; i++) {
            this.buffer[i] = new Vector3f();
        }
    }

    /**
     * Fill the specified array with transformed data starting at the given index
     *
     * @param tranformation Description of parameter
     * @param scale Description of parameter
     * @param vectorArray Description of parameter
     * @param index Description of parameter
     */
    public final void transform(Matrix3f tranformation, Tuple3f scale,
        VectorArray3f vectorArray, int index) {
        Vector3f vector;
        for (int i = 0; i < buffer.length; i++) {
            vector = (Vector3f) vectorArray.getAt(i + index);
            vector.set(buffer[i]);
            vector.x *= scale.x;
            vector.y *= scale.y;
            vector.z *= scale.z;
            tranformation.transform(vector);
        }
    }

    /**
     * Description of the method.
     */
    public final void normalize() {
        for (int i = 0; i < buffer.length; i++) {
            ((Vector3f) buffer[i]).normalize();
        }
    }
}
