/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.visualization;

import org.srs3d.viewer.bioatlas.objects.Atom;

/**
 * <code>AtomVolume</code> implementation of the Volume interface. It basically wrappes
 * an <code>Atom</code> as <code>Volume</code>.
 *
 * @author Karsten Klein
 *
 * @created May 31, 2001
 */
public class AtomVolume extends Sphere {
    private Atom atom = null;

    /**
     * <code>AtomVolume</code> constructor.
     *
     * @param atom Description of parameter.
     * @param radiusOffset Description of parameter.
     */
    public AtomVolume(Atom atom, float radiusOffset) {
        this.atom = atom;
        setCoordinate(atom.getCoordinate());
        setRadius(atom.getTemplate().getVanDerWaalsRadius() + radiusOffset);
    }

    /**
     * Gets the <code>atom</code> attribute of the <code>AtomVolume</code> object.
     *
     * @return The <code>atom</code> value.
     */
    public Atom getAtom() {
        return atom;
    }
}
