/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.visualization;


/**
 * Keeps the triangle table for each case of cube sampling.
 *
 * @author Karsten Klein
 *
 * @created April 26, 2001
 */
public final class Cases {

    /**
     * Case table of triangle list for the marching cube implementation. The encoding of
     * the edges is described in the MarchingCube.doc file.
     */
    public static final int[][][] caseCodes =
    {
        {  },
        {
            { 2, 1, 4 }
        },
        {
            { 10, 12, 1 }
        },
        {
            { 2, 10, 4 },
            { 4, 10, 12 }
        },
        {
            { 2, 20, 17 }
        },
        {
            { 4, 20, 1 },
            { 1, 20, 17 }
        },
        {
            { 20, 12, 1 },
            { 1, 2, 20 },
            { 20, 17, 10 },
            { 10, 12, 20 }
        }, //{{1, 10, 12}, {2, 20, 17}},
        {
            { 17, 10, 12 },
            { 17, 12, 20 },
            { 12, 4, 20 }
        },
        {
            { 10, 17, 28 }
        },
        {
            { 2, 17, 28 },
            { 28, 4, 2 },
            { 1, 4, 28 },
            { 28, 10, 1 }
        }, //{{2, 1, 4}, {17, 28, 10}},
        {
            { 1, 17, 12 },
            { 12, 17, 28 }
        },
        {
            { 2, 17, 28 },
            { 2, 28, 4 },
            { 28, 12, 4 }
        },
        {
            { 10, 2, 28 },
            { 28, 2, 20 }
        },
        {
            { 10, 1, 4 },
            { 10, 4, 28 },
            { 4, 20, 28 }
        },
        {
            { 1, 2, 20 },
            { 1, 20, 12 },
            { 20, 28, 12 }
        },
        {
            { 28, 12, 20 },
            { 4, 20, 12 }
        },
        {
            { 34, 4, 33 }
        },
        {
            { 34, 2, 33 },
            { 33, 2, 1 }
        },
        {
            { 1, 10, 34 },
            { 34, 4, 1 },
            { 10, 12, 33 },
            { 33, 34, 10 }
        }, // {{12, 1, 10}, {33, 34, 4}},
        {
            { 12, 33, 34 },
            { 12, 34, 10 },
            { 34, 2, 10 }
        },
        {
            { 33, 17, 2 },
            { 2, 4, 33 },
            { 33, 34, 20 },
            { 20, 17, 33 }
        }, // {{34, 4, 33}, {20, 17, 2}},
        {
            { 34, 20, 17 },
            { 34, 17, 33 },
            { 17, 1, 33 }
        },
        {
            { 10, 12, 1 },
            { 34, 4, 33 },
            { 17, 2, 20 }
        },
        {
            { 10, 20, 17 },
            { 12, 20, 10 },
            { 12, 34, 20 },
            { 12, 33, 34 }
        },
        {
            { 4, 33, 34 },
            { 28, 10, 17 }
        },
        {
            { 34, 2, 33 },
            { 33, 2, 1 },
            { 28, 10, 17 }
        },
        {
            { 28, 12, 17 },
            { 17, 12, 1 },
            { 34, 4, 33 }
        },
        {
            { 33, 28, 12 },
            { 28, 33, 2 },
            { 34, 2, 33 },
            { 28, 2, 17 }
        },
        {
            { 10, 2, 28 },
            { 28, 2, 20 },
            { 33, 34, 4 }
        },
        {
            { 20, 28, 10 },
            { 20, 10, 33 },
            { 20, 33, 34 },
            { 1, 33, 10 }
        },
        {
            { 1, 2, 20 },
            { 1, 20, 12 },
            { 20, 28, 12 },
            { 34, 4, 33 }
        },
        {
            { 33, 34, 12 },
            { 12, 34, 20 },
            { 28, 12, 20 }
        },
        {
            { 42, 33, 12 }
        },
        {
            { 1, 12, 42 },
            { 42, 2, 1 },
            { 2, 42, 33 },
            { 33, 4, 2 }
        }, // {{33, 12, 42}, {4, 2, 1}},
        {
            { 10, 42, 1 },
            { 1, 42, 33 }
        },
        {
            { 33, 4, 2 },
            { 33, 2, 42 },
            { 2, 10, 42 }
        },
        {
            { 33, 12, 42 },
            { 17, 2, 20 }
        },
        {
            { 17, 1, 20 },
            { 20, 1, 4 },
            { 42, 33, 12 }
        },
        {
            { 33, 1, 42 },
            { 42, 1, 10 },
            { 20, 17, 2 }
        },
        {
            { 4, 20, 33 },
            { 33, 20, 10 },
            { 10, 20, 17 },
            { 10, 42, 33 }
        },
        {
            { 17, 28, 42 },
            { 42, 33, 17 },
            { 10, 17, 33 },
            { 33, 12, 10 }
        }, // {{12, 42, 33}, {10, 17, 28}},
        {
            { 42, 33, 12 },
            { 2, 1, 4 },
            { 28, 10, 17 }
        },
        {
            { 28, 42, 33 },
            { 28, 33, 17 },
            { 33, 1, 17 }
        },
        {
            { 42, 17, 28 },
            { 33, 17, 42 },
            { 33, 2, 17 },
            { 33, 4, 2 }
        },
        {
            { 20, 28, 2 },
            { 2, 28, 10 },
            { 33, 12, 42 }
        },
        {
            { 42, 33, 28 },
            { 28, 33, 4 },
            { 20, 28, 4 },
            { 12, 10, 1 }
        },
        {
            { 1, 42, 33 },
            { 42, 1, 20 },
            { 20, 28, 42 },
            { 20, 1, 2 }
        },
        {
            { 42, 33, 28 },
            { 28, 33, 4 },
            { 20, 28, 4 }
        },
        {
            { 42, 34, 12 },
            { 12, 34, 4 }
        },
        {
            { 1, 12, 42 },
            { 1, 42, 2 },
            { 42, 34, 2 }
        },
        {
            { 4, 1, 10 },
            { 4, 10, 34 },
            { 10, 42, 34 }
        },
        {
            { 2, 10, 42 },
            { 42, 34, 2 }
        },
        {
            { 42, 34, 12 },
            { 12, 34, 4 },
            { 17, 2, 20 }
        },
        {
            { 12, 17, 1 },
            { 17, 12, 34 },
            { 42, 34, 12 },
            { 17, 34, 20 }
        },
        {
            { 4, 1, 10 },
            { 4, 10, 34 },
            { 10, 42, 34 },
            { 17, 2, 20 }
        },
        {
            { 20, 17, 34 },
            { 34, 17, 10 },
            { 42, 34, 10 }
        },
        {
            { 4, 12, 34 },
            { 34, 12, 42 },
            { 17, 28, 10 }
        },
        {
            { 1, 12, 42 },
            { 1, 42, 2 },
            { 42, 34, 2 },
            { 28, 10, 17 }
        },
        {
            { 1, 17, 4 },
            { 4, 17, 42 },
            { 42, 17, 28 },
            { 42, 34, 4 }
        },
        {
            { 17, 28, 2 },
            { 2, 28, 42 },
            { 34, 2, 42 }
        },
        {
            { 2, 10, 12 },
            { 12, 4, 2 },
            { 42, 34, 20 },
            { 28, 42, 20 }
        },
        {
            { 42, 34, 20 },
            { 42, 20, 28 },
            { 10, 1, 12 }
        },
        {
            { 20, 28, 42 },
            { 20, 42, 34 },
            { 4, 1, 2 }
        },
        {
            { 42, 34, 20 },
            { 42, 20, 28 }
        },
        {
            { 34, 49, 20 }
        },
        {
            { 1, 49, 20 },
            { 20, 2, 1 },
            { 1, 4, 34 },
            { 34, 49, 1 }
        }, // {{4, 2, 1}, {34, 49, 20}},
        {
            { 12, 1, 10 },
            { 20, 34, 49 }
        },
        {
            { 12, 4, 10 },
            { 10, 4, 2 },
            { 49, 20, 34 }
        },
        {
            { 2, 34, 17 },
            { 17, 34, 49 }
        },
        {
            { 4, 34, 49 },
            { 4, 49, 1 },
            { 49, 17, 1 }
        },
        {
            { 49, 17, 34 },
            { 34, 17, 2 },
            { 12, 1, 10 }
        },
        {
            { 4, 10, 12 },
            { 10, 4, 49 },
            { 49, 17, 10 },
            { 49, 4, 34 }
        },
        {
            { 10, 34, 49 },
            { 49, 28, 10 },
            { 10, 17, 20 },
            { 20, 34, 10 }
        }, // {{17, 28, 10}, {20, 34, 49}},
        {
            { 34, 49, 20 },
            { 10, 17, 28 },
            { 4, 2, 1 }
        },
        {
            { 1, 17, 12 },
            { 12, 17, 28 },
            { 34, 49, 20 }
        },
        {
            { 34, 49, 4 },
            { 4, 49, 28 },
            { 12, 4, 28 },
            { 20, 2, 17 }
        },
        {
            { 49, 28, 10 },
            { 49, 10, 34 },
            { 10, 2, 34 }
        },
        {
            { 28, 34, 49 },
            { 28, 4, 34 },
            { 10, 4, 28 },
            { 1, 4, 10 }
        },
        {
            { 28, 12, 1 },
            { 28, 1, 34 },
            { 28, 34, 49 },
            { 2, 34, 1 }
        },
        {
            { 34, 49, 4 },
            { 4, 49, 28 },
            { 12, 4, 28 }
        },
        {
            { 33, 49, 4 },
            { 4, 49, 20 }
        },
        {
            { 20, 2, 1 },
            { 20, 1, 49 },
            { 1, 33, 49 }
        },
        {
            { 20, 4, 49 },
            { 49, 4, 33 },
            { 10, 12, 1 }
        },
        {
            { 33, 49, 12 },
            { 12, 49, 2 },
            { 2, 49, 20 },
            { 2, 10, 12 }
        },
        {
            { 2, 4, 33 },
            { 2, 33, 17 },
            { 33, 49, 17 }
        },
        {
            { 17, 1, 49 },
            { 33, 49, 1 }
        },
        {
            { 10, 12, 17 },
            { 17, 12, 33 },
            { 49, 17, 33 },
            { 1, 2, 4 }
        },
        {
            { 10, 12, 17 },
            { 17, 12, 33 },
            { 49, 17, 33 }
        },
        {
            { 33, 49, 4 },
            { 4, 49, 20 },
            { 10, 17, 28 }
        },
        {
            { 20, 2, 1 },
            { 20, 1, 49 },
            { 1, 33, 49 },
            { 10, 17, 28 }
        },
        {
            { 17, 1, 4 },
            { 4, 20, 17 },
            { 33, 49, 28 },
            { 12, 33, 28 }
        },
        {
            { 28, 12, 33 },
            { 28, 33, 49 },
            { 20, 2, 17 }
        },
        {
            { 49, 4, 33 },
            { 4, 49, 10 },
            { 10, 2, 4 },
            { 10, 49, 28 }
        },
        {
            { 28, 10, 49 },
            { 49, 10, 1 },
            { 33, 49, 1 }
        },
        {
            { 33, 49, 28 },
            { 33, 28, 12 },
            { 1, 2, 4 }
        },
        {
            { 28, 12, 33 },
            { 28, 33, 49 }
        },
        {
            { 12, 20, 36 },
            { 36, 33, 12 },
            { 12, 42, 49 },
            { 49, 20, 12 }
        }, // {{42, 33, 12}, {49, 20, 34}},
        {
            { 2, 1, 4 },
            { 42, 33, 12 },
            { 20, 34, 49 }
        },
        {
            { 10, 42, 1 },
            { 1, 42, 33 },
            { 20, 34, 49 }
        },
        {
            { 33, 4, 2 },
            { 33, 2, 42 },
            { 2, 10, 42 },
            { 20, 34, 49 }
        },
        {
            { 2, 34, 17 },
            { 17, 34, 49 },
            { 12, 42, 33 }
        },
        {
            { 4, 34, 49 },
            { 4, 49, 1 },
            { 49, 17, 1 },
            { 42, 33, 12 }
        },
        {
            { 42, 10, 33 },
            { 1, 33, 10 },
            { 2, 34, 17 },
            { 34, 49, 17 }
        },
        {
            { 49, 17, 10 },
            { 49, 10, 42 },
            { 33, 4, 34 }
        },
        {
            { 10, 17, 28 },
            { 34, 49, 20 },
            { 12, 42, 33 }
        },
        {
            { 1, 4, 2 },
            { 12, 42, 33 },
            { 20, 34, 49 },
            { 17, 28, 10 }
        },
        {
            { 28, 42, 33 },
            { 28, 33, 17 },
            { 33, 1, 17 },
            { 34, 49, 20 }
        },
        {
            { 28, 42, 49 },
            { 20, 2, 17 },
            { 34, 33, 4 }
        },
        {
            { 33, 12, 34 },
            { 34, 12, 10 },
            { 2, 34, 10 },
            { 42, 49, 28 }
        },
        {
            { 4, 34, 33 },
            { 12, 10, 1 },
            { 42, 49, 28 }
        },
        {
            { 33, 1, 2 },
            { 33, 2, 34 },
            { 49, 28, 42 }
        },
        {
            { 34, 33, 4 },
            { 28, 42, 49 }
        },
        {
            { 42, 49, 20 },
            { 42, 20, 12 },
            { 20, 4, 12 }
        },
        {
            { 2, 49, 20 },
            { 1, 49, 2 },
            { 1, 42, 49 },
            { 1, 12, 42 }
        },
        {
            { 1, 20, 4 },
            { 20, 1, 42 },
            { 10, 42, 1 },
            { 20, 42, 49 }
        },
        {
            { 49, 20, 42 },
            { 42, 20, 2 },
            { 10, 42, 2 }
        },
        {
            { 49, 17, 42 },
            { 42, 17, 4 },
            { 4, 17, 2 },
            { 4, 12, 42 }
        },
        {
            { 12, 42, 1 },
            { 1, 42, 49 },
            { 17, 1, 49 }
        },
        {
            { 10, 42, 49 },
            { 10, 49, 17 },
            { 2, 4, 1 }
        },
        {
            { 10, 42, 49 },
            { 10, 49, 17 }
        },
        {
            { 10, 17, 12 },
            { 12, 17, 20 },
            { 4, 12, 20 },
            { 28, 42, 49 }
        },
        {
            { 20, 2, 17 },
            { 28, 42, 49 },
            { 10, 1, 12 }
        },
        {
            { 20, 4, 1 },
            { 20, 1, 17 },
            { 28, 42, 49 }
        },
        {
            { 17, 20, 2 },
            { 42, 49, 28 }
        },
        {
            { 10, 2, 4 },
            { 10, 4, 12 },
            { 42, 49, 28 }
        },
        {
            { 12, 10, 1 },
            { 49, 28, 42 }
        },
        {
            { 4, 1, 2 },
            { 42, 49, 28 }
        },
        {
            { 28, 42, 49 }
        },
        {
            { 42, 28, 49 }
        },
        {
            { 1, 4, 2 },
            { 49, 42, 28 }
        },
        {
            { 1, 10, 28 },
            { 28, 49, 1 },
            { 1, 49, 42 },
            { 42, 12, 1 }
        }, // {{10, 12, 1}, {28, 49, 42}},
        {
            { 2, 10, 4 },
            { 4, 10, 12 },
            { 49, 42, 28 }
        },
        {
            { 2, 42, 28 },
            { 28, 17, 2 },
            { 2, 20, 49 },
            { 49, 42, 2 }
        }, // {{20, 17, 2}, {49, 42, 28}},
        {
            { 4, 20, 1 },
            { 1, 20, 17 },
            { 42, 28, 49 }
        },
        {
            { 2, 20, 17 },
            { 42, 28, 49 },
            { 1, 10, 12 }
        },
        {
            { 17, 10, 12 },
            { 17, 12, 20 },
            { 12, 4, 20 },
            { 42, 28, 49 }
        },
        {
            { 42, 10, 49 },
            { 49, 10, 17 }
        },
        {
            { 42, 10, 49 },
            { 49, 10, 17 },
            { 4, 2, 1 }
        },
        {
            { 42, 12, 1 },
            { 42, 1, 49 },
            { 1, 17, 49 }
        },
        {
            { 17, 49, 42 },
            { 17, 42, 4 },
            { 17, 4, 2 },
            { 12, 4, 42 }
        },
        {
            { 20, 49, 42 },
            { 20, 42, 2 },
            { 42, 10, 2 }
        },
        {
            { 20, 1, 4 },
            { 1, 20, 42 },
            { 42, 10, 1 },
            { 42, 20, 49 }
        },
        {
            { 49, 2, 20 },
            { 49, 1, 2 },
            { 42, 1, 49 },
            { 12, 1, 42 }
        },
        {
            { 49, 42, 20 },
            { 20, 42, 12 },
            { 4, 20, 12 }
        },
        {
            { 4, 28, 49 },
            { 49, 34, 4 },
            { 4, 33, 42 },
            { 42, 28, 4 }
        }, // {{33, 34, 4}, {42, 28, 49}},
        {
            { 1, 33, 2 },
            { 2, 33, 34 },
            { 28, 49, 42 }
        },
        {
            { 34, 4, 33 },
            { 10, 12, 1 },
            { 49, 42, 28 }
        },
        {
            { 12, 33, 34 },
            { 12, 34, 10 },
            { 34, 2, 10 },
            { 49, 42, 28 }
        },
        {
            { 42, 28, 49 },
            { 2, 20, 17 },
            { 33, 34, 4 }
        },
        {
            { 42, 28, 33 },
            { 33, 28, 17 },
            { 1, 33, 17 },
            { 49, 34, 20 }
        },
        {
            { 4, 1, 2 },
            { 42, 12, 33 },
            { 34, 20, 49 },
            { 28, 17, 10 }
        },
        {
            { 17, 10, 28 },
            { 49, 34, 20 },
            { 42, 12, 33 }
        },
        {
            { 17, 49, 10 },
            { 10, 49, 42 },
            { 4, 33, 34 }
        },
        {
            { 10, 42, 33 },
            { 33, 1, 10 },
            { 34, 2, 17 },
            { 49, 34, 17 }
        },
        {
            { 34, 4, 49 },
            { 49, 4, 1 },
            { 17, 49, 1 },
            { 33, 42, 12 }
        },
        {
            { 34, 2, 17 },
            { 34, 17, 49 },
            { 42, 12, 33 }
        },
        {
            { 4, 33, 2 },
            { 2, 33, 42 },
            { 10, 2, 42 },
            { 34, 20, 49 }
        },
        {
            { 42, 10, 1 },
            { 42, 1, 33 },
            { 34, 20, 49 }
        },
        {
            { 1, 2, 4 },
            { 33, 42, 12 },
            { 34, 20, 49 }
        },
        {
            { 33, 42, 12 },
            { 20, 49, 34 }
        },
        {
            { 12, 28, 33 },
            { 33, 28, 49 }
        },
        {
            { 49, 33, 28 },
            { 28, 33, 12 },
            { 2, 1, 4 }
        },
        {
            { 10, 28, 49 },
            { 10, 49, 1 },
            { 49, 33, 1 }
        },
        {
            { 4, 49, 33 },
            { 49, 4, 10 },
            { 2, 10, 4 },
            { 49, 10, 28 }
        },
        {
            { 12, 28, 33 },
            { 33, 28, 49 },
            { 2, 20, 17 }
        },
        {
            { 1, 17, 4 },
            { 20, 4, 17 },
            { 49, 33, 28 },
            { 33, 12, 28 }
        },
        {
            { 2, 20, 1 },
            { 1, 20, 49 },
            { 33, 1, 49 },
            { 17, 10, 28 }
        },
        {
            { 49, 33, 4 },
            { 49, 4, 20 },
            { 17, 10, 28 }
        },
        {
            { 12, 10, 17 },
            { 12, 17, 33 },
            { 17, 49, 33 }
        },
        {
            { 12, 10, 17 },
            { 12, 17, 33 },
            { 17, 49, 33 },
            { 2, 1, 4 }
        },
        {
            { 1, 17, 49 },
            { 49, 33, 1 }
        },
        {
            { 4, 2, 33 },
            { 33, 2, 17 },
            { 49, 33, 17 }
        },
        {
            { 49, 33, 12 },
            { 49, 12, 2 },
            { 49, 2, 20 },
            { 10, 2, 12 }
        },
        {
            { 4, 20, 49 },
            { 4, 49, 33 },
            { 12, 10, 1 }
        },
        {
            { 2, 20, 1 },
            { 1, 20, 49 },
            { 33, 1, 49 }
        },
        {
            { 49, 33, 4 },
            { 49, 4, 20 }
        },
        {
            { 49, 34, 4 },
            { 49, 4, 28 },
            { 4, 12, 28 }
        },
        {
            { 12, 28, 1 },
            { 1, 28, 34 },
            { 34, 28, 49 },
            { 34, 2, 1 }
        },
        {
            { 34, 28, 49 },
            { 4, 28, 34 },
            { 4, 10, 28 },
            { 4, 1, 10 }
        },
        {
            { 28, 49, 10 },
            { 10, 49, 34 },
            { 2, 10, 34 }
        },
        {
            { 49, 34, 4 },
            { 49, 4, 28 },
            { 4, 12, 28 },
            { 2, 20, 17 }
        },
        {
            { 17, 1, 12 },
            { 17, 12, 28 },
            { 49, 34, 20 }
        },
        {
            { 49, 34, 20 },
            { 17, 10, 28 },
            { 2, 4, 1 }
        },
        {
            { 28, 17, 10 },
            { 34, 20, 49 }
        },
        {
            { 10, 4, 12 },
            { 4, 10, 49 },
            { 17, 49, 10 },
            { 4, 49, 34 }
        },
        {
            { 17, 49, 34 },
            { 17, 34, 2 },
            { 1, 12, 10 }
        },
        {
            { 34, 4, 49 },
            { 49, 4, 1 },
            { 17, 49, 1 }
        },
        {
            { 34, 2, 17 },
            { 34, 17, 49 }
        },
        {
            { 4, 12, 10 },
            { 4, 10, 2 },
            { 20, 49, 34 }
        },
        {
            { 1, 12, 10 },
            { 34, 20, 49 }
        },
        {
            { 2, 4, 1 },
            { 49, 34, 20 }
        },
        {
            { 49, 34, 20 }
        },
        {
            { 34, 42, 20 },
            { 20, 42, 28 }
        },
        {
            { 28, 20, 42 },
            { 42, 20, 34 },
            { 1, 4, 2 }
        },
        {
            { 34, 42, 20 },
            { 20, 42, 28 },
            { 1, 10, 12 }
        },
        {
            { 10, 2, 12 },
            { 4, 12, 2 },
            { 34, 42, 20 },
            { 42, 28, 20 }
        },
        {
            { 28, 17, 2 },
            { 28, 2, 42 },
            { 2, 34, 42 }
        },
        {
            { 17, 1, 4 },
            { 17, 4, 42 },
            { 17, 42, 28 },
            { 34, 42, 4 }
        },
        {
            { 12, 1, 42 },
            { 42, 1, 2 },
            { 34, 42, 2 },
            { 10, 28, 17 }
        },
        {
            { 12, 4, 34 },
            { 12, 34, 42 },
            { 28, 17, 10 }
        },
        {
            { 17, 20, 34 },
            { 17, 34, 10 },
            { 34, 42, 10 }
        },
        {
            { 1, 4, 10 },
            { 10, 4, 34 },
            { 42, 10, 34 },
            { 2, 17, 20 }
        },
        {
            { 17, 12, 1 },
            { 12, 17, 34 },
            { 34, 42, 12 },
            { 34, 17, 20 }
        },
        {
            { 34, 42, 12 },
            { 34, 12, 4 },
            { 2, 17, 20 }
        },
        {
            { 10, 2, 42 },
            { 34, 42, 2 }
        },
        {
            { 1, 4, 10 },
            { 10, 4, 34 },
            { 42, 10, 34 }
        },
        {
            { 12, 1, 42 },
            { 42, 1, 2 },
            { 34, 42, 2 }
        },
        {
            { 34, 42, 12 },
            { 34, 12, 4 }
        },
        {
            { 33, 42, 28 },
            { 33, 28, 4 },
            { 28, 20, 4 }
        },
        {
            { 42, 1, 33 },
            { 1, 42, 20 },
            { 28, 20, 42 },
            { 1, 20, 2 }
        },
        {
            { 33, 42, 28 },
            { 33, 28, 4 },
            { 28, 20, 4 },
            { 10, 12, 1 }
        },
        {
            { 28, 20, 2 },
            { 28, 2, 10 },
            { 12, 33, 42 }
        },
        {
            { 17, 42, 28 },
            { 17, 33, 42 },
            { 2, 33, 17 },
            { 4, 33, 2 }
        },
        {
            { 42, 28, 33 },
            { 33, 28, 17 },
            { 1, 33, 17 }
        },
        {
            { 33, 42, 12 },
            { 1, 2, 4 },
            { 10, 28, 17 }
        },
        {
            { 42, 12, 33 },
            { 17, 10, 28 }
        },
        {
            { 20, 4, 33 },
            { 20, 33, 10 },
            { 20, 10, 17 },
            { 42, 10, 33 }
        },
        {
            { 1, 33, 42 },
            { 1, 42, 10 },
            { 17, 20, 2 }
        },
        {
            { 1, 17, 20 },
            { 1, 20, 4 },
            { 33, 42, 12 }
        },
        {
            { 12, 33, 42 },
            { 2, 17, 20 }
        },
        {
            { 4, 33, 2 },
            { 2, 33, 42 },
            { 10, 2, 42 }
        },
        {
            { 42, 10, 1 },
            { 42, 1, 33 }
        },
        {
            { 12, 33, 42 },
            { 2, 4, 1 }
        },
        {
            { 33, 42, 12 }
        },
        {
            { 34, 33, 12 },
            { 34, 12, 20 },
            { 12, 28, 20 }
        },
        {
            { 2, 1, 20 },
            { 20, 1, 12 },
            { 28, 20, 12 },
            { 4, 34, 33 }
        },
        {
            { 28, 20, 10 },
            { 10, 20, 33 },
            { 33, 20, 34 },
            { 33, 1, 10 }
        },
        {
            { 2, 10, 28 },
            { 2, 28, 20 },
            { 34, 33, 4 }
        },
        {
            { 28, 33, 12 },
            { 33, 28, 2 },
            { 2, 34, 33 },
            { 2, 28, 17 }
        },
        {
            { 12, 28, 17 },
            { 12, 17, 1 },
            { 4, 34, 33 }
        },
        {
            { 2, 34, 33 },
            { 2, 33, 1 },
            { 10, 28, 17 }
        },
        {
            { 33, 4, 34 },
            { 10, 28, 17 }
        },
        {
            { 20, 10, 17 },
            { 20, 12, 10 },
            { 34, 12, 20 },
            { 33, 12, 34 }
        },
        {
            { 12, 10, 1 },
            { 4, 34, 33 },
            { 2, 17, 20 }
        },
        {
            { 20, 34, 17 },
            { 17, 34, 33 },
            { 1, 17, 33 }
        },
        {
            { 4, 34, 33 },
            { 17, 20, 2 }
        },
        {
            { 33, 12, 34 },
            { 34, 12, 10 },
            { 2, 34, 10 }
        },
        {
            { 1, 12, 10 },
            { 34, 33, 4 }
        },
        {
            { 2, 34, 33 },
            { 2, 33, 1 }
        },
        {
            { 4, 34, 33 }
        },
        {
            { 12, 28, 20 },
            { 20, 4, 12 }
        },
        {
            { 2, 1, 20 },
            { 20, 1, 12 },
            { 28, 20, 12 }
        },
        {
            { 1, 10, 4 },
            { 4, 10, 28 },
            { 20, 4, 28 }
        },
        {
            { 2, 10, 28 },
            { 2, 28, 20 }
        },
        {
            { 17, 2, 28 },
            { 28, 2, 4 },
            { 12, 28, 4 }
        },
        {
            { 17, 1, 12 },
            { 17, 12, 28 }
        },
        {
            { 1, 2, 4 },
            { 28, 17, 10 }
        },
        {
            { 17, 10, 28 }
        },
        {
            { 10, 17, 12 },
            { 12, 17, 20 },
            { 4, 12, 20 }
        },
        {
            { 10, 1, 12 },
            { 20, 2, 17 }
        },
        {
            { 20, 4, 1 },
            { 20, 1, 17 }
        },
        {
            { 20, 2, 17 }
        },
        {
            { 10, 2, 4 },
            { 10, 4, 12 }
        },
        {
            { 12, 10, 1 }
        },
        {
            { 1, 2, 4 }
        },
        {  }
    };
}
