/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.visualization;

import javax.vecmath.Point3f;
import javax.vecmath.Tuple3f;
import javax.vecmath.Vector3f;

import org.srs3d.viewer.vecmath.PointArray3f;

/**
 * Class description.
 *
 * @author Karsten Klein, LION bioscience AG
 */
public class DistanceProperty implements ScalarProperty {
    private PointArray3f coordinates = new PointArray3f(4);

    /**
     * Constructor description.
     */
    public DistanceProperty() {
        coordinates.setAt(0, new Point3f(-4.5f, 0, 0));
        coordinates.setAt(1, new Point3f(+5, 0, 0));
        coordinates.setAt(2, new Point3f(+5, 3, 0));
        coordinates.setAt(3, new Point3f(+5.5f, 3, -5));
    }

    /**
     * Method description.
     *
     * @param vector Parameter description.
     *
     * @return Return description.
     */
    public float getValue(Tuple3f vector) {
        Vector3f v = new Vector3f();
        float min = Float.MAX_VALUE;
        float value = 0;
        float l = 0;
        for (int i = 0; i < coordinates.getSize(); i++) {
            v.set(coordinates.getAt(i));
            v.sub(vector);
            l = v.length();
            if (l < min) {
                min = l;
            }
        }
        return min;
    }

    /**
     * Method description.
     *
     * @return Return description.
     */
    public PointArray3f getCoordinates() {
        return coordinates;
    }

    /**
     * Method description.
     *
     * @param coordinates Parameter description.
     */
    public void setCoordinates(PointArray3f coordinates) {
        this.coordinates = coordinates;
    }
}
