/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.visualization;

import java.util.HashMap;
import java.util.Map;

/**
 * Description of the class.
 *
 * @author Karsten Klein
 *
 * @created August 28, 2001
 */
public final class PointManager {

    // this is a hashmap by intention
    // class should not be used by multiple threads
    private Map pointMap = new HashMap();
    private Map mapI = null;
    private Map mapJ = null;
    private Map mapK = null;
    private int i = -1;
    private int j = -1;
    private int k = -1;
    private int capacity = 0;

    /**
     * Sets the <code>capacity</code> attribute of the <code>PointManager</code> object.
     * The capacity is used for specifying th initial capacity of the hashmaps nad
     * -sets.
     *
     * @param capacity The new <code>capacity</code> value.
     */
    public void setCapacity(int capacity) {
        this.capacity = capacity;
    }

    /**
     * Gets the <code>point</code> attribute of the <code>PointManager</code> object.
     *
     * @param i Description of parameter.
     * @param j Description of parameter.
     * @param k Description of parameter.
     * @param edge Description of parameter.
     *
     * @return The <code>point</code> value.
     */
    public final Point getPoint(int i, int j, int k, int edge) {
        Integer key;

        // check mapI cached
        if (mapI == null || i != this.i) {

            // look it up
            key = new Integer(i);
            this.i = i;
            mapI = (Map) pointMap.get(key);
            mapJ = null;
            mapK = null;
        }
        if (mapI != null) {

            // check mapJ cached
            if (mapJ == null || j != this.j) {

                // look it up
                key = new Integer(j);
                this.j = j;
                mapJ = (Map) mapI.get(key);
                mapK = null;
            }
            if (mapJ != null) {

                // check mapK cached
                if (mapK == null || k != this.k) {
                    key = new Integer(k);
                    this.k = k;
                    mapK = (Map) mapJ.get(key);
                }
                if (mapK != null) {
                    return (Point) mapK.get(new Integer(edge));
                }
            }
        }
        return null;
    }

    // i, j, k identifies the cube, edge [0, 1, 2] identifies the edge

    /**
     * Adds a <code>Point</code> object to the <code>PointManager</code> object.
     *
     * @param i The <code>Point</code> object to be added.
     * @param j The <code>Point</code> object to be added.
     * @param k The <code>Point</code> object to be added.
     * @param edge The <code>Point</code> object to be added.
     * @param point The <code>Point</code> object to be added.
     */
    public final void addPoint(int i, int j, int k, int edge, Point point) {
        Integer key;

        // check mapI cached
        if (mapI == null || i != this.i) {

            // look it up
            key = new Integer(i);
            this.i = i;
            mapI = (Map) pointMap.get(key);

            // if no mapI available generate one
            if (mapI == null) {
                mapI = new HashMap(capacity);
                pointMap.put(key, mapI);
            }
            mapJ = null;
            mapK = null;
        }

        // check mapJ cached
        if (mapJ == null || j != this.j) {

            // look it up
            key = new Integer(j);
            this.j = j;
            mapJ = (Map) mapI.get(key);

            // if no mapJ available generate one
            if (mapJ == null) {
                mapJ = new HashMap(capacity);
                mapI.put(key, mapJ);
            }
            mapK = null;
        }

        // :FIXME: there is a bug in the caching, caused by mapK
        if (true || mapK == null || k != this.k) {

            // look it up
            key = new Integer(k);
            this.k = k;
            mapK = (Map) mapJ.get(key);

            // if no mapK available generate one
            if (mapK == null) {
                mapK = new HashMap(capacity);
                mapJ.put(key, mapK);
            }
            key = new Integer(edge);
            mapK.put(key, point);
        }
    }

    /**
     * Description of the method.
     *
     * @param i Description of parameter.
     */
    public final void clear(int i) {
        pointMap.remove(new Integer(i));
    }

    /**
     * Description of the method.
     */
    public final void clear() {
        pointMap.clear();
    }
}
