/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.visualization;

import java.util.HashMap;
import java.util.Map;

/**
 * Description of the class.
 *
 * @author Karsten Klein
 *
 * @created August 28, 2001
 */
public final class SampleManager {

    // this is a hashmap by intention
    // class should not be used by multiple threads
    private Map sampleMap = new HashMap();
    private Map mapI = null;
    private Map mapJ = null;
    private Sample sampleIJK = null;
    private int i = -1;
    private int j = -1;
    private int k = -1;
    private int capacity = 0;

    /**
     * Sets the <code>capacity</code> attribute of the <code>PointManager</code> object.
     * The capacity is used for specifying th initial capacity of the hashmaps nad
     * -sets.
     *
     * @param capacity The new <code>capacity</code> value.
     */
    public void setCapacity(int capacity) {
        this.capacity = capacity;
    }

    /**
     * Gets the <code>sample</code> attribute of the <code>SampleManager</code> object.
     *
     * @param i Description of parameter.
     * @param j Description of parameter.
     * @param k Description of parameter.
     *
     * @return The <code>sample</code> value.
     */
    public final Sample getSample(int i, int j, int k) {
        Integer key;

        // check mapI cached
        if (mapI == null || i != this.i) {

            // look it up
            key = new Integer(i);
            this.i = i;
            mapJ = null;
            sampleIJK = null;
            mapI = (Map) sampleMap.get(key);
        }
        if (mapI != null) {

            // check mapJ cached
            if (mapJ == null || j != this.j) {

                // look it up
                key = new Integer(j);
                this.j = j;
                sampleIJK = null;
                mapJ = (Map) mapI.get(key);
            }
            if (mapJ != null) {

                // check sample cached
                if (sampleIJK == null || k != this.k) {
                    key = new Integer(k);
                    this.k = k;
                    sampleIJK = (Sample) mapJ.get(key);
                }
                return sampleIJK;
            }
        }
        return null;
    }

    /**
     * Adds a <code>Sample</code> object to the <code>SampleManager</code> object.
     *
     * @param i The <code>Sample</code> object to be added.
     * @param j The <code>Sample</code> object to be added.
     * @param k The <code>Sample</code> object to be added.
     * @param sample The <code>Sample</code> object to be added.
     */
    public final void addSample(int i, int j, int k, Sample sample) {
        Integer key;

        // check mapI cached
        if (mapI == null || i != this.i) {

            // look it up
            key = new Integer(i);
            this.i = i;
            mapI = (Map) sampleMap.get(key);

            // if no mapI available generate one
            if (mapI == null) {
                mapI = new HashMap(capacity);
                sampleMap.put(key, mapI);
            }
            mapJ = null;
        }

        // check mapJ cached
        if (mapJ == null || j != this.j) {

            // look it up
            key = new Integer(j);
            this.j = j;
            mapJ = (Map) mapI.get(key);

            // if no mapJ available generate one
            if (mapJ == null) {
                mapJ = new HashMap(capacity);
                mapI.put(key, mapJ);
            }
        }
        key = new Integer(k);
        mapJ.put(key, sample);
        this.k = k;
        sampleIJK = sample;
    }

    /**
     * Description of the method.
     */
    public final void clearCached() {
        mapI = null;
        mapJ = null;
        sampleIJK = null;
    }

    /**
     * Description of the method.
     *
     * @param i Description of parameter.
     */
    public final void clear(int i) {
        sampleMap.remove(new Integer(i));
    }

    /**
     * Description of the method.
     *
     * @param i Description of parameter.
     * @param j Description of parameter.
     * @param k Description of parameter.
     */
    public final void clear(int i, int j, int k) {
        Integer key;

        // check mapI cached
        if (mapI == null || i != this.i) {

            // look it up
            key = new Integer(i);
            this.i = i;
            mapJ = null;
            sampleIJK = null;
            mapI = (Map) sampleMap.get(key);
        }
        if (mapI != null) {

            // check mapJ cached
            if (mapJ == null || j != this.j) {

                // look it up
                key = new Integer(j);
                this.j = j;
                sampleIJK = null;
                mapJ = (Map) mapI.get(key);
            }
            if (mapJ != null) {
                mapJ.remove(new Integer(k));
            }
        }
    }

    /**
     * Description of the method.
     */
    public final void clear() {
        sampleMap.clear();
    }
}
