/** 
 *    Part of SRS 3D, a system for viewing macromolecules (http://srs3d.org)
 *    
 *    Copyright (C) 2007-2008
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package org.srs3d.viewer.visualization;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;

import javax.vecmath.Tuple3f;

/**
 * Interface for filtering subvolumes.
 *
 * @author Karsten Klein
 *
 * @created April 26, 2001
 */
public class VolumeFilterProperty implements FilterProperty {
    private Subspace subspace = null;
    private transient Collection candidates = new HashSet();
    private transient Sphere sphere = new Sphere();

    /**
     * <code>VolumeFilterProperty</code> constructor.
     *
     * @param subspace Description of parameter.
     * @param epsilon Description of parameter.
     */
    public VolumeFilterProperty(Subspace subspace, float epsilon) {
        this.subspace = subspace;
        sphere.setRadius(epsilon);
    }

    /**
     * Filteres a subvolume position.
     *
     * @param position Position.
     *
     * @return Filter result.
     */
    public boolean filter(Tuple3f position) {
        sphere.setCoordinate(position);
        subspace.collectCandidates(sphere, candidates);
        Iterator iterator = candidates.iterator();
        while (iterator.hasNext()) {
            if (sphere.intersects((Volume) iterator.next())) {
                candidates.clear();
                return true;
            }
        }
        candidates.clear();
        return false;
    }
}
